/**
 * GnutellaPDF.java
 *
 * @author	Jeremy Stribling
 * @version	$Id: GnutellaPDF.java,v 1.3 2002/11/27 01:52:40 hweather Exp $
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.apps.gnutella_trace;

import java.util.HashMap;
import ostore.util.PriorityQueue;
import ostore.util.Debug;

/** Specifies a CDF and a PDF (kind of) for a Gnutella trace*/
public class GnutellaPDF implements ostore.tapestry.patchwork.CDF {

    private static final boolean DEBUG = false;
    private static final String class_tag = "GnutellaPDF";

    private PriorityQueue pq;
    
    private HashMap hash_to_int;
    private boolean[] ints_used;
    private int numNodes;
    private String filename;

    public void populate( String filename, int numNodes ) {
	final String tag = class_tag+".populate";
	GnutellaOrder go = new GnutellaOrder( filename, 0, numNodes );
	long startTime = go.atLeastAliveTime();
	go = new GnutellaOrder( filename, startTime+1, numNodes );
	pq = go.getQueue();
	hash_to_int = new HashMap();
	this.filename = filename;
	this.numNodes = numNodes;
	ints_used = new boolean[ numNodes ];
	if( DEBUG ) 
	    Debug.printtagln( tag, "populated it starting from " + startTime );
    }

    public double getCDFEntry( int n ) {

	final String tag = class_tag+".getCDFEntry";
	GnutellaOrder.GnutellaNode node = 
	    (GnutellaOrder.GnutellaNode) pq.topObject( false );
	Integer index = (Integer) hash_to_int.get( node.hashed_ip );
	// if the next event is a start, only assign if
	// the node has already been mapped
	// that way, all nodes are required to be up at the beginning
	if( node.is_start ) {
	    if( index != null && index.intValue() == n ) {
		if( DEBUG ) 
		    Debug.printtagln( tag, "starting index " + n + "/node=" 
				      + node );
		return 1.01;
	    } else {
		return -1;
	    }
	} else {
	    // otherwise, if this hash is unassigned, give it to this guy
	    if( index == null && !ints_used[n]) {
		index = new Integer(n);
		if( DEBUG )
		    Debug.printtagln( tag, "associating index " + n + 
				      " with hash " + node.hashed_ip );
		hash_to_int.put( node.hashed_ip, index );
		ints_used[n] = true;
	    }
	    if( index != null && index.intValue() == n ) {
		if( DEBUG ) 
		    Debug.printtagln( tag, "failing index " + n + "/node=" 
				      + node );
		return 1.01;
	    } else {
		return -1;
	    }
	}
    }

    public int getCDFSize() {
	return numNodes;
    }

    /**
     * ASSUMPTION: assume this is only called once every cycle
     */
    public int getCDFIncrement() {
	final String tag = class_tag+".getIncrement";
	GnutellaOrder.GnutellaNode node = 
	    (GnutellaOrder.GnutellaNode) pq.extract( false );
	// if we've run out of events, it's time to start the trace over
	if( pq.topObject(false) == null ) {
	    GnutellaOrder go = new GnutellaOrder( filename, 0, numNodes );
	    pq = go.getQueue();
	}
	int increment =  (int) 
	    (((GnutellaOrder.GnutellaNode)pq.topObject(false)).event_time -
	     node.event_time);
	if( DEBUG ) 
	    Debug.printtagln( tag, "returning increment " + increment + 
			      " until node " + 
			      ((GnutellaOrder.GnutellaNode)pq.topObject(false)) );
	return increment;
    }

    /**
     * Is the next event a startup?
     */
    public boolean isNextStart() {
	return ((GnutellaOrder.GnutellaNode)pq.topObject(false)).is_start;
    }

}

