//
// $Id: BloomFilter.java,v 1.3 2002/08/02 19:38:09 geels Exp $
//
// Copyright (C) 2001 Sean C. Rhea.  All rights reserved.
//

package ostore.bloom;
import ostore.util.Carp;

class BloomFilter {

    private static final String class_tag = "BloomFilter";

    public BloomFilter (int depth, int size /* in bytes */) {
	final String tag = class_tag + ".BloomFilter";

	if ((size * 8) % depth != 0)
	    Carp.die( tag + ": size (" + size + ") in bits must be a " + 
		"multiple of depth (" + depth + ").");

	depth_ = depth; 
	width_ = (size * 8) / depth;

	if (depth_ <= 0) Carp.die( tag + "depth must be > 0");
	if (width_ <= 0) Carp.die( tag + "width must be > 0");

	bits_ = new byte [size];
    }

    public int width () { return width_; } /* in bits */
    public int depth () { return depth_; }

    public boolean bit (int row, int column) {
	// Do column major indexing, since filters will likely be read for
	// only a few columns, but at all depths.

	int the_bit = depth_ * column + row;
	int the_byte = the_bit / 8;
	int the_offset = the_bit % 8;
	return ((bits_ [the_byte] & SET_MASK [the_offset]) != 0);
    }
	
    public void set_bit (int row, int column) {
	// Do column major indexing, since filters will likely be read for
	// only a few columns, but at all depths.

	int the_bit = depth_ * column + row;
	int the_byte = the_bit / 8;
	int the_offset = the_bit % 8;
	bits_ [the_byte] |= SET_MASK [the_offset];
    }
	
    public void clear_bit (int row, int column) {
	// Do column major indexing, since filters will likely be read for
	// only a few columns, but at all depths.

	int the_bit = depth_ * column + row;
	int the_byte = the_bit / 8;
	int the_offset = the_bit % 8;
	bits_ [the_byte] &= ~SET_MASK [the_offset];
    }

    public void clear_all () {
	for (int i = 0; i < bits_.length; ++i)
	    bits_ [i] = 0;
    }

    public boolean empty () {
	for (int i = 0; i < bits_.length; ++i) {
	    if (bits_ [i] != 0) return false;
	}
	return true;
    }

    public String toString () {
	String result = "";
	for (int row = 0; row < depth (); ++row) {
	    for (int col = 0; col < width (); ++col)
		result += (bit (row, col) ? "1" : "0");
	    result += "\n";
	}
	return result;
    }

    private byte [] bits_;
    private int depth_, width_;

    private static final byte SET_MASK [] = 
    { 0x1, 0x2, 0x4, 0x8, 0x10, 0x20, 0x40, -128 };

    public static void main (String [] args) {
	
	BloomFilter b = new BloomFilter (9, 18);
	b.set_bit (1, 3);
	b.set_bit (8, 15);
	System.out.println (b);
	
    }
}

