/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.bloom;
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.HashMap;
import java.util.Random;
import java.util.Vector;
import java.util.LinkedList;
import java.util.Iterator;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.StreamTokenizer;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.Reader;
import ostore.util.Pair;
import ostore.util.PriorityQueue;
import soss.network.TransitStub;

/**
 * Creates scf's based on a graph, a number of participating nodes, and a
 * SURGE trace.
 * 
 * Example invocation:
 * 
 *    rm /tmp/bar/*; ./run-java ostore.bloom.CreateTest /tmp/bar 1 ../regression/graphs/ts495-1.hier /home/eecs/srhea/oceanstore/papers/bloomfilters/surge/zipf_20000_2000-match_1 100 true 4 
 *
 */
public class CreateSurgeTest {

    public static int get_node (Set available_nodes, Random rand) {
	int choice = rand.nextInt () % available_nodes.size ();
	Iterator i = available_nodes.iterator ();
	Integer result = (Integer) i.next ();
	for (int j = 0; j < choice; ++j)
	    result = (Integer) i.next ();
	i.remove ();
	return result.intValue ();
	
    }
    public static void bfs (Integer src, Map neighbor_sets, Set visited,
	    int hops_left) {


	if (hops_left > 0) {
	    Set neighbors = (Set) neighbor_sets.get (src);
	    for (Iterator i = neighbors.iterator (); i.hasNext (); ) {
		Integer dst = (Integer) i.next ();
		visited.add (dst);
	    }
	    for (Iterator i = neighbors.iterator (); i.hasNext (); ) {
		Integer dst = (Integer) i.next ();
		bfs (dst, neighbor_sets, visited, hops_left - 1);
	    }
	}
    }

    public static double avg_reachability (Map neighbor_sets, int depth) {
	if (depth == 0) return 0.0;

	int total = 0;
	int count = 0;

	for (Iterator i = neighbor_sets.keySet ().iterator ();
		i.hasNext (); ) {

	    Integer src = (Integer) i.next ();
	    Set neighbors = (Set) neighbor_sets.get (src);
	    Set visited = new HashSet ();
	    visited.add (src);
	    bfs (src, neighbor_sets, visited, depth);
	    total += visited.size () - 1;
	    count += 1;
	}

	return ((double) total) / count;
    }

    public static void main (String [] args) {

	try {

	int argnum = 0;
	String /* 1 */ outputdir           = args [argnum++];
	long   /*  2 */ seed                = Long.parseLong (args [argnum++]);
	String /*  3 */ graph               = args [argnum++];
	String /*  4 */ surgedir            = args [argnum++];
	int    /*  5 */ participating_nodes = Integer.parseInt (args [argnum++]);
	boolean/*  6 */ use_bloom           = args [argnum++].equals ("true");
	int    /*  7 */ bloom_degree        = Integer.parseInt (args [argnum++]);
	int    /*  8 */ bloom_depth         = Integer.parseInt (args [argnum++]);
	int    /*  9 */ bloom_size         = Integer.parseInt (args [argnum++]);
	int    /* 10 */ bloom_limit        = Integer.parseInt (args [argnum++]);
	int    /* 11 */ cross_thresh       = Integer.parseInt (args [argnum++]);

	boolean interdomain_aware = (cross_thresh != -1);

	if (bloom_depth == 0) {
	    use_bloom = false;
	    bloom_degree = 0;
	}

	TransitStub ts = new TransitStub (graph);
	int total_nodes = ts.vertices ();

	// Build the overlay network

	Random overlay_rand = new Random (seed);

	System.out.println ("Creating test.scf.");

	PrintStream scf = null;
	{
	    FileOutputStream fos = 
		new FileOutputStream (outputdir + "/test.scf");
	    BufferedOutputStream bos = new BufferedOutputStream (fos);
	    scf = new PrintStream (bos);
	}

	scf.println ("Seed " + seed);
	scf.println ("Graph " + graph);
	scf.println ("DesiredSuccesses " + participating_nodes);

	Set available_nodes = new HashSet ();
	for (int i = 0; i < total_nodes; ++i) {
	    available_nodes.add (new Integer (i));
	}

	int fedport = total_nodes - 1;
	available_nodes.remove (new Integer (fedport));
	
	scf.println ("Algorithm ostore.tapestry.impl.FederationBuilder");
	scf.println ("Arguments 0");
	scf.println ("ParticipatingNodes 1");
	scf.println ("Node 2 192.168.0.1:" + fedport + " " + 
		participating_nodes);

	scf.println ("Algorithm ostore.bloom.DualBuilder");
	scf.println ("Arguments 11 192.168.0.1:" + fedport + 
		" " + bloom_size + " " + bloom_depth + " 5 true false " + 
		use_bloom + " dynamic " + bloom_limit + " true " + seed);
	scf.println ("ParticipatingNodes " + participating_nodes);

	// Create the neighbor graph.
	Map neighbor_sets = new HashMap ();
	for (int i = 0; i < participating_nodes; ++i) {
	    neighbor_sets.put (
		    new Integer (get_node (available_nodes, overlay_rand)), 
		    new HashSet ());
	}

	if (! interdomain_aware) {
	    PriorityQueue pq = new PriorityQueue (
		    neighbor_sets.size () * neighbor_sets.size ());

	    for (Iterator i = neighbor_sets.keySet ().iterator ();
		    i.hasNext (); ) {
		Integer src = (Integer) i.next ();

		for (Iterator j = neighbor_sets.keySet ().iterator ();
			j.hasNext (); ) {
		    Integer dst = (Integer) j.next ();

		    if (src.intValue () > dst.intValue ()) {
			soss.network.TransitStub.PathInfo p = ts.shortest_path (
				src.intValue (), dst.intValue (), 0, false, 0);
			double route_time = p.route_time;
			long rt_us = Math.round (route_time * 1000000);
			pq.insert (new Pair (p, new Pair (src, dst)), 
				rt_us, false);
		    }
		}
	    }

	    while (! pq.empty ()) {

		Pair q = (Pair) pq.extract (false);
		Pair p = (Pair) q.second;
		soss.network.TransitStub.PathInfo pi = 
		    (soss.network.TransitStub.PathInfo) q.first;

		Integer src = (Integer) p.first;
		Set sneighbors = (Set) neighbor_sets.get (src);

		if (sneighbors.size () < bloom_degree) {
		    Integer dst = (Integer) p.second;
		    Set dneighbors = (Set) neighbor_sets.get (dst);
		    if (dneighbors.size () < bloom_degree) {
			System.out.println ("Adding neighbors " + src 
				+ " " + dst);
			sneighbors.add (dst);
			dneighbors.add (src);
		    }
		}
	    }
	}
	else {

	    PriorityQueue inter_pq = new PriorityQueue (
		    neighbor_sets.size () * neighbor_sets.size ());
	    PriorityQueue intra_pq = new PriorityQueue (
		    neighbor_sets.size () * neighbor_sets.size ());

	    for (Iterator i = neighbor_sets.keySet ().iterator ();
		    i.hasNext (); ) {
		Integer src = (Integer) i.next ();

		for (Iterator j = neighbor_sets.keySet ().iterator ();
			j.hasNext (); ) {
		    Integer dst = (Integer) j.next ();

		    if (src.intValue () > dst.intValue ()) {
			soss.network.TransitStub.PathInfo p = ts.shortest_path (
				src.intValue (), dst.intValue (), 0, false, 0);
			double route_time = p.route_time;
			long rt_us = Math.round (route_time * 1000000);
			if (p.interdomain_edges == null)
			    intra_pq.insert (new Pair (p, new Pair (src, dst)), 
				    rt_us, false);
			else if (cross_thresh > 0)
			    inter_pq.insert (new Pair (p, new Pair (src, dst)), 
				    rt_us, false);
		    }
		}
	    }

	    while (! intra_pq.empty ()) {

		Pair q = (Pair) intra_pq.extract (false);
		Pair p = (Pair) q.second;
		soss.network.TransitStub.PathInfo pi = 
		    (soss.network.TransitStub.PathInfo) q.first;

		Integer src = (Integer) p.first;
		Set sneighbors = (Set) neighbor_sets.get (src);

		if (sneighbors.size () < bloom_degree) {
		    Integer dst = (Integer) p.second;
		    Set dneighbors = (Set) neighbor_sets.get (dst);
		    if (dneighbors.size () < bloom_degree) {
			System.out.println ("Adding neighbors " + src 
				+ " " + dst);
			sneighbors.add (dst);
			dneighbors.add (src);
		    }
		}
	    }
	    Map crossed = new HashMap ();
	    while (! inter_pq.empty ()) {

		Pair q = (Pair) inter_pq.extract (false);
		Pair p = (Pair) q.second;
		soss.network.TransitStub.PathInfo pi = 
		    (soss.network.TransitStub.PathInfo) q.first;

		boolean too_many_crossed = false;
		if (pi.interdomain_edges != null) {
		    for (Iterator i = pi.interdomain_edges.iterator ();
			    i.hasNext (); ) {
			Integer edge = (Integer) i.next ();
			Integer crossings = (Integer) crossed.get (edge);
			if ((crossings != null) &&
				(crossings.intValue () >= cross_thresh)) {
			    too_many_crossed = true;
			    break;
			}
		    }
		}

		if (! too_many_crossed) {
		    Integer src = (Integer) p.first;
		    Set sneighbors = (Set) neighbor_sets.get (src);

		    if (sneighbors.size () < bloom_degree) {
			Integer dst = (Integer) p.second;
			Set dneighbors = (Set) neighbor_sets.get (dst);
			if (dneighbors.size () < bloom_degree) {
			    sneighbors.add (dst);
			    dneighbors.add (src);
			    System.out.println ("Adding neighbors " + src 
				    + " " + dst);
			    if (pi.interdomain_edges != null) {
				for (Iterator i = pi.interdomain_edges.iterator ();
					i.hasNext (); ) {
				    Integer edge = (Integer) i.next ();
				    Integer crossings = (Integer) crossed.get (edge);
				    System.out.println ("IDE " + edge + 
					    " being crossed to connect " + src +
					    " and " + dst);
				    if (crossings == null)
					crossed.put (edge, new Integer (1));
				    else
					crossed.put (edge, new Integer (
						    crossings.intValue () + 1));
				}
			    }
			}
		    }
		}
	    }

	    int total_crossings = 0;
	    for (Iterator i = crossed.keySet ().iterator (); i.hasNext (); ) {
		Integer edge = (Integer) i.next ();
		Integer crossings = (Integer) crossed.get (edge);
		total_crossings += crossings.intValue ();
	    }
	    System.out.println ("Avg Interdomain Crossings/Edge = " +
		    (((double) total_crossings) / crossed.size ()));

	}

	{
	    double total = 0;
	    for (int i = 1; i <= bloom_depth; ++i) {
		double reach = avg_reachability (neighbor_sets, i);
		System.out.println ("Avg reachability at depth " + i + 
			" = " + (reach - total));
		total = reach;
	    }
	    System.out.println ("Total avg reachability = " + total);
	}

	// Free up some memory.
	ts = null;
	System.gc ();

	// Print the node lines for each node.

	for (Iterator i = neighbor_sets.keySet ().iterator ();
		i.hasNext (); ) {
	    
	    Integer src = (Integer) i.next ();
	    //scf.println ("printing " + src);
	    Set neighbors = (Set) neighbor_sets.get (src);
	    scf.print ("Node " + (3 + neighbors.size ()) + " 192.168.0.1:" +
		    src.intValue () + " " + outputdir + "/file-list." +
		    src.intValue () + " " + outputdir);
	    for (Iterator j = neighbors.iterator (); j.hasNext (); ) {
		Integer dst = (Integer) j.next ();
		scf.print (" 192.168.0.1:" + dst.intValue ());
	    }
	    scf.println ("");
	}

	scf.close ();

	// Create the file-lists and reads-files.
	// First, get the file sizes.

	System.out.println ("Reading file sizes.");

	int total_file_size = 0;
	Vector sizes = new Vector ();
	{
	    int lineno = 1;
	    FileInputStream is = new FileInputStream (surgedir + "/yout.txt");
	    Reader reader = new BufferedReader (new InputStreamReader (is));
	    StreamTokenizer tok = new StreamTokenizer (reader);
	    tok.parseNumbers ();

	    tok.nextToken ();
	    while (tok.ttype == StreamTokenizer.TT_EOL) {
		tok.nextToken ();
		lineno++;
	    }

	    while (tok.ttype != StreamTokenizer.TT_EOF) {

		if (tok.ttype != StreamTokenizer.TT_NUMBER) {
		    System.err.println ("Expected file type on line " + 
			    lineno + ": " + tok);
		    System.exit (1); 
		}

		tok.nextToken ();
		if (tok.ttype != StreamTokenizer.TT_NUMBER) {
		    System.err.println ("Expected size on line " + 
			    lineno + ".");
		    System.exit (1); 
		}
		int size = (int) tok.nval;
		total_file_size += size;
		sizes.addElement (new Integer (size));

		tok.nextToken ();
		while (tok.ttype == StreamTokenizer.TT_EOL) {
		    tok.nextToken ();
		    lineno++;
		}
	    }

	    is.close ();
	}
	System.out.println ("Total files = " + sizes.size ());
	System.out.println ("Avg file size = " + ((double) total_file_size
	/ sizes.size ()));

	// Now get the list of files in each object.

	System.out.println ("Reading objects.");

	Vector /* of LinkedList */ objects = new Vector ();
	{
	    int lineno = 1;
	    FileInputStream is = new FileInputStream (surgedir + "/objout.txt");
	    Reader reader = new BufferedReader (new InputStreamReader (is));
	    StreamTokenizer tok = new StreamTokenizer (reader);
	    tok.parseNumbers ();
	    tok.eolIsSignificant (true);

	    tok.nextToken ();
	    while (tok.ttype != StreamTokenizer.TT_EOF) {

		LinkedList files = new LinkedList ();
		while ((tok.ttype != StreamTokenizer.TT_EOL) &&
			    (tok.ttype != StreamTokenizer.TT_EOF)) {
		    if (tok.ttype != StreamTokenizer.TT_NUMBER) {
			System.err.println ("Expected file on line " + 
				lineno + ", instead of " + tok);
			System.exit (1); 
		    }
		    int file = (int) tok.nval;
		    files.addLast (new Integer (file));
		    tok.nextToken ();
		}
		objects.addElement (files);
		tok.nextToken ();
	    }

	    is.close ();
	}

	// Find the total number of requests.

	lru_lineno = 1;
	{
	    FileInputStream is = new FileInputStream (surgedir + "/name.txt");
	    Reader reader = new BufferedReader (new InputStreamReader (is));
	    lru_tok = new StreamTokenizer (reader);
	}
	lru_tok.parseNumbers ();

	int reqcnt = 0;
	int next_rd = next_read ();
	while (next_rd != -1) {
	    LinkedList files = (LinkedList) objects.elementAt (next_rd);
	    reqcnt += files.size ();
	    next_rd = next_read ();
	}

	System.out.println (reqcnt + " total file requests.");

	// Finally, assign each of the objects to one machine's disk.

	Random rand = new Random (seed); // make independent of graph construction
	System.out.println ("Assigning objects to nodes.");

	Map /* Integer -> LinkedList of Integer */ disks = new HashMap ();
	for (Iterator j = neighbor_sets.keySet ().iterator ();
		j.hasNext (); ) {
	    disks.put (j.next (), new LinkedList ());
	}

	for (int i = 0; i < objects.size (); ++i) {
	    int client_num = rand.nextInt () % participating_nodes;
	    Iterator j = neighbor_sets.keySet ().iterator ();
	    Integer client = (Integer) j.next ();
	    while (client_num > 0) {
		client = (Integer) j.next ();
		--client_num;
	    }
	    LinkedList files = (LinkedList) disks.get (client);
	    j = ((LinkedList) objects.elementAt (i)).iterator ();
	    while (j.hasNext ())
		files.addLast (j.next ());
	}

	// Print them out.

	for (Iterator i = disks.keySet ().iterator (); i.hasNext (); ) {
	    Integer client = (Integer) i.next ();
	    FileOutputStream fos = new FileOutputStream (outputdir + 
		    "/file-list." + client.intValue ());
	    BufferedOutputStream bos = new BufferedOutputStream (fos);
	    PrintStream os = new PrintStream (bos);
	    LinkedList files = (LinkedList) disks.get (client);
	    while (! files.isEmpty ()) {
		Integer file = (Integer) files.removeFirst ();
		Integer size = (Integer) sizes.elementAt (file.intValue ());
		os.println ("file-" + file + "\t" + size);
	    }
	    os.close ();
	}

	// Copy the necessary SURGE files to the test dir.

	Runtime.getRuntime ().exec (
		"cp " + surgedir + "/objout.txt " + outputdir);
	Runtime.getRuntime ().exec (
		"cp " + surgedir + "/name.txt " + outputdir);
	Runtime.getRuntime ().exec (
		"cp " + surgedir + "/off.txt " + outputdir);

	}
	catch (Exception e) {
	    System.err.println ("Exception caught:");
	    e.printStackTrace ();
	    System.exit (1);
	}
    }
	    
    static StreamTokenizer lru_tok = null;
    static StreamTokenizer off_tok = null;
    static int lru_lineno = 0;
    static int off_lineno = 0;

    static int next_read () throws Exception {
	lru_tok.nextToken ();
	while (lru_tok.ttype == StreamTokenizer.TT_EOL) {
	    lru_tok.nextToken ();
	    lru_lineno++;
	}

	if (lru_tok.ttype == StreamTokenizer.TT_EOF)
	    return -1;

	if (lru_tok.ttype != StreamTokenizer.TT_NUMBER) {
	    System.err.println ("Expected object on line " + 
		    lru_lineno + ".");
	    System.exit (1); 
	}
	int obj = (int) lru_tok.nval;
	return obj;
    }
}

