/**
 * STSVerify.
 *
 * @author     Gifford Cheung 
 * @version    $Id: STSVerify.java,v 1.13 2002/07/20 19:18:14 srhea Exp $
 */

package ostore.mac;
import ostore.util.*;
import ostore.security.*;

/**
 * Specifically made for the Station-to-Station Protocol to send
 * public keys back and forth. Tailored for and used specifically 
 * by {@link HMACDHHandler}. <p></p>
 *
 * refer to the 1992 Codes and Cryptography paper by Diffie and Hellman on
 * STS. (note we've been using the suggestion at the end of the paper:
 * using a MAC instead of encryption as part of the algorithm)
 *
 * @author   Gifford Cheung 
 * @version    $Id: STSVerify.java,v 1.13 2002/07/20 19:18:14 srhea Exp $
 */
public class STSVerify implements QuickSerializable {


    private QSDHPublicKey myPu;
    private QSDHPublicKey otherPu;
    private int type; //MAC Type (not type_code)

    private byte[] digest; //for STS
    
    /**
     * Constructor.
     */
    public STSVerify ( QSDHPublicKey myPu, 
                      QSDHPublicKey otherPu ) {

        this.myPu = myPu;
        this.otherPu = otherPu;
        this.type = type;

        // create STS digest
        digest = encode();
    }

    public byte[] getEncoded() { return digest; }

    /**
     * Note, since STSVerify is only used to transport the <tt> digest
     * </tt>, all this will decode is <tt>digest</tt> and nothing more
     * <br></br>
     * The deserialization constructor.  See the comments at the top of
     * {@link ostore.util.QuickSerializable} for details.
     */
    public STSVerify (InputBuffer buffer) throws QSException {
        int dlen = buffer.nextInt ();
        digest = new byte[dlen];    
	buffer.nextBytes (digest, 0, dlen);
    }

    public void to_bytes_helper (OutputBuffer buf) {
        myPu.serialize (buf);
        otherPu.serialize (buf);
	buf.add (type);
    }

    /**
     * @return a byte array containing the to_bytes of <tt>this</tt>
     */
    private byte [] encode() {
	CountBuffer cb = new CountBuffer ();
        to_bytes_helper (cb);
        byte [] data = new byte [cb.size ()];
	ByteArrayOutputBuffer outb = new ByteArrayOutputBuffer (data);
        to_bytes_helper (outb);
        return data;
    }


    public void serialize (OutputBuffer buffer ) {
        buffer.add(digest.length);
        buffer.add(digest, 0, digest.length);
    }


}
