/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.monitor;

import seda.sandStorm.api.*;
import ostore.util.*;

/**
 * Response to <tt>RegisterResourceReq</tt>. The resource object will be
 * <tt>==</tt> to the object sent in the request.
 *
 * @author  Jeff Pang
 * @version $Id: RegisterResourceResp.java,v 1.2 2003/02/21 20:37:16 jeffpang Exp $
 **/
public class RegisterResourceResp implements QueueElementIF
{
    /** 
     * Resource registered successfully
     */
    public static final int SUCCESS = 1;
    /**
     * Failed to register resource; there was already a resource registered
     * with the same name.
     */
    public static final int ALREADY_REGISTERED = 2;

    // resource registered
    private Resource rsrc;
    // one of the static final ints above
    private int code;

    /**
     * Response to successful registration
     */
    public RegisterResourceResp(RegisterResourceReq req)
    {
	this.rsrc = req.getResource();
	this.code = SUCCESS;
    }

    /**
     * Response to registration that failed because of <tt>code</tt>
     */
    public RegisterResourceResp(RegisterResourceReq req, int code)
    {
	this.rsrc = req.getResource();
	this.code = code;
    }

    /**
     * The original resource object registered. Will be <tt>==</tt> to the
     * object in the req.
     */
    public Resource getResource()
    {
	return rsrc;
    }

    /**
     * Get status code.
     */
    public int getCode()
    {
	return code;
    }

    public boolean success()
    {
	return code == SUCCESS;
    }
}
