/**
 * LatTest.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.network;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StreamTokenizer;
import java.net.InetAddress;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.dispatch.Signal;
import ostore.util.AssertionViolatedException;
import ostore.util.ByteUtils;
import ostore.util.Carp;
import ostore.util.CountBuffer;
import ostore.util.Debug;
import ostore.util.DebugFlags;
import ostore.util.InputBuffer;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.Pair;
import ostore.util.PriorityQueue;
import ostore.util.QSException;
import ostore.util.Queue;
import ostore.util.QuickSerializable;
import ostore.util.QuickSerializable;
import ostore.util.QuickSerializable;
import ostore.util.TypeTable;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.NoSuchStageException;
import seda.sandStorm.api.NoSuchStageException;
import seda.sandStorm.api.ProfilableIF;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SingleThreadedEventHandlerIF;
import seda.sandStorm.api.SinkClosedEvent;
import seda.sandStorm.api.SinkClosedException;
import seda.sandStorm.api.SinkException;
import seda.sandStorm.api.SinkFlushedEvent;
import seda.sandStorm.api.SinkFullException;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.api.StagesInitializedSignal;
import seda.sandStorm.core.BufferElement;
import seda.sandStorm.internal.SinkProxy;
import seda.sandStorm.lib.aSocket.ATcpClientSocket;
import seda.sandStorm.lib.aSocket.ATcpConnectFailedEvent;
import seda.sandStorm.lib.aSocket.ATcpConnection;
import seda.sandStorm.lib.aSocket.ATcpInPacket;
import seda.sandStorm.lib.aSocket.ATcpListenFailedEvent;
import seda.sandStorm.lib.aSocket.ATcpListenSuccessEvent;
import seda.sandStorm.lib.aSocket.ATcpServerSocket;
import seda.sandStorm.lib.aSocket.aSocketInputStream;
import soss.network.TransitStub;

/**
 * TODO
 *
 * @author     Sean C. Rhea
 * @version    $Id: LatTest.java,v 1.3 2002/09/07 21:49:01 geels Exp $
 */
public class LatTest 
implements SingleThreadedEventHandlerIF, EventHandlerIF {

    protected String TAG = "LatTest";
    protected static final boolean DEBUG = false;

    protected SinkIF this_sink;
    protected int msgnum, msgcnt, rcvcnt, msgsz;
    protected int job;
    protected static final int SENDER = 1;
    protected static final int ROUTER = 2;
    protected static final int RECEIVER = 3;
    protected long start_time;
    protected Classifier classifier;
    protected NodeId peer;
    protected NodeId peer2;	// HACK - second peer for man in middle

    public static byte [] dummy;

    public static class Msg extends NetworkMessage {
	public int value, size;
	public Msg (NodeId peer, int v, int sz) { 
	    super (peer, false); value = v; size = sz;
	}
	public Msg (InputBuffer buffer) throws QSException {
	    super (buffer);
	    value = buffer.nextInt ();
	    size = buffer.nextInt ();
	    buffer.nextBytes (dummy, 0, size - 8);
	}
	public void serialize (OutputBuffer buffer) {
	    super.serialize (buffer);
	    buffer.add (value);
	    buffer.add (LatTest.dummy.length + 8);
	    buffer.add (LatTest.dummy);
	}
	public Object clone () throws CloneNotSupportedException {
	    Msg result = (Msg) super.clone ();
	    result.value = value;
	    result.size = size;
	    return result;
	}
    }

    public void init (ConfigDataIF config) throws Exception {

	// Set up some debug stuff.

	TAG = config.getStage().getName();
        if (DEBUG) Debug.turnOn (TAG);

	// Set this_sink.

	this_sink = config.getStage().getSink();

	// Create all of our filters.

	Filter nm_filter = new Filter ();
	if (! nm_filter.requireType (
		    Class.forName ("ostore.network.LatTest$Msg")))
	    BUG (TAG + ": could not require type.");
	if (! nm_filter.requireValue ("inbound", new Boolean (true)))
	    BUG (TAG + ": could not require inbound.");

	// Read in all of the node_ids we represent, find their
	// classifiers, and register our filters.

	NodeId node_id = new NodeId (config.getString ("node_id"));
	peer = new NodeId (config.getString ("peer"));

	classifier = Classifier.getClassifier (node_id);
	Filter init_filter = new Filter ();
	if (! init_filter.requireType (Class.forName (
			"seda.sandStorm.api.StagesInitializedSignal")))
	    BUG (TAG + ": could not require type.");
	classifier.subscribe (init_filter, this_sink);
	classifier.subscribe (nm_filter, this_sink);

	msgnum = 0;
	String type = config.getString( "type" );
	if( type.equals( "sender" ) ) {
	    job = SENDER;
	} else if( type.equals( "router" ) ) {
	    job = ROUTER;
	    peer2 = new NodeId( config.getString( "peer2" ) );
	} else if( type.equals( "receiver" ) ) {
	    job = RECEIVER;
	} else {
	    BUG( TAG + ": bad type: " + type );
	}
	msgcnt = config.getInt ("msgcnt");
	msgsz = config.getInt ("msgsz");
	dummy = new byte [msgsz - 8];

	TypeTable.register_type ("ostore.network.LatTest$Msg");
    }

    public void handleEvent (QueueElementIF item) 
    throws EventHandlerException {

	if (DEBUG) Debug.printtagln (TAG, "event " + item);

	if (item instanceof StagesInitializedSignal) {
	    if (job == SENDER)
		send_new_msg ( peer );
	}
	else if (item instanceof Msg) {

	    NodeId dest = peer;
	    
	    if (job == SENDER) {
		if (((Msg) item).value != msgnum - 1)
		    throw new AssertionViolatedException ("bad seq");
	    }
	    else if( job == RECEIVER ){
		if (((Msg) item).value != msgnum)
		    throw new AssertionViolatedException ("bad seq");
	    }

	    if( job == ROUTER ) {
		if( ((Msg)item).peer.equals( peer ) ) {
		    // Always forward to the opposite peer.
		    dest = peer2;
		} else {
		    // Correct for double-incrementing counter.
		    msgnum--;
		}
	    }

	    send_new_msg (dest);
	}
	else {
	    Carp.die (TAG + ": got unknown event of type " + 
		    item.getClass ().getName () + ": " + item + ".");
	}
    }

    public void handleEvents(QueueElementIF items[]) 
    throws EventHandlerException {
	for(int i=0; i<items.length; i++)
	    handleEvent(items[i]);
    }

    public void destroy() throws Exception {
    }

    protected void send_new_msg ( NodeId dest ) {
		
	if (job == SENDER) {
	    if (msgnum == msgcnt / 2) {
		start_time = System.currentTimeMillis ();
	    }
	    else if (msgnum == msgcnt) {
		long finish_time = System.currentTimeMillis ();
		System.err.println ("Success!");
		System.err.println ("Sent " + (msgnum / 2) + 
			" messages of " + msgsz + " bytes in " +
			((finish_time - start_time) / 1000.0) + " seconds.");
		System.err.println ("Avg. round-trip latency is " +
			(((double) (finish_time - start_time))/(msgnum / 2)) +
			" ms.");
		System.exit (0);
	    }
	}

	Msg m = new Msg (dest, msgnum++, msgsz);

	if (DEBUG) Debug.printtagln (TAG, "Sending " + m + " #" + m.value );

	try {
	    classifier.dispatch (m);
	}
	catch (SinkFullException e) {
	    BUG (TAG + ": SFE");
	}
    }

    protected static final void BUG (String msg) {
	Carp.die (msg + "  This is a bug in ostore.network.Network.  " +
		"Please let Sean know about it ASAP.");
    }
}

