/**
 * QuickStatResp.java
 *
 * @author Jeremy Stribling
 * @version    $Id: QuickStatResp.java,v 1.4 2003/03/24 00:13:14 geels Exp $
 *
 * Copyright (c) 2003 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.network.patchwork;
import ostore.util.NodeId;
import seda.sandStorm.api.QueueElementIF;

/**
 * Sent by Patchwork in response to a <code>QuickStatReq</code>. Contains
 * statistics collected over a short amount of time for a set of nodes.
 *
 * @author Jeremy Stribling
 * @version    $Id: QuickStatResp.java,v 1.4 2003/03/24 00:13:14 geels Exp $
 */
public final class QuickStatResp implements QueueElementIF, PatchMsg {

    public byte type;
    public NodeId[] peers;
    public Object userData;
    /**
     * Set to true if Patchwork got no response from the corresponding
     * node during the recalculation period
     */
    public boolean[] failures;

    /**
     * Stats for the corresponding nodes in <code>peers</code>.  Note that
     * the "overall" and "current" stats should be identical.
     */
    public PatchworkStats[] stats;
   
    public QuickStatResp (NodeId[] peers, byte type, PatchworkStats[] stats,
			  Object userData ) {

	this.peers = peers;
	this.type = type;
	this.stats = stats;
	this.userData = userData;
	failures = new boolean[peers.length];

    }

    public Object clone () throws CloneNotSupportedException {
	QuickStatResp newone = (QuickStatResp) super.clone ();
	newone.type = type;
	newone.peers = peers;
	newone.stats = stats;
	newone.userData = userData;
	return newone;
    }

    public byte getType() {
	return type;
    }

    public String toString () {
	String s =  "(QuickStatResp, type " + type + ", stats: ";

	for( int i = 0; i < stats.length; i++ ) {
	    s += stats[i];
	}

	return s;

    }
}

