/**
 * ThresholdSignedQS.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.security.thresh;
import ostore.security.SignedQS;
import ostore.security.QSSignature;
import ostore.util.QuickSerializable;
import ostore.util.ByteUtils;
import ostore.util.TypeTable;
import ostore.util.QSException;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.SHA1HashBuffer;
import ostore.util.SecureHashBuffer;

/**
 * A SignedQS for threshold signatures.
 *
 * @author  Sean C. Rhea
 * @version $Id: ThresholdSignedQS.java,v 1.12 2002/08/14 01:13:34 srhea Exp $
 */
public class ThresholdSignedQS extends SignedQS {

    /**
     * Deserialize the same way as a SignedQS.
     */
    public ThresholdSignedQS (InputBuffer buffer) throws QSException {
	super (buffer);
    }
    
    /**
     * The default constructor.
     *
     * <p>TODO: this interface practically encourages bugs.
     */
    public ThresholdSignedQS (QuickSerializable user_data, byte [] signature) {

	_user_data = user_data;
	_qs_signature = new ThresholdSignature ();
	_qs_signature_type = TypeTable.type_code (_qs_signature);
	_signature = signature;
    }

    public static SigShare create_sig_share (KeyShare keyshare, 
	    QuickSerializable user_data, long qs_signature_type) {
	return keyshare.sign (bytes_to_sign (user_data, qs_signature_type));
    }

    public static byte [] bytes_to_sign (
	    QuickSerializable user_data, long qs_signature_type) {
	SecureHashBuffer hb = new SHA1HashBuffer ();
	hb.add (user_data);
	//hb.add (qs_signature_type);
	return hb.toHash ().bytes ();
    }
}

