/**
 * TapestryLocationMsg.
 *
 * @author     Sean C. Rhea
 * @version    $Id: TapestryPublishMsg.java,v 1.9 2002/11/27 02:01:14 hweather Exp $
 */
package ostore.tapestry.api;
import ostore.tapestry.channel.MsgQos;
import ostore.tapestry.impl.Router;
import ostore.util.SecureHash;
import ostore.util.QuickSerializable;
import seda.sandStorm.api.QueueElementIF;

/**
 * Event that causes Tapestry to publish a backpointer to the given GUID
 * with the given tag.  
 *
 * @see TapestryLocateMsg
 */
public class TapestryPublishMsg implements QueueElementIF {

    /**
     * The GUID.
     */
    public SecureHash guid;

    /**
     * The tag.
     */
    public TapestryTag tag;

    /**
     * The delivery qos of this msg, added by Ling Huang
     */
    public int qos;

    /**
     * A bitmask indicating which local optimizations to use.
     * Bits indicate the following optimizations, from least significant bit
     *   to most significant bit:
     *   Bit 1) Publish along secondary links, for a few hops
     *   Bit 2) Publish along all links at the current level, for a few hops
     *   Bit 3) Before leaving a LAN, misroute and publish.
     * Note: "a few hops" is defined by Router.LOCAL_OPT_HOPS
     */
    public int local_opts;

    /**
     * Construct according to the given fields.
     */
    public TapestryPublishMsg (SecureHash guid, TapestryTag tag) {

	this.guid = guid;
	this.tag = tag;
	this.qos = MsgQos.UNRELIABLE;
	// default: no local optimizations
	local_opts = 0;
    }

    public TapestryPublishMsg () 
    {
    }

    public void setLocalOptPublishBackup() {
	local_opts = local_opts | Router.LOCAL_OPT_PUBLISH_BACKUP;
    }

    public void setLocalOptPublishAllLinks() {
	local_opts = local_opts | Router.LOCAL_OPT_PUBLISH_ALL_LINKS;
    }

    public void setLocalOptPublishLocalMisroute() {
	local_opts = local_opts | Router.LOCAL_OPT_PUBLISH_LOCAL_MISROUTE;
    }

    /**
     * Returns a human-readable representation of this
     * <code>TapestryPublishMsg</code>. */
    public String toString()
    {
      String str = new String();

      str += "<TapestryPublishMsg";
      str += " guid==" + ostore.tapestry.impl.RoutingTable.printGuid(guid);
      str += " tag==" + tag;
      str += ">";
      
      return str;
    }
}



