/**
 * ChannelRouteMsg class.
 *
 * @author     Ling Huang
 * @version    $Id: ChannelRouteMsg.java,v 1.3 2002/07/20 19:38:03 srhea Exp $
 */

package ostore.tapestry.channel;
import ostore.tapestry.impl.*;
import ostore.network.NetworkMessage;
import ostore.util.AssertionViolatedException;
import ostore.util.ByteUtils;
import ostore.util.CountBuffer;
import ostore.util.QuickSerializable;
import ostore.util.InputBuffer;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.QSArray;
import ostore.util.QSByteArray;
import ostore.util.QSException;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.SHA1Hash;


//Should we implements QuickSerializable? We already done this for RouteMsg.
public class ChannelRouteMsg extends RouteMsg
{
/* 
Member variables that RouteMsg has: 
  Member variables inheriting from RouteMsg:
    public SecureHash src;
    public SecureHash dest;
    public int hop;
    public int TTL;
    public QSByteArray mac;
    public boolean prefix_only;
*/

//Member variables:
//From our point view, ChannelRouteMsg
//has nothing to do with Channel, except that Channel will produce this
//message and deliver it by Tapestry. So another alternative is only give
//the message a channel ID.

//Channel ID: should it be local unique(in src and dest), or global unique (in all 
//the opened channels)?
    protected int channel_id;  //At this time, we make it local unique. 
                               //It define a unique channel together with src and dest.
    //    protected int qos;         //The QOS of this message, should it be reliable or un-.
    
//For fragmentation and reassembly:
//Notices: In some sense, ChannelRouteMsg = packt in TCP, but we don't have a sequence 
//number for every individual ChannelRouteMsg. We only have a unique sequence number
//(msg_id) for a whole message. This message may be fragmentedd into several 
//ChannelRouteMsgs, and those ChannelRouteMsgs have the same msg_id. We don't send ACKs in 
//the granuality of ChannelRouteMsg, but the whole message.
    protected int  sequence;        //Sequence number to identify the message in a channel.
                                  //The first msg in the channel is with index 0.

    protected int  frg_index;     //Which fragment in the message is this? 
                                  //It start from 0. 
    protected int  frg_num;       //how many total fragments are there in this msg

    protected SecureHash msg_id;        

    protected SecureHash ack_id;  //Is this msg piggy back an ack for previous
                                  //msg? If yes indicate which msg this ACK is for.


//ChannelRouteMsg don't have its own payload. It uses _user_data from RouteMsg.

//  Member functions:
//  Construction functions:
    public ChannelRouteMsg()
    {
	super();

	this.channel_id = 0;

	this.sequence = -1;
	this.frg_index = 0;
	this.frg_num = 1;

	this.prefix_only = false; //ChannelRouteMsg never do prefix only routing.

	// Default values for hop and TTL: hop start from 0, and TTL is big enough!
	this.hop = 0;
	this.TTL = 100;
	this.msg_id = null;
	this.ack_id = null;
    }

    public ChannelRouteMsg(RouteMsg msg)
    {

    }
    public ChannelRouteMsg(NodeId peer, SecureHash src, SecureHash dest,int channel_id)
    {
	super(peer,src,dest);
	this.channel_id = channel_id;
	//	this.qos = MsgQos.UNRELIABLE;

	this.sequence = -1;
	this.frg_index = 0;
	this.frg_num = 1;

	this.prefix_only = false; //ChannelRouteMsg never do prefix only routing.

	// Default values for hop and TTL: hop start from 0, and TTL is big enough!
	this.hop = 0;
	this.TTL = 100;
	this.msg_id = null;
	this.ack_id = null;
    }

    public ChannelRouteMsg(NodeId peer, SecureHash src, SecureHash dest,int channel_id,
		      QuickSerializable user_data)
    {
	super(peer,src,dest);
	this.channel_id = channel_id;
	//	this.qos = MsgQos.UNRELIABLE;

	this.sequence = -1;
	this.frg_index = 0;
	this.frg_num = 1;

	this.prefix_only = false; //ChannelRouteMsg never do prefix only routing.

	this.hop = 0;
	this.TTL = 100;
	this.msg_id = null;
	this.ack_id = null;

	_user_data = user_data;
    }

    public ChannelRouteMsg(NodeId peer, SecureHash src, SecureHash dest,int channel_id,
		      int hop, int ttl, QuickSerializable user_data)
    {
	super(peer,src,dest);
	this.channel_id = channel_id;
	//	this.qos = MsgQos.UNRELIABLE;

	this.sequence = -1;
	this.frg_index = 0;
	this.frg_num = 1;

	this.prefix_only = false; //ChannelRouteMsg never do prefix only routing.

	this.hop = hop;
	this.TTL = ttl;
	this.msg_id = null;
	this.ack_id = null;

	_user_data = user_data;
    }

    public ChannelRouteMsg(NodeId peer, SecureHash src, SecureHash dest, int channel_id,
		      int sequence, int frg_index, int frg_num)
    {
	super(peer,src,dest);
	this.channel_id = channel_id;
	//	this.qos = MsgQos.UNRELIABLE;

	this.sequence = sequence;
	this.frg_index = frg_index;
	this.frg_num = frg_num;

	this.prefix_only = false; //ChannelRouteMsg never do prefix only routing.

	// Default values for hop and TTL: hop start from 0, and TTL is big enough!
	this.hop = 0;
	this.TTL = 100;
	this.msg_id = generate_msg_id();
	this.ack_id = null;
    }


    public ChannelRouteMsg(NodeId peer, SecureHash src, SecureHash dest, int channel_id,
		      int sequence, int frg_index, int frg_num, QuickSerializable user_data)
    {
	super(peer,src,dest);
	this.channel_id = channel_id;
	//	this.qos = MsgQos.UNRELIABLE;

	this.sequence = sequence;
	this.frg_index = frg_index;
	this.frg_num = frg_num;

	this.prefix_only = false; //ChannelRouteMsg never do prefix only routing.

	// Default values for hop and TTL: hop start from 0, and TTL is big enough!
	this.hop = 0;
	this.TTL = 100;
	this.msg_id = generate_msg_id();
	this.ack_id = null;

	_user_data = user_data;
    }


    public ChannelRouteMsg(NodeId peer, SecureHash src, SecureHash dest,int channel_id,
		      int sequence, int frg_index, int frg_num, int hop, int ttl)
    {
	super(peer,src,dest);
	this.channel_id = channel_id;
	//	this.qos = MsgQos.UNRELIABLE;

	this.sequence = sequence;
	this.frg_index = frg_index;
	this.frg_num = frg_num;

	this.prefix_only = false; //ChannelRouteMsg never do prefix only routing.

	this.hop = hop;
	this.TTL = ttl;
	this.msg_id = generate_msg_id();
	this.ack_id = null;
    }

    public ChannelRouteMsg(NodeId peer, SecureHash src, SecureHash dest,int channel_id,
		      int sequence, int frg_index, int frg_num, int hop, int ttl, 
		      QuickSerializable user_data)
    {
	super(peer,src,dest);
	this.channel_id = channel_id;
	//	this.qos = MsgQos.UNRELIABLE;

	this.sequence = sequence;
	this.frg_index = frg_index;
	this.frg_num = frg_num;

	this.prefix_only = false; //ChannelRouteMsg never do prefix only routing.

	this.hop = hop;
	this.TTL = ttl;
	this.msg_id = generate_msg_id();
	this.ack_id = null;

	_user_data = user_data;
    }

//  Functions to get member variables:
    public NodeId get_peer()                    { return peer; }
    //    public int get_qos()                        { return qos; }
    public int get_sequence()                   { return sequence; }
    public int get_frg_index()                  { return frg_index; }
    public int get_frg_num()                    { return frg_num; }
    public SecureHash get_msg_id()              { return msg_id; }
    public SecureHash get_ack_id()              { return ack_id; }

//  Functions to set member variables:
    public void set_channel_id(int id)           { channel_id = id; }
    //    public void set_qos(int quality)             { qos = quality; }
    public void set_sequence(int num)            { sequence   = num; }
    public void set_frg_index(int index)         { frg_index = index; }
    public void set_frg_num(int num)             { frg_num   = num; }
    public void set_msg_id(SecureHash id)        { msg_id    = id; }
    public void set_ack_id(SecureHash id)        { ack_id    = id; } 

    public SecureHash generate_msg_id()
    {
	String  id;

	id = src.toString();
	id.concat( dest.toString() );
	id.concat( Integer.toString(channel_id) );
	id.concat( Integer.toString(sequence) );
	id.concat( Integer.toString(frg_index) );
	msg_id = new SHA1Hash(id);

	return msg_id;
    }

//  Deserialize functions:
    public ChannelRouteMsg (InputBuffer buffer) throws QSException 
    {
	super (buffer);
	channel_id = buffer.nextInt();
	//	qos        = buffer.nextInt();
	sequence    = buffer.nextInt();
	frg_index  = buffer.nextInt();
	frg_num    = buffer.nextInt();
	msg_id     = (SecureHash)buffer.nextObject();
	if (buffer.nextBoolean ())
	    ack_id = (SecureHash)buffer.nextObject ();
    }

//  Serialize functions:
    public void serialize (OutputBuffer buffer) 
    {
	super.serialize (buffer);

	buffer.add (channel_id);
	//	buffer.add (qos);
	buffer.add (sequence);
	buffer.add (frg_index);
	buffer.add (frg_num);
	buffer.add (msg_id);
	if (ack_id == null)
	    buffer.add (false);
	else 
	{
	    buffer.add (true);
	    buffer.add (ack_id);
	}
    }

//  Clone functions:
    public Object clone () throws CloneNotSupportedException 
    {
	ChannelRouteMsg result = (ChannelRouteMsg) super.clone ();
	
	result.channel_id = channel_id;
	//	result.qos = qos;
	result.sequence   = sequence;
	result.frg_index = frg_index;
	result.frg_num   = frg_num;
	result.msg_id    = msg_id;
	result.ack_id    = ack_id;

	return result;
    }

//  Functions to get member variables:
//  Functions to set member variables:
//  Serialize functions:
//  Clone functions:
    public String toString () 
    {
	return "(ChannelRouteMsg " + super.toString () + " " + src + "->" + dest + 
	    " hop " + hop + " TTL " + TTL +
	    (mac == null ? " without" : " with") + " mac " + _user_data + ")";
    }
}













