/**
 * DynamicRepairReqMsg.java<BR>
 * Message sent by a node to a node in its routing table to request
 * additional nodes with the same matching prefix.
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicRepairReqMsg.java,v 1.4 2003/04/16 00:39:53 strib Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.NonceMsg;
import ostore.util.SecureHash;
import ostore.util.QSException;
import ostore.util.Carp;
import java.util.Iterator;
import java.util.ArrayList;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QuickSerializable;

public class DynamicRepairReqMsg
  extends NonceMsg implements QuickSerializable {

  public SecureHash src;  //the GUID of whoever sent this message
  public int digit;       //the digit level of routing table to look at
  public int value;       //for response: the value of routing table to look at
  public int level;       //which level of the repair algorithm is this for
                          //1=same routeentry, 2=same level, 3= prev level
  public int number;      //number of desired entries requested

  public DynamicRepairReqMsg (NodeId peer, SecureHash src, 
			      int digit, int value, int level, int num) {
    super (peer);
    this.src = src;
    this.digit = digit;
    this.value = value;
    this.level = level;
    this.number = num;
  }

  public DynamicRepairReqMsg (InputBuffer buffer) throws QSException {
    super(buffer) ;
    src = (SecureHash) buffer.nextObject();
    digit = buffer.nextInt();
    value = buffer.nextInt();
    level = buffer.nextInt();
    number = buffer.nextInt();
  }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add(src);
    buffer.add(digit);
    buffer.add(value);
    buffer.add(level);
    buffer.add(number);
  }
  
  public Object clone () throws CloneNotSupportedException {
    DynamicRepairReqMsg result = (DynamicRepairReqMsg) super.clone();
    result.src = src;
    result.digit = digit;
    result.value = value;
    result.level = level;
    result.number = number;
    return result;
  }

  public String toString() {
    String s = "(DynamicRepairReqMsg: from " + src + 
	"to " + peer + " for digit " +digit+
	" at value "+value+" at level "+level;
    return s;
  }
}

