/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.impl;
import soss.core.BuilderIF;
import ostore.util.Carp;
import ostore.util.NodeId;
import seda.sandStorm.main.SandstormConfig;

/**
 * Stage builder for Tapestry SOSS experiments, implements the SOSS
 * BuilderIF, an interface for building config files.
 *
 * <p>Must have a constructor of the form 
 * <code>public Builder (String [] args)</code>.
 *
 * @author     Ben Y. Zhao
 * @version    $Id: TapestryBuilder.java,v 1.2 2002/07/22 20:54:53 srhea Exp $
 */
public class TapestryBuilder implements BuilderIF {

  public TapestryBuilder (String [] args) throws Exception {
    
    if (args.length != 3) {
      System.err.print ("usage: TapestryBuilder <fed> " +
			"<static/dynamic> <seed>\ngot:");
      for (int i = 0; i < args.length; ++i)
	System.err.print (" " + args [i]);
      System.err.println ("");
      System.exit (1);
    }
    
    _constructor_args = args;
  }

  public SandstormConfig build_node (String [] args) throws Exception {

    if (args.length < 3) {
      Carp.die ("usage: TapestryBuilder.build_node <node id> " +
		"<file list> <reads file> <neighbors>");
    }
    
    String num_neighbors = new Integer (args.length - 3).toString ();
    String node_id = args [0];
    String file_list = (args [1].equals ("NULL") ? null : args [1]);
    String surgedir = (args [2].equals ("NULL") ? null : args [2]);
    String federation = _constructor_args [0];
    boolean dynamic = _constructor_args [1].equals ("dynamic");
    String seed = _constructor_args [2];
    
    String [] default_args = {
      "node_id=" + node_id,
    };
    SandstormConfig result = new SandstormConfig (default_args);
    
    // StaticTClient
    {
      String [] init_args = {
	"node_id=" + node_id,
	"federation=" + federation
      };
      result.addStage ("StaticTClient", 
		       "ostore.tapestry.impl.StaticTClient", init_args);
    }

    // Router
    String [] init_args = {
      "node_id=" + node_id,
      //      "using_bloom=" + use_bloom, 
      "fake_keys=true"
    };
    result.addStage ("Router", 
		     "ostore.tapestry.impl.Router", init_args);
    
    return result;
  }
  
  String [] _constructor_args;
}
