/**
 * VerifyRespMsg.java
 *
 * @author Jeremy Stribling
 * @version $Id: VerifyRespMsg.java,v 1.1 2002/10/23 19:32:41 strib Exp $
 */

package ostore.tapestry.impl;
import java.util.HashMap;
import java.util.Iterator;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;

/**
 * Message used by a new FullTestMember node to respond to the 
 * FullTestDriver with some info to be verified.
 */
public class VerifyRespMsg extends FullTestRespMsg {

  private RoutingTable rt;
  private HashMap revpointers;
  
  public VerifyRespMsg (NodeId peer, boolean success, int requestType,
			  SecureHash requestID, SecureHash src, String output,
			  RoutingTable rt, HashMap revpointers){
    super (peer, success, requestType, requestID, src, output);
    this.rt = rt;
    this.revpointers = revpointers;
  }

  public VerifyRespMsg (InputBuffer buffer) throws QSException {
    super(buffer);
    rt = (RoutingTable) buffer.nextObject();

    revpointers = new HashMap();
    // get number of levels
    int numLevels = buffer.nextInt();
    for( int i = 0; i < numLevels; i++ ) {
	// get number of guids on level
	int numGuids = buffer.nextInt();
	// get the number of this level
	int levelNum = buffer.nextInt();
	HashMap guids = new HashMap();
	for( int j = 0; j < numGuids; j++ ) {
	    // get each guid/node_id mapping
	    SecureHash guid = (SecureHash) buffer.nextObject();
	    NodeId nid = (NodeId) buffer.nextObject();
	    guids.put( guid, nid );
	}
	revpointers.put( new Integer(levelNum), guids );
    }
    
  }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add( rt );
    // add number of levels
    buffer.add( revpointers.size() );
    // add revpointers level by level, one at a time
    Iterator levelIter = revpointers.keySet().iterator();
    while( levelIter.hasNext() ) {
	Integer level = (Integer) levelIter.next();
	HashMap guids = (HashMap) revpointers.get( level );
	// first add the number of guids, then the level, then the guids
	buffer.add( guids.size() );
	buffer.add( level.intValue() );
	Iterator guidIter = guids.keySet().iterator();
	while( guidIter.hasNext() ) {
	    SecureHash guid = (SecureHash) guidIter.next();
	    NodeId nid = (NodeId) guids.get( guid );
	    buffer.add( guid );
	    buffer.add( nid );
	}
    }
  }

  public Object clone () throws CloneNotSupportedException {
      //VerifyRespMsg result = (VerifyRespMsg) super.clone();
      //return result;
      throw new CloneNotSupportedException( "VerifyMsg cannot yet clone" );
  }

  public RoutingTable getRoutingTable() {
      return rt;
  }

  public HashMap getRevpointers() {
      return revpointers;
  }

  public String toString() {
      return "VerifyRespMsg: " + super.toString();
  }
}
