/**
 * LatTest.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.impl.test;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.dispatch.Signal;
import ostore.util.AssertionViolatedException;
import ostore.util.Carp;
import ostore.util.Debug;
import ostore.util.DebugFlags;
import ostore.util.InputBuffer;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.Pair;
import ostore.util.PriorityQueue;
import ostore.util.QSException;
import ostore.util.Queue;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.util.SHA1Hash;
import ostore.util.TypeTable;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.NoSuchStageException;
import seda.sandStorm.api.ProfilableIF;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SingleThreadedEventHandlerIF;
import seda.sandStorm.api.SinkFullException;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.api.StagesInitializedSignal;

/**
 * Modified latency test for Tapestry messages from Sean's Latency
 * test for NetworkMessages.
 *
 * @author     Ben Y Zhao
 * @version    $Id: LatTest.java,v 1.3 2002/09/06 01:37:29 hling Exp $
 */
public class LatTest 
implements SingleThreadedEventHandlerIF, EventHandlerIF {

    protected String TAG = "LatTest";
    protected static final boolean DEBUG = false;

    protected SinkIF this_sink;
    protected int msgnum, msgcnt, rcvcnt, msgsz;
    protected boolean start;
    protected long start_time;
    protected Classifier classifier;
    protected NodeId peer;
    protected SecureHash peerguid;

    public static byte [] dummy;

    public static class Msg extends ostore.tapestry.api.TapestryRouteMsg {
	public int value, size;
	public Msg (SecureHash peerguid, int v, int sz) { 
	    super (peerguid); value = v; size = sz;
	}
	public Msg (InputBuffer buffer) throws QSException {
	    super (buffer);
	    value = buffer.nextInt ();
	    size = buffer.nextInt ();
	    buffer.nextBytes (dummy, 0, size - 8);
	}
	public void serialize (OutputBuffer buffer) {
	    super.serialize (buffer);
	    buffer.add (value);
	    buffer.add (LatTest.dummy.length + 8);
	    buffer.add (LatTest.dummy);
	}
	public Object clone () throws CloneNotSupportedException {
	    Msg result = (Msg) super.clone ();
	    result.value = value;
	    result.size = size;
	    return result;
	}
    }

    public void init (ConfigDataIF config) throws Exception {

	// Set up some debug stuff.

	TAG = config.getStage().getName();
        if (DEBUG) Debug.turnOn (TAG);

	// Set this_sink.

	this_sink = config.getStage().getSink();

	// Create all of our filters.

	Filter nm_filter = new Filter ();
	if (! nm_filter.requireType (
		    Class.forName ("ostore.tapestry.impl.test.LatTest$Msg")))
	    BUG (TAG + ": could not require type.");
	if (! nm_filter.requireValue ("inbound", new Boolean (true)))
	    BUG (TAG + ": could not require inbound.");

	// Read in all of the node_ids we represent, find their
	// classifiers, and register our filters.

	NodeId node_id = new NodeId (config.getString ("node_id"));
	peer = new NodeId (config.getString ("peer"));
	peerguid = new SHA1Hash(peer);

	classifier = Classifier.getClassifier (node_id);
	Filter init_filter = new Filter ();
	if (! init_filter.requireType (Class.forName (
			"seda.sandStorm.api.StagesInitializedSignal")))
	    BUG (TAG + ": could not require type.");
	classifier.subscribe (init_filter, this_sink);
	classifier.subscribe (nm_filter, this_sink);

	msgnum = 0;
	start = config.getBoolean ("start");
	msgcnt = config.getInt ("msgcnt");
	msgsz = config.getInt ("msgsz");
	dummy = new byte [msgsz - 8];

	TypeTable.register_type ("ostore.tapestry.impl.test.LatTest$Msg");
	/*
	int         i, j, k;
	String []   node_str1, node_str2, node_str3, node_str4;
	String      str1, str2, str3, str4;
	NodeId      node1, node2, node3, node4;
	SecureHash  [] guid1, guid2, guid3, guid4;

	node_str1 = new String[100];
	node_str2 = new String[100];
	node_str3 = new String[100];
	node_str4 = new String[100];
	guid1 = new SecureHash[100];
	guid2 = new SecureHash[100];
	guid3 = new SecureHash[100];
	guid4 = new SecureHash[100];
	for (i = 0; i < 100; i++)
	{
	    node_str1[i] = "x31:" + Integer.toString(3600+i);
	    node1 = new NodeId (node_str1[i]);
	    guid1[i] = new SHA1Hash (node1);
	    node_str2[i] = "x40:" + Integer.toString(3600+i);
	    node2 = new NodeId (node_str2[i]);
	    guid2[i] = new SHA1Hash (node2);
	    node_str3[i] = "x42:" + Integer.toString(3600+i);
	    node3 = new NodeId (node_str3[i]);
	    guid3[i] = new SHA1Hash (node3);
	    node_str4[i] = "x36:" + Integer.toString(3600+i);
	    node4 = new NodeId (node_str4[i]);
	    guid4[i] = new SHA1Hash (node4);
	    System.err.println (node_str1[i] + " : " + guid1[i].toString() + "    " + 
				node_str2[i] + " : " + guid2[i].toString() + "    " + 
				node_str3[i] + " : " + guid3[i].toString() + "    " + 
				node_str4[i] + " : " + guid4[i].toString());
	}

	for (i = 0 ; i < 100; i ++)
	{
	    str2 = (guid2[i].toString()).substring(10, 12);
	    for (j = 0 ; j < 100; j ++)
	    {
		str3 = (guid3[j].toString()).substring(10, 12);
		for (k = 0 ; k < 100; k ++)
		{
		    str4 = (guid4[k].toString()).substring(10, 12);
		    if (str2.compareTo(str3) == 0 &&
			str3.compareTo(str4) == 0 )
		    {
			System.err.println ("\n"); 
			System.err.println ("Find the guid we want:"); 
			System.err.println (node_str2[i] + " : " + guid2[i].toString() + "    " + 
					    node_str3[j] + " : " + guid3[j].toString() + "    " + 
					    node_str4[k] + " : " + guid4[k].toString());
			return;
		    }
		}
	    }
	}
	*/


    }

    public void handleEvent (QueueElementIF item) 
    throws EventHandlerException {

	if (DEBUG) Debug.printtagln (TAG, "event " + item);

	if (item instanceof StagesInitializedSignal) 
	{
	    if (start)
		send_new_msg ();
	}
	else if (item instanceof Msg) 
	{
	    if (start) 
	    {
		if (((Msg) item).value != msgnum - 1)
		    throw new AssertionViolatedException ("bad seq");
	    }
	    else 
	    {
		if (((Msg) item).value != msgnum)
		    throw new AssertionViolatedException ("bad seq");
	    }

	    send_new_msg ();
	}
	else {
	    Carp.die (TAG + ": got unknown event of type " + 
		    item.getClass ().getName () + ": " + item + ".");
	}
    }

    public void handleEvents(QueueElementIF items[]) 
    throws EventHandlerException {
	for(int i=0; i<items.length; i++)
	    handleEvent(items[i]);
    }

    public void destroy() throws Exception {
    }

    protected void send_new_msg () {
		
	if (start) 
	{
	    if (msgnum == msgcnt / 2) 
	    {
		start_time = System.currentTimeMillis ();
		System.err.println ("Starting  time is: " + start_time);
		start_time = System.currentTimeMillis ();
	    }
	    else if (msgnum == msgcnt) 
	    {
		long finish_time = System.currentTimeMillis ();
		double duration = (finish_time - start_time) / 1000.0;
		double latency = (1000 * duration) / (msgnum / 2);
		System.err.println ("Success!");
		System.err.println ("Starting  time is: " + start_time);
		System.err.println ("Finishing time is: " + finish_time);
		System.err.println ("Sent " + (msgnum / 2) + 
			" messages of " + msgsz + " bytes in " +
			duration + " seconds.");
		System.err.println ("The average latency is: " +
				    latency + " milliseconds.");	
		System.exit (0);
	    }
	}

	Msg m = new Msg (peerguid, msgnum++, msgsz);

	if (DEBUG) Debug.printtagln (TAG, "Sending " + m);
	/*
	if (msgnum >= msgcnt / 2) 
	{
	    long curr_time = System.currentTimeMillis ();
	    System.err.println ("Send " + msgnum + "th msg with size " +
				msgsz + " at time = " + 
				(curr_time-start_time) +" .");
	}
	*/
	try {
	    classifier.dispatch (m);
	}
	catch (SinkFullException e) {
	    BUG (TAG + ": SFE");
	}
    }

    protected static final void BUG (String msg) {
	Carp.die (msg + "  This is a bug in ostore.network.Network.  " +
		"Please let Sean know about it ASAP.");
    }
}

