/**
 * LocalOptBuilder.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.tapestry.impl.test;

import soss.core.BuilderIF;
import ostore.util.Carp;
import seda.sandStorm.main.SandstormConfig;
import seda.sandStorm.api.StageNameAlreadyBoundException;
import java.io.IOException;
import java.util.StringTokenizer;

/**
 * A configuration builder for Patchwork clients
 * Modelled after <code>ostore.bloom.DualBuilder</code>.
 *
 * @author	Jeremy Stribling
 * @version	$Id: LocalOptBuilder.java,v 1.5 2002/11/22 01:36:10 strib Exp $
 */
public class LocalOptBuilder implements BuilderIF
{
  ////////////////////       CONSTANTS	////////////////////
  ////////////////////    STATIC VARIABLES 	////////////////////
  ////////////////////       VARIABLES	////////////////////

  /**
   * The node id of the Federation. */
  private String _federation;

  ////////////////////  STATIC CONSTRUCTORS	////////////////////
  ////////////////////      CONSTRUCTORS	////////////////////

  /**
   * Set global parameters. */
  public LocalOptBuilder(String[] args)
  {
    if (args.length != 1) 
      Carp.die ("usage: PatchworkBuilder <fed>");
	
    _federation = args[0];
  }

  ////////////////////     STATIC METHODS 	////////////////////
  //////////////////// 	METHODS		////////////////////

  /**
   * Create a configuration object for a single virtual machine. */
  public SandstormConfig build_node(String[] args)
      throws StageNameAlreadyBoundException, IOException
  {
      if(args.length != 12)
    {
      String str = new String();
      for(int i = 0; i < args.length; i++)
	  str += args[i] + " ";
      Carp.die ("usage: LocalOptBuilder.build_node <node_id>" +
		" <objtotal> <objstart> <objeach> <objfind> <optimize_on>" +
		" <random_seed> <local_opt_hops> <local_opt_backups>" + 
		" <local_opt_nearest> <local_opt_thresh> <stanford_test>");
    }

    String node_id = args[0];
    String objtotal = args[1];
    String objstart = args[2];
    String objeach = args[3];
    String objfind = args[4];
    String optimize_on = args[5];
    String random_seed = args[6];
    String local_opt_hops = args[7];
    String local_opt_backups = args[8];
    String local_opt_nearest = args[9];
    String local_opt_thresh = args[10];
    String stanford_test = args[11];

    SandstormConfig result = new SandstormConfig();
    
    // Router
    {
      String[] init_args = {
	  "node_id=" + node_id,
	  "dynamic_route=static",
	  "local_opt_hops=" + local_opt_hops,
	  "local_opt_backups=" + local_opt_backups,
	  "local_opt_nearest=" + local_opt_nearest,
	  "local_opt_threshold_factor=" + local_opt_thresh,
	  "fake_keys=true"
        };
      result.addStage("Router", 
		      "ostore.tapestry.impl.Router",
		      init_args);
    }

    // StaticTClient
    {
      String[] init_args = {
	  "node_id=" + node_id,
	  "dynamic_route=static",
	  "federation=" + _federation
        };
      result.addStage("StaticTClient", 
		      "ostore.tapestry.impl.StaticTClient",
		      init_args);
    }

    // LocalOpt
    {
      String[] init_args = {
	  "node_id=" + node_id,
	  "objtotal=" + objtotal,
	  "objstart=" + objstart,
	  "objeach=" + objeach,
	  "objfind=" + objfind,
	  "optimize_on=" + optimize_on,
	  "random_seed=" + random_seed,
	  "stanford_test=" + stanford_test
        };
      result.addStage("LocalOptStage", 
			  "ostore.tapestry.impl.test.LocalOptimizationTest",
			  init_args);
    }

    return result;
  }  
  ////////////////////     INNER CLASSES	////////////////////
}

