/**
 * Interweave.java
 *
 *  @author     Benjamin K. Poon
 *  @version    $Id: Interweave.java,v 1.3 2003/02/19 10:32:31 hweather Exp $
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.interweave;
import ostore.tapestry.impl.*;
import java.util.*;
import java.io.*;
import java.security.*;
import ostore.util.*;
import ostore.dispatch.*;
import ostore.tapestry.*;
import ostore.tapestry.api.*;
import ostore.dataobj.*;
import ostore.update.*;
import ostore.security.*;
import ostore.security.thresh.*;
import ostore.startrek.*;
import seda.sandStorm.api.*;
import seda.sandStorm.core.*;
import seda.sandStorm.internal.SinkProxy;
import cryptix.jce.provider.random.DevRandom;



import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Interweave extends JFrame
    implements EventHandlerIF, ProfilableIF, ActionListener {

    public String class_tag = "Interweave";

    private void dispatch (QueueElementIF item) {
	try {
	    _classifier.dispatch(item);
	} catch (SinkException e) {
	    Carp.out ("Could not dispatch " + item);
	    System.exit (1);
	}
    }

    public void handleEvents(QueueElementIF items[]) 
    throws EventHandlerException {
	for(int i=0; i<items.length; i++)
	    handleEvent(items[i]);
    }

    public void printtagln(String tag, String msg) {
        System.err.print(tag+":"+msg+"\n");
    }

    public void init (ConfigDataIF config) throws Exception {

	DEBUG = true;
	
	Types.registerAllTypes ();

	_classifier = Classifier.getClassifier (
                new NodeId (config.getString ("node_id")));
	

	_self_node_id = new NodeId (config.getString ("node_id"));

	class_tag = class_tag + ".["+_self_node_id.port()+"]";
	if (DEBUG) ostore.util.Debug.turnOn(class_tag);

	final String tag = class_tag +".init";
	
	String stageName = config.getStage().getName();
	try {
	    if(config.getBoolean("Profile"))
		{
		    config.getManager().getProfiler().add("ostore.tapestry.interweave.Interweave", this);
		    SinkIF sink =
			config.getManager().getStage(stageName).getSink();
		    this_sink = new SinkProxy(sink,
					      config.getManager(),
					      config.getStage().getWrapper());
		}
	    else
		this_sink =
		    config.getManager().getStage(stageName).getSink();
	} 
	catch (NoSuchStageException e) {
	    Carp.die (tag + ": could not get own sink.");
	}
	
	Filter filter;
	String [] event_types = {
	    "seda.sandStorm.api.StagesInitializedSignal",
	    "ostore.tapestry.impl.TapestryReadyMsg",
	    "ostore.tapestry.api.TapestryDetachConfirm"
	};	  

	for (int i=0; i< event_types.length; i++) {
	  filter = new Filter ();
	  if (! filter.requireType (Class.forName(event_types[i]))) {
	    Carp.out ("Could not require type " + event_types[i]);
	    System.exit (1);
	  }
	  _classifier.subscribe (filter, this_sink);
	}

	String [] msg_types = {
	    "ostore.tapestry.interweave.InterweaveErrorMsg",
	    "ostore.tapestry.interweave.InterweaveLocateMsg",
	    "ostore.tapestry.interweave.InterweaveMetadataMsg",
	    "ostore.tapestry.interweave.InterweaveFileMsg",
	    "ostore.tapestry.interweave.InterweaveFileRequestMsg",
	    "ostore.tapestry.api.TapestryLocateFailure"
	};

	for (int i=0; i< msg_types.length; i++) {
	  filter = new Filter ();
	  if (! filter.requireType (Class.forName(msg_types[i])))
	    Carp.die ("Could not require type " + msg_types[i]);
	  if (! filter.requireValue("inbound", new Boolean(true)))
	    Carp.die ("Could not require inbound = true for "+
		      msg_types[i]);
	  _classifier.subscribe (filter, this_sink);
	}

	// Global variable initialization
	_resultCount = 0;
	_curSearchResultCount = 0;
	_searchResults = new HashMap();
	_publications = new LinkedList();
	_state = "initializing";

	// Get login from config file
	_login = config.getString("login").trim();
	if (_login == null) {
	    error("Please specify a login in your config file.");
	    System.exit(1);
	}

	// Get shared dir from config file
	_sharedDir = null;
	File newDir = new File((config.getString("shared_dir")).trim());
	if (newDir == null) {
	    System.out.println("No shared directory specified.");
	}
	setDir(newDir, true); // set shared directory

	// Get download dir from config file
	_downloadDir = null;
	newDir = new File((config.getString("download_dir")).trim());
	if (newDir == null && _sharedDir != null) {
	    error("No download directory specified -- Using shared directory.");
	    _downloadDir = _sharedDir;
	} else if (newDir == null && _sharedDir == null) {
	    error("No download directory or shared specified");
	}
	setDir(newDir, false); // set download directory
	
	// GUI
	_JInterweaveFrame = makeJInterweaveFrame();

      	error("[Interweave]: Tapestry stage initialized!!");
	_state = "initialized";
    }

    public void handleEvent(QueueElementIF item) 
	throws EventHandlerException {
	
	final String tag = class_tag+".handleEvent";
	//if (DEBUG) printtagln(tag, "handleEvent: item received: " + item);
	
	if (item instanceof StagesInitializedSignal) {
	    if (DEBUG) printtagln(tag, "StagesInitializedSignal received");
	    
	} else if (item instanceof TapestryReadyMsg) {
	    _state = "connected";
	    
	    _client_guid = ((TapestryReadyMsg)item).node_guid;
	    // dispatch TapestryPublishMsgs for all metadata in SHAREDDIR
	    if (_sharedDir != null) {
		dispatchPublishes(); 
		publishLogin();
	    }

	    output("Ready");

	} else if (item instanceof TapestryDetachConfirm) {
	    _state = "disconnected";
	    handleTapestryDetachConfirm();
	    
	} else if (item instanceof InterweaveLocateMsg) {
	    InterweaveLocateMsg ilm = (InterweaveLocateMsg) item;
	    handleInterweaveLocateMsg(ilm);

	} else if (item instanceof TapestryLocateFailure) {
	    output("Search failed.");
	    
	} else if (item instanceof InterweaveMetadataMsg) {
	    InterweaveMetadataMsg imm = (InterweaveMetadataMsg) item;
	    handleInterweaveMetadataMsg(imm);

	} else if (item instanceof InterweaveFileMsg) {
	    InterweaveFileMsg ifm = (InterweaveFileMsg) item;
	    handleInterweaveFileMsg(ifm);

	} else if (item instanceof InterweaveFileRequestMsg) {
	    InterweaveFileRequestMsg ifrm = (InterweaveFileRequestMsg) item;
	    if (_sharedDir == null) {		
		// File request fails. There is no shared directory specified, so we can't retrieve the file
		// Send an error message to requestor
		InterweaveErrorMsg irm = new InterweaveErrorMsg(_client_guid, ifrm.from, "Could not find file");
		return;
	    }
	    handleInterweaveFileRequestMsg(ifrm);

	} else if (item instanceof InterweaveErrorMsg) {
	    InterweaveErrorMsg irm = (InterweaveErrorMsg) item;
	    output("Remote computer responded with error: " + irm.errorMsg);

	} else {
	    Carp.die ("ostore.tapestry.interweave.Interweave: Received unknown "
		      + "QueueElementIF: " + item);
	}
    }

    public void handleTapestryDetachConfirm() {
	output("Disconnected");
    }

    public void handleInterweaveLocateMsg(InterweaveLocateMsg ilm) {
	LinkedList immList = new LinkedList();
	String mfName = "";
	long mfLastModified = 0;
	long mfSize = 0;
	String mfType = "";
	int mfNumKeywords = 0;
	File[] files = _sharedDir.listFiles();
	boolean send = false;
	
	for (int i = 0; i < files.length; i++) {
	    LinkedList mfKeywords = new LinkedList();

	    send = false;
	    
	    // only look at hidden files (all metafiles are hidden)
	    if (!files[i].isHidden()) continue;
	    
	    // read in the metafile
	    Reader r = null;
	    try {
		r = new InputStreamReader(new FileInputStream(files[i]));
	    } catch (FileNotFoundException e) {
		error("Error: metafile for " + files[i].getName() + " could not be found.");
		continue;
	    }		    
	    StreamTokenizer input =
		new StreamTokenizer(r);
	    
	    try {
		input.nextToken();
		mfName = input.sval;
		input.nextToken();
		mfLastModified = (long)input.nval;
		input.nextToken();
		mfSize = (long)input.nval;
		input.nextToken();
		mfType = input.sval;
		input.nextToken();
		mfNumKeywords = (int)input.nval; // get number of keywords associated with the file
		input.nextToken();
		for (int j = 0; j < mfNumKeywords; j++) { // get the keywords
		    mfKeywords.add(input.sval);
		    input.nextToken();
		}
	    } catch (IOException e) {
		error("Meta file error in " + mfName);
		System.exit(1);
	    }	
	    
	    StringTokenizer typeStrST = new StringTokenizer(ilm.typeStr, " ");
	    StringTokenizer searchStrST = new StringTokenizer(ilm.searchStr, " ");
	    
	    String primType = typeStrST.nextToken();
	    String primSearchStr = searchStrST.nextToken();
	    
	    if (primType.equals("filename")) {
		    if (mfName.equals(primSearchStr))
			send = true;
		    
	    } else if (primType.equals("login")) {
		send = true;
		
	    } else if (primType.equals("keyword")) {
		// compare the search string to all keywords in the metadata file
		for (int k = 0; k < mfNumKeywords; k++) {
		    String keyword = (String)mfKeywords.get(k);
		    if (keyword.equals(primSearchStr)) // if the search string matches a keyword, send the metadata file
			send = true;
		}
		
	    } // if
	    
	    while (typeStrST.hasMoreTokens()) {
		// if the secondary and x-ary field criteria are not filled, do not send
		// never turn send to true here, because send will already be true from the primary
		if (!metafileCriterionFilled(typeStrST.nextToken(), searchStrST.nextToken(),
					     mfName, mfLastModified, mfSize, mfType, mfKeywords)) 
		    send = false;
	    }
	    
	    if (send) {
		InterweaveMetadataMsg imm = new InterweaveMetadataMsg(ilm.senderGUIDHash, 
								      _client_guid,
								      mfName, _login,
								      mfLastModified,
								      mfSize, mfType);
		immList.add(imm);
	    }
	} // for
	
	// Send all IMMs
	while (immList.size() > 0) {
	    InterweaveMetadataMsg imm = (InterweaveMetadataMsg)immList.remove(0);
	    dispatch(imm);
	}
    }

    public void handleInterweaveMetadataMsg(InterweaveMetadataMsg imm) {
	_curSearchResultCount++;
	if (_curSearchResultCount == 1) {
	    output("Search returned " + _curSearchResultCount + " result so far");
	} else {
	    output("Search returned " + _curSearchResultCount + " results so far");
	}

	InterweaveSearchResult isr = new InterweaveSearchResult(imm.from, 
								imm.filename, imm.login,
								imm.lastModified, 
								imm.size, imm.type);
	if (!existsInSearchResults(isr)) {
	    _searchResults.put(new Integer(_resultCount), isr);
	    _resultCount++;

	    // Update the GUI
	    InterweaveSearchResult[] data = new InterweaveSearchResult[_resultCount];
	    for (int i = 0; i < _resultCount; i++) {
		data[i] = (InterweaveSearchResult)_searchResults.get(new Integer(i));
	    }
	    searchResultTableModel.refresh(data,
					   new javax.swing.event.TableModelEvent(searchResultTableModel));
	}
    }

    public void handleInterweaveFileMsg(InterweaveFileMsg ifm) {
	FileOutputStream fos = null;
	FileInputStream fis = null;
	File toFile = new File (_downloadDir + "/" + ifm.filename);
	int existsConfirmation = -1;
	
	// If TOFILE exists, ask user whether or not to replace it
	if (toFile.exists()) {
	    existsConfirmation = JOptionPane.showConfirmDialog(_JInterweaveFrame,
						  "The file " + ifm.filename + " exists. Would you like to overwrite?",
						  "File Exists",
						  JOptionPane.YES_NO_OPTION);
	}
	
	if (existsConfirmation == JOptionPane.NO_OPTION) {
	    return;
	}

	// Open output file
	try {
	    fos = new FileOutputStream(toFile);
	} catch (Exception e) {
	    error("Requested file could not be written");
	}
	
	// Open input file
	try {
	    fis = new FileInputStream(ifm.file);
	} catch (Exception e) { 
	    error("Transmitted file was corrupt");
	}
	
	// Copy the sent file into TOFILE
	try {
	    for (long i = 0; i < ifm.file.length(); i++) {
		fos.write(fis.read()); // write to outfile
	    }
	} catch (Exception e) {
	    error("Requested file " + _downloadDir + "/" + ifm.filename + 
			       " could not be written.");
	}
	output("Received file " + ifm.filename + " into " + _downloadDir);
    }

    public void handleInterweaveFileRequestMsg(InterweaveFileRequestMsg ifrm) {
	FileInputStream fis = null;
	File file = new File (_sharedDir + "/" + ifrm.filename);
	
	InterweaveFileMsg ifm = new InterweaveFileMsg(ifrm.from, ifrm.filename, file);
	dispatch(ifm);
	output("Sent file: " + ifrm.filename + " to " + ifrm.from);
    }
    
    /* Returns true if SEARCHSTR appears in the corresponding TYPE field of the metafile (sent fields) */
    public boolean metafileCriterionFilled(String type, String searchStr, 
					   String mfName, long mfLastModified, long mfSize, 
					   String mfType, LinkedList mfKeywords) {
	if (type.equals("filename")) {
	    return mfName.equals(searchStr);
	} else if (type.equals("keyword")) {
	    boolean returnVal = false;
	    for (int k = 0; k < mfKeywords.size(); k++) {
		if (searchStr.equals((String)mfKeywords.get(k)))
		    returnVal = true;
	    }
	    return returnVal;
	} else if (type.equals("lastmodbefore")) {
	    return mfLastModified < (new Long(searchStr)).longValue();
	} else if (type.equals("lastmodafter")) {
	    return mfLastModified > (new Long(searchStr)).longValue();
	} else if (type.equals("lastmodon")) {
	    return mfLastModified == (new Long(searchStr)).longValue();
	} else if (type.equals("size")) {
	    return mfSize == (new Long(searchStr)).longValue();
	} else if (type.equals("type")) {
	    return mfType.equals(searchStr);
	}
	return false;
    }

    /* Handles input from standard in
    public void handleInput(String s) {
	StringTokenizer st = new StringTokenizer(s, " ");
        if (!st.hasMoreTokens()) return;
        String msgType = st.nextToken();
        msgType = msgType.trim().toLowerCase();

        if (msgType.equals("set-shared-dir") || msgType.equals("ssd")) {
            if (!st.hasMoreTokens()) {
                error:("Format: set-shared-dir [directory_name]");
                return;
            }
	    String newDirName = st.nextToken();
	    File newDir = new File(newDirName);
	    setDir(newDir, true); // set shared directory

	} else if (msgType.equals("display-shared-files") || msgType.equals("dsf")) {
	    displaySharedFiles();

	} else if (msgType.equals("set-download-dir") || msgType.equals("sdd")) {
	    if (!st.hasMoreTokens()) {
		error("Format: set-download-dir [directory_name]");
		return;
	    }
	    String newDirName = st.nextToken();
	    File newDir = new File(newDirName);
	    setDir(newDir, false); // set download directory

	} else if (msgType.equals("display-downloaded-files") || msgType.equals("ddf")) {
	    
	    error("Files in download directory " + _downloadDir + ":");
	    File[] files = _downloadDir.listFiles();
	    for (int i = 0; i < files.length; i++) {
		if (files[i].isHidden()) continue;
                String name = files[i].getName();
		name = name.substring(0, name.lastIndexOf("."));
                error("File " + i + ": " + name +
		" Last Modified: " + files[i].lastModified() +
		" Size: " + files[i].length() + " bytes" +
		" Type: " + name.substring(name.lastIndexOf(".") + 1, name.length()));
	    }
		
        } else if (msgType.equals("search") || msgType.equals("s")) {
	    search(st);
	
        } else if (msgType.equals("display-search-results") || msgType.equals("dsr")) {
	    displaySearchResults();

	} else if (msgType.equals("sort-search-results") || msgType.equals("sort")) {
	    if (!st.hasMoreTokens()) {
		error("Format: sort [by] where by = {filename, login, lastmod, size, type}");
	    }

	    String by = st.nextToken();

	    sortSearchResults(by);

	} else if (msgType.equals("clear-search-results") || msgType.equals("csr")) {
	    for (int i = 0; i < _resultCount; i++) {
		_searchResults.remove(new Integer(i));
	    }
	    _resultCount = 0;

	} else if (msgType.equals("download") || msgType.equals("d")) {
	    if (_downloadDir == null) {
	        error("No download directory set. Please set one before downloading by using set-download-dir");
		return;
	    }
	    if (!st.hasMoreTokens()) {
		error("Format: download [search_result_number]");
		return;
	    }

	    int num = (new Integer(st.nextToken())).intValue();
	    InterweaveSearchResult isr = (InterweaveSearchResult)_searchResults.get(new Integer(num));
	    if (isr == null) {
		error("Invalid search result number");
		return;
	    }

	    InterweaveFileRequestMsg ifrm = new InterweaveFileRequestMsg(isr.peer, _client_guid, 
									 "" + isr.filename + "." + isr.type);
	    dispatch(ifrm);

	} else if (msgType.equals("add-keyword") || msgType.equals("ak")) {
	    if (!st.hasMoreTokens()) {
		error("Format: add-keyword [keyword] [filename]");
	    }
	    String newKeyword = st.nextToken();
	    if (!st.hasMoreTokens()) {
		error("Format: add-keyword [keyword] [filename]");
	    }

	    String filename = st.nextToken();
	    String mfName;
	    long mfLastModified = 0;
	    long mfSize = 0;
	    String mfType; // should be "meta"
	    int mfNumKeywords = 0;
	    String keyword;
	    KeywordTag newKeywordTag;
	    TapestryPublishMsg tpm;
	    boolean found = false;

	    File[] files = _sharedDir.listFiles();
	    for (int i = 0; i < files.length; i++) {
		LinkedList mfKeywords = new LinkedList();

		if (!files[i].isHidden()) continue; // if it's a hidden file go on
		
		mfName = files[i].getName();
		mfType = mfName.substring(mfName.lastIndexOf(".") + 1, mfName.length());
		mfName = mfName.substring(1, mfName.lastIndexOf(".")); // get rid of the leading .
		if (!mfType.equals("meta")) continue; // if it's a meta file go on

		if (!filename.equals(mfName)) continue; // if this is the file the user wants to update go on
		found = true;

		// read in the metafile
		Reader r = null;
		FileInputStream fis = null;

		try {
		    fis = new FileInputStream(files[i]);
		    r = new InputStreamReader(fis);
		} catch (FileNotFoundException e) {
		    error("Metafile for " + mfName + " could not be found");
		    return;
		} catch (Exception e) {
		    error("Could not read metafile");
		    return;
		}
		
		StreamTokenizer input = new StreamTokenizer(r);
		
		try {
		    input.nextToken();
		    mfName = input.sval; // get metafile's name
		    input.nextToken();
		    mfLastModified = (long)input.nval; // get lastModified
		    input.nextToken();
		    mfSize = (long)input.nval; // get size
		    input.nextToken();
		    mfType = input.sval; // get type
		    input.nextToken();
		    mfNumKeywords = (int)input.nval; // get number of keywords associated with the file
		    input.nextToken();
		    for (int j = 0; j < mfNumKeywords; j++) { // get the keywords
			mfKeywords.add(input.sval);
			input.nextToken();
		    }
		} catch (IOException e) {
		    error("Meta file error in " + mfName);
		    System.exit(1);
		}

		try {
		    r.close();
		    fis.close();
		} catch (Exception e) {
		    error("Could not close metafile.");
		    return;
		}

		// recreate the metafile and package it into a printstream to write to
		PrintStream metafileWriter = null;
                try {
                    metafileWriter = new PrintStream(new FileOutputStream(files[i]));
                } catch (FileNotFoundException e) {
		    System.err.println("Metafile " + mfName + "." + mfType + " cannot be written to.");
                }

		// write to the metafile
		mfNumKeywords++;
                metafileWriter.println(mfName + "\n" + 
				       mfLastModified + "\n" + 
				       mfSize + "\n" + 
				       mfType + "\n" +
				       mfNumKeywords);
		for (int j = 0; j < mfNumKeywords - 1; j++) { // write the keywords
		    keyword = (String)mfKeywords.get(j);
		    metafileWriter.println(keyword);
		}		    
		metafileWriter.println(newKeyword);
		
		// publish the new keyword
		byte[] newKeywordBytes = newKeyword.getBytes();
		SHA1Hash newKeywordHash = new SHA1Hash(newKeywordBytes);
		newKeywordTag = new KeywordTag(newKeyword);
		tpm = new TapestryPublishMsg(newKeywordHash, newKeywordTag);
		dispatch(tpm);
		_publications.add(new InterweavePublication(newKeywordHash, newKeywordTag));
            }
	    if (!found) {
		error("No such file.");
		return;
	    }

        } else if (msgType.equals("exit") || msgType.equals("e")) {
	    error("Logging out of the Tapestry network. Please wait.");
	    _state = "disconnecting";
	    dispatch(new TapestryNodeDetachMsg());

        } else if (msgType.equals("help") || msgType.equals("?") || msgType.equals("h")) {
            error("Available commands are: ");
            error("set-shared-dir (ssd) [directory_name]\n" +
	    "display-shared-files (dsf)\n" +
	    "set-download-dir (sdd) [directory_name]\n" +
	    "display-downloaded-files (ddf)\n" +
	    "search (s) [primary field] [primary search string] [field2] [field2 search string] ... [fieldx] [fieldx search string]\n" + 
	    "display-search-results (dsr)\n" +
	    "sort-search-results (sort) [by]\n" +
	    "clear-search-results (csr)\n" + 
	    "download (d) [search_result_number]\n" +
	    "add-keyword (ak) [keyword] [filename]\n" +
	    "help (h, ?)\n" +
	    "exit (e)");

        } else {
            error("Unrecognized command: " + msgType);
        }
    }
    */

    public void search(StringTokenizer st) {
	HashMap fields = new HashMap();
	String typeStr;
	String searchStr;

	_curSearchResultCount = 0;
	
	if (!st.hasMoreTokens()) {
	    error("Format: search [primary field] [primary search string] [field2] [field2 search string] ... where\n" +
		  "primary field = {filename, keyword, login} and\n" +
		  "fieldx = {filename, keyword, lastmodbefore, lastmodafter, lastmodon, size, type}");
	    return;
	}
	String primField = st.nextToken();
	if (!st.hasMoreTokens()) {
	    error("Format: search [primary field] [primary search string] [field2] [field2 search string] ... where\n" +
		  "primary field = {filename, keyword, login} and\n" +
		  "fieldx = {filename, keyword, lastmodbefore, lastmodafter, lastmodon, size, type}");
	    return;
	}
	String primSearchStr = st.nextToken();
	
	while (st.hasMoreTokens()) {
	    String field = st.nextToken();
	    if (!st.hasMoreTokens()) {
		error("Format: search [primary field] [primary search string] [field2] [field2 search string] ... where\n" +
		      "primary field = {filename, keyword, login} and\n" +
		      "fieldx = {filename, keyword, lastmodbefore, lastmodafter, lastmodon, size, type}");
		return;
	    }		    
	    String fieldSearchStr = st.nextToken();
	    fields.put(field, fieldSearchStr);
	}
	
	InterweaveLocateMsg ilm = null;
	
	SHA1Hash h;
	TapestryTag t;
	TapestryQuery q;
	
	if (primField.equals("filename")) {
	    output("Locating files with filename " + primSearchStr);
	    byte[] filenameBytes = primSearchStr.getBytes();
	    h = new SHA1Hash(filenameBytes);
	    t = new FilenameTag(primSearchStr);
	    q = new FilenameQuery();
	    typeStr = "filename ";
	    searchStr = primSearchStr + " ";
	    
	} else if (primField.equals("login")) {
	    output("Locating " + primSearchStr + "'s files");
	    byte[] cornedbeefBytes = primSearchStr.getBytes();
	    h = new SHA1Hash(cornedbeefBytes);
	    t = new LoginTag(primSearchStr);
	    q = new LoginQuery();
	    typeStr = "login ";
	    searchStr = primSearchStr + " ";
	    
	} else if (primField.equals("keyword")) {
	    output("Locating files associated with keyword " + primSearchStr);
	    byte[] keywordBytes = primSearchStr.getBytes();
	    h = new SHA1Hash(keywordBytes);
	    t = new KeywordTag(primSearchStr);
	    q = new KeywordQuery();
	    typeStr = "keyword ";
	    searchStr = primSearchStr + " ";
	    
	} else {
	    error("Format: search [primary field] [primary search string] [field2] [field2 search string] ... where\n" +
		  "primary field = {filename, keyword, login} and\n" +
		  "fieldx = {filename, keyword, lastmodbefore, lastmodafter, lastmodon, size, type}");
	    return;
	}
	
	Iterator fieldSet = fields.keySet().iterator();
	Iterator searchStrSet = fields.values().iterator(); 
	while (fieldSet.hasNext()) {
	    String nextField = (String)fieldSet.next();
	    if (!nextField.equals("filename") &&
		!nextField.equals("keyword") &&
		!nextField.equals("lastmodbefore") &&
		!nextField.equals("lastmodafter") &&
		!nextField.equals("lastmodon") &&
		!nextField.equals("size") &&
		!nextField.equals("type")) {
		error("Format: search [primary field] [primary search string] [field2] [field2 search string] ... where\n" +
		      "primary field = {filename, keyword, login} and\n" +
		      "fieldx = {filename, keyword, lastmodbefore, lastmodafter, lastmodon, size, type}");
		return;
	    }		    
	    typeStr = typeStr + nextField + " ";
	    searchStr = searchStr + (String)searchStrSet.next() + " ";
	}
	
	ilm = new InterweaveLocateMsg(h, _client_guid, q, null, typeStr, searchStr);
	dispatch(ilm);	
    }

    /* Sets shared dir if SHARED is true and download dir if SHARED is false
     */
    public boolean setDir(File newDir, boolean shared) {
	if (!newDir.exists()) {
	    output(newDir.getAbsolutePath() + " does not exist. Please specify another.");
	    return false;
	} else if (!newDir.isDirectory()) {
	    output(newDir.getAbsolutePath() + " is not a directory. Please specify another.");
	    return false;
	}

	if (shared) {
	    File oldDir = _sharedDir;
	    _sharedDir = newDir;

	    prepSharedDir(); // create or update metadata for each file in the directory ARG
	    if (_state == "connected") {
		dispatchPublishes(); // publish all metadata info for files in SHAREDDIR
	    }
	    if (oldDir != null) {
		// TODO: unpublish all files in old directory
		output("Changed shared directory from " + _sharedDir +
				   " to " + newDir);

	    } else {
		output("Set shared directory to " + newDir);
	    }
	} else {
	    _downloadDir = newDir;
	    if (_downloadDir != null) {
		output("Changed download directory from " + _downloadDir +
				   " to " + newDir);
	    } else {
		output("Set download directory to " + newDir);
	    }
	}

	return true;
    }

    public boolean existsInSearchResults(InterweaveSearchResult isr) {
	for (int i = 0; i < _resultCount; i++) {
	    InterweaveSearchResult isr_i = (InterweaveSearchResult)_searchResults.get(new Integer(i));
	    if (isr_i.equals(isr)) return true;
	}
	return false;
    }

    /*
    public void displaySearchResults() {
	System.out.println("Search Results:\n");
	System.out.println("#\tFilename\tLogin\t\tLast Modified\t\t\tSize (bytes)\t\tType");
	for (int i = 0; i < _resultCount; i++) {
	    InterweaveSearchResult isr = (InterweaveSearchResult)_searchResults.get(new Integer(i));
	    System.out.println("[" + i + "]\t" + 
			       isr.filename + "\t\t" + 
			       isr.login + "\t\t" + 
			       new Date(isr.lastModified) + "\t\t" + 
			       isr.size + "\t\t" + 
			       isr.type);
	}
    }

    public void sortSearchResults(String by) {
	if (by.equals("filename")) {
	} else if (by.equals("login")) {
	} else if (by.equals("lastmod")) {
	} else if (by.equals("size")) {
	} else if (by.equals("type")) {
	} else {
	    System.out.println("Format: sort [by] where by = {filename, login, lastmod, size, type}");
	}
	displaySearchResults();
    }

    public void displaySharedFiles() {
	String name;
	long lastModified = 0;
	long size = 0;
	String type;
	int numKeywords = 0;
	String keyword;

	if (_sharedDir == null) {
	    System.out.println("No shared directory set. Use set-shared-dir to set one.");
	    return;
	}
	
	System.out.println("Files in shared directory " + _sharedDir + ":");
	File[] files = _sharedDir.listFiles();
	for (int i = 0; i < files.length; i++) {
	    LinkedList keywords = new LinkedList();

            if (!files[i].isHidden()) continue; // if it's not hidden, skip it

            name = files[i].getName();
            type = name.substring(name.lastIndexOf(".") + 1, name.length());
	    name = name.substring(0, name.lastIndexOf("."));
            if (!type.equals("meta")) continue; // if it's not a metafile, skip it

            Reader r = null;

            try {
                r = new InputStreamReader(new FileInputStream(files[i]));
            } catch (FileNotFoundException e) {
                System.out.println("Error: metafile for " + name + " could not be found.");
		return;
            }

            StreamTokenizer input = new StreamTokenizer(r);

	    try {
		input.nextToken();
		name = input.sval; // get filename
		input.nextToken();
		lastModified = (long)input.nval; // get lastModified
		input.nextToken();
		size = (long)input.nval; // get size
		input.nextToken();
		type = input.sval; // get type
		input.nextToken();
		numKeywords = (int)input.nval; // get number of keywords associated with the file
		input.nextToken();
		for (int j = 0; j < numKeywords; j++) { // get the keywords
		    keywords.add(input.sval);
		    input.nextToken();
		}
	    } catch (IOException e) {
		System.out.println("Meta file error in " + name);
		System.exit(1);
	    }

	    System.out.print("\nName: " + name +
			     "\nLast Modified: " + lastModified +
			     "\nSize: " + size +
			     "\nType: " + type +
			     "\nNumKeywords: " + numKeywords +
			     "\nKeywords: ");
	    for (int j = 0; j < numKeywords; j++) {
		keyword = (String)keywords.get(j);
		System.out.print(keyword + " ");
	    }
	} // for
    }
    */

    /* Dispatches TapestryPublishMsgs for all metadata in the shared directory
     */
    public void dispatchPublishes() {
        TapestryPublishMsg tpm;
        SecureHash id;
        FilenameTag filenameTag;
	LoginTag loginTag;
        LastModifiedTag lastModifiedTag;
        SizeTag sizeTag;
        TypeTag typeTag;
	KeywordTag keywordTag;
	String name;
	long lastModified = 0;
	long size = 0;
	String type;
	int numKeywords = 0;
	String keyword;

        File[] files = _sharedDir.listFiles();

        for (int i = 0; i < files.length; i++) {
	    LinkedList keywords = new LinkedList();

            if (!files[i].isHidden()) continue;
            name = files[i].getName();
            type = name.substring(name.lastIndexOf(".") + 1, name.length());
	    name = name.substring(0, name.lastIndexOf("."));
            if (!type.equals("meta")) continue;

            Reader r = null;

            try {
                r = new InputStreamReader(new FileInputStream(files[i]));
            } catch (FileNotFoundException e) {
                error("Metafile for " + name + " could not be found.");
                System.exit(1);
            }

            StreamTokenizer input = new StreamTokenizer(r);

	    try {
		input.nextToken();
		name = input.sval; // get filename
		input.nextToken();
		lastModified = (long)input.nval; // get lastModified
		input.nextToken();
		size = (long)input.nval; // get size
		input.nextToken();
		type = input.sval; // get type
		input.nextToken();
		numKeywords = (int)input.nval; // get number of keywords associated with the file
		input.nextToken();
		for (int j = 0; j < numKeywords; j++) { // get the keywords
		    keywords.add(input.sval);
		    input.nextToken();
		}
	    } catch (IOException e) {
		error("Metafile error in " + name);
		System.exit(1);
	    }

            // send a TPM for every piece of metadata in each file
	    byte[] filenameBytes = name.getBytes();
            SHA1Hash filenameHash = new SHA1Hash(filenameBytes);
            filenameTag = new FilenameTag(name);
	    tpm = new TapestryPublishMsg(filenameHash, filenameTag);
	    output("Publishing filename " + name);
            dispatch(tpm);
	    _publications.add(new InterweavePublication(filenameHash, filenameTag));

	    byte[] lastModifiedBytes = new byte[1];
	    lastModifiedBytes[0] = new Long(lastModified).byteValue();
	    SHA1Hash lastModifiedHash = new SHA1Hash(lastModifiedBytes);
            lastModifiedTag = new LastModifiedTag(lastModified);
            tpm = new TapestryPublishMsg(lastModifiedHash, lastModifiedTag);
	    System.out.println("Publishing last modified date " + lastModified + " with hash " + lastModifiedHash);
            dispatch(tpm);
	    _publications.add(new InterweavePublication(lastModifiedHash, lastModifiedTag));

	    byte[] sizeBytes = new byte[1];
	    sizeBytes[0] = new Long(size).byteValue();
	    SHA1Hash sizeHash = new SHA1Hash(sizeBytes);
            sizeTag = new SizeTag(size);
            tpm = new TapestryPublishMsg(sizeHash, sizeTag);
	    System.out.println("Publishing size " + size + " with hash " + sizeHash);
            dispatch(tpm);
	    _publications.add(new InterweavePublication(sizeHash, sizeTag));

	    byte[] typeBytes = type.getBytes();
	    SHA1Hash typeHash = new SHA1Hash(typeBytes);
            typeTag = new TypeTag(type);
            tpm = new TapestryPublishMsg(typeHash, typeTag);
	    System.out.println("Publishing type " + type);
            dispatch(tpm);
	    _publications.add(new InterweavePublication(typeHash, typeTag));

	    for (int k = 0; k < numKeywords; k++) {
		keyword = (String)keywords.get(k);
		byte[] keywordBytes = keyword.getBytes();
		SHA1Hash keywordHash = new SHA1Hash(keywordBytes);
		keywordTag = new KeywordTag(keyword);
		tpm = new TapestryPublishMsg(keywordHash, keywordTag);
		System.out.println("Publishing keyword " + keyword + " in association with file " + name);
		dispatch(tpm);
		_publications.add(new InterweavePublication(keywordHash, keywordTag));
	    }
        }
    }

    /* Publish this client's login
     */
    public void publishLogin() {
        TapestryPublishMsg tpm;
	LoginTag loginTag;

	byte[] loginBytes = _login.getBytes();
	SHA1Hash loginHash = new SHA1Hash(loginBytes);
	loginTag = new LoginTag(_login);
	tpm = new TapestryPublishMsg(loginHash, loginTag);
	System.out.println("Publishing login " + _login);
	dispatch(tpm);
	_publications.add(new InterweavePublication(loginHash, loginTag));
    }

    /* Creates or updates metadata for all files in the shared directory DIR
     * The meta file is created as .FILENAME.TYPE.meta
     */
    public void prepSharedDir() {
	File[] files = _sharedDir.listFiles();
	for (int i = 0; i < files.length; i++) {
	    if (!files[i].isHidden()) { // if it's not a hidden file, we have to make or update a metafile
		String name = files[i].getName();
		String type = name.substring(name.lastIndexOf(".") + 1, name.length());
		name = name.substring(0, name.lastIndexOf("."));
		
		String metaFilename = files[i].getAbsolutePath().substring(0,
									   files[i].getAbsolutePath().lastIndexOf("/"))
		    + "/." + name + "." + type + ".meta";

		File metaFile = new File(metaFilename);
		if (!metaFile.exists()) {
		    // create the metafile and package it into a printstream to write to
		    PrintStream metaFileWriter = null;
		    try {
			metaFileWriter = new PrintStream(new FileOutputStream(new File(metaFilename)));
		    } catch (FileNotFoundException e) {
			System.err.println("File " + metaFilename + " cannot be written to.");
		    }
		    
		    // write to the metafile
		    metaFileWriter.println(name + "\n" + 
					   files[i].lastModified() + "\n" + 
					   files[i].length() + "\n" + 
					   type + "\n" +
					   "0"); // num keywords is initially 0
		} else {
		    updateMetafile(metaFile);
		}
	    } else { // the current file is hidden, so make sure it is up to date
		updateMetafile(files[i]);
	    }
	} // for
    }

    public void updateMetafile(File metafile) {
	String mfName = metafile.getName();
	String mfType = mfName.substring(mfName.lastIndexOf(".") + 1, mfName.length());
	mfName = mfName.substring(1, mfName.lastIndexOf(".")); // get rid of the leading .
	
	int mfNumKeywords = 0;
	LinkedList mfKeywords = new LinkedList();
	String keyword;
	
	if (!mfType.equals("meta")) return; // if it's not a metafile, we're done
		
	// read in the metafile
	Reader r = null;
	FileInputStream fis = null;
	
	try {
	    fis = new FileInputStream(metafile);
	    r = new InputStreamReader(fis);
	} catch (FileNotFoundException e) {
	    error("Metafile for " + mfName + " could not be found");
	    return;
	} catch (Exception e) {
	    error("Could not read metafile");
	    return;
	}
	
	StreamTokenizer input = new StreamTokenizer(r);
	
	try {
	    input.nextToken();
	    // skip getting the name field, we'll update it to ensure correctness
	    input.nextToken();
	    // skip getting the last modified field, we'll update it to ensure correctness
	    input.nextToken();
	    // skip getting the size field, we'll update it to ensure correctness
	    input.nextToken();
	    // skip getting the type field, we'll update it to ensure correctness
	    input.nextToken();
	    mfNumKeywords = (int)input.nval; // get number of keywords associated with the file
	    input.nextToken();
	    for (int j = 0; j < mfNumKeywords; j++) { // get the keywords
		mfKeywords.add(input.sval);
		input.nextToken();
	    }
	} catch (IOException e) {
	    error("Metafile error in " + mfName);
	    return;
	}
	
	try {
	    r.close();
	    fis.close();
	} catch (Exception e) {
	    error("Couldl not close metafile");
	    return;
	}
	
	mfType = mfName.substring(mfName.lastIndexOf(".") + 1, mfName.length());
	mfName = mfName.substring(0, mfName.lastIndexOf("."));
	File file = new File(_sharedDir + "/" + mfName + "." + mfType);
	if (!file.exists()) {
	    error("File " + _sharedDir + "/" + mfName + "." + mfType + " could not be found.");
	    return;
	}
	
	// recreate the metafile and package it into a printstream to write to
	PrintStream metaFileWriter = null;
	try {
	    metaFileWriter = new PrintStream(new FileOutputStream(metafile));
	} catch (FileNotFoundException e) {
	    error("Metafile ." + mfName + "." + mfType + " cannot be written to.");
	    return;
	}

	// write to the metafile
	metaFileWriter.println(mfName + "\n" + 
			       file.lastModified() + "\n" +
			       file.length() + "\n" + 
			       mfType + "\n" +
			       mfNumKeywords);
	for (int j = 0; j < mfNumKeywords; j++) { // write the keywords
	    keyword = (String)mfKeywords.get(j);
	    metaFileWriter.println(keyword);
	}		    
    }

    public void dispatchUnpublishes() {
	for (int i = 0; i < _publications.size(); i++) {
	    InterweavePublication ip = (InterweavePublication)_publications.remove(0);
	    dispatch(new TapestryUnpublishMsg(ip.guid, ip.tag));
	}
    }

    public void output(String s) {
	// System.out.println(s);
	if (status != null) {
	    status.setText(s);
	} else {
	    System.out.println(s);
	}
    }

    public void error(String s) {
	System.err.println("Error: " + s);
    }

    public void exit() {
	output("Logging out of the Tapestry network. Please wait.");
	dispatchUnpublishes();
	dispatch(new TapestryNodeDetachMsg());
	System.exit(0);	
    }

    /*
    private class InterweaveInputThread extends Thread {
	Interweave owner;
	BufferedReader in;
	String input;

	public InterweaveInputThread(Interweave o) {
	    owner = o;
	    in = new java.io.BufferedReader(new java.io.InputStreamReader(System.in));
	}

	public void run() {
	    while (true) {
		input = "//";
		System.out.print("\n[Interweave] % ");
		while (input.indexOf("//") == 0) {
		    try {
			input = in.readLine();
			yield();
		    } catch(Exception e) {
			e.printStackTrace();
		    }
		    if (input != null) owner.handleInput(input);
		}
	    }
	}
    }
    */

    /**************************************** GUI METHODS ************************************/

    public void actionPerformed(ActionEvent e) {
	if (e.getSource() instanceof JMenuItem) {
	    JMenuItem source = (JMenuItem)e.getSource();
	    if ("Connect".equals(source.getText())) {
		if (_state != "disconnected") {
		    JOptionPane.showMessageDialog(_JInterweaveFrame,
						  "Interweave is already connected",
						  "Connect",
						  JOptionPane.WARNING_MESSAGE);
		    return;
		}
		
		// TODO: reconnect

	    } else if ("Disconnect".equals(source.getText())) {
		_state = "disconnecting";
		dispatchUnpublishes();
		dispatch(new TapestryNodeDetachMsg());

	    } else if ("Exit".equals(source.getText())) {
		exit();

	    } else if ("Clear Search Results".equals(source.getText())) {
		for (int i = 0; i < _resultCount; i++) {
		    _searchResults.remove(new Integer(i));
		}
		_resultCount = 0;
		output("Search results cleared");

	    } else if ("Refresh Search Results".equals(source.getText())) {
		InterweaveSearchResult[] data = new InterweaveSearchResult[_resultCount];
		for (int i = 0; i < _resultCount; i++) {
		    data[i] = (InterweaveSearchResult)_searchResults.get(new Integer(i));
		}
		searchResultTableModel.refresh(data,
				new javax.swing.event.TableModelEvent(searchResultTableModel));
		
	    } else if ("Refresh Downloaded Files".equals(source.getText())) {
		downloadTableModel.refresh(_downloadDir.listFiles(new InterweaveFileFilter()), 
				new javax.swing.event.TableModelEvent(downloadTableModel));

	    } else if ("Refresh Shared Files".equals(source.getText())) {
		sharedTableModel.refresh(_sharedDir.listFiles(new InterweaveFileFilter()), 
				new javax.swing.event.TableModelEvent(sharedTableModel));

	    } else if ("Set Download Directory".equals(source.getText())) {
		fc.setDialogTitle("Set Download Directory");
		fc.setFileFilter(new DirectoryFilter());
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		fc.setApproveButtonText("Set");
		fc.setApproveButtonToolTipText("Sets the download directory");
		int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
		    File file = fc.getSelectedFile();
		    boolean done = setDir(file, false); // set download dir
		    if (done) {
			output("Download directory changed to " + file.getAbsolutePath());
		    } else {
			output("Download directory not changed.");
		    }
		}
		
	    } else if ("Set Shared Directory".equals(source.getText())) {
		fc.setDialogTitle("Set Shared Directory");
		fc.setFileFilter(new DirectoryFilter());
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		fc.setApproveButtonText("Set");
		fc.setApproveButtonToolTipText("Sets the shared directory");
		int returnVal = fc.showOpenDialog(this);	    
		if (returnVal == JFileChooser.APPROVE_OPTION) {
		    File file = fc.getSelectedFile();
		    boolean done = setDir(file, true); // set shared dir
		    if (done) {
			output("Shared directory changed to " + file.getAbsolutePath());
		    } else {
			output("Shared directory not changed.");
		    }
		}
	    } else if ("Contents".equals(source.getText())) {
		JOptionPane.showMessageDialog(_JInterweaveFrame,
					      "Not yet implemented",
					      "Help Contents",
					      JOptionPane.PLAIN_MESSAGE);
	    } else if ("About Interweave".equals(source.getText())) {
		JOptionPane.showMessageDialog(_JInterweaveFrame,
					      "Interweave for Tapestry\nVersion: 2.0\nCreated By: Benjamin Poon",
					      "About Interweave",
					      JOptionPane.PLAIN_MESSAGE);
	    }
	} else if (e.getSource() instanceof JTextField) {
	    String primFieldType = (String)primaryFieldList.getSelectedItem();
	    String primSearchStr = primaryTextField.getText();
	    primaryTextField.selectAll();
	    String s = primFieldType.toLowerCase() + " " + primSearchStr;

	    status.setText("Searching...");
	    StringTokenizer st = new StringTokenizer(s, " ");
	    search(st);
	} else if (e.getSource() instanceof JButton) {
	    JButton source = (JButton)e.getSource();
	    if ("Download Selected".equals(source.getText())) {
		if (_downloadDir == null) {
		    JOptionPane.showMessageDialog(_JInterweaveFrame,
						  "No download directory is set. Please set one before downloading.",
						  "Download Selected",
						  JOptionPane.WARNING_MESSAGE);
		    return;
		}
		int[] selections = searchResultTable.getSelectedRows();

		Iterator isrs = null;
		for (int i = 0; i < selections.length; i++) {
		    isrs = _searchResults.values().iterator();
		    while (isrs.hasNext()) {
			InterweaveSearchResult isr = (InterweaveSearchResult)isrs.next();
			if ((isr.filename).equals((String)searchResultTableModel.getValueAt(selections[i], 1))
			    && (isr.type).equals((String)searchResultTableModel.getValueAt(selections[i], 5))) {
			    output("Requesting download from " + isr.peer + " for " + isr.filename + "." + isr.type);
			    InterweaveFileRequestMsg ifrm = new InterweaveFileRequestMsg(isr.peer, _client_guid, 
											 "" + isr.filename + "." + isr.type);
			    dispatch(ifrm);
			}
		    }
		}
	    }
	} else if (e.getSource() instanceof ViewSearchAction) {
	    tabPanel.setSelectedIndex(0);
	}
    }

    private JFrame makeJInterweaveFrame() {
	JFrame f = new JFrame("Interweave");

        f.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    exit();
		}
	    });

        Container contentPane = f.getContentPane();

	// Make tabbed panel
	tabPanel = makeTabPanel();
	contentPane.add(tabPanel, BorderLayout.CENTER);

	// Create the status bar
        status = new JLabel("Connecting to the Tapestry network");
        status.setBorder(BorderFactory.createEtchedBorder());
	contentPane.add(status, BorderLayout.SOUTH);

	// Create the menu bar
	menuBar = new JMenuBar();
	f.setJMenuBar(menuBar);

	// Build the FILE menu
	menu = new JMenu("File");
	menu.setMnemonic(KeyEvent.VK_F);
	menuBar.add(menu);

	menuItem = new JMenuItem("Connect", KeyEvent.VK_C);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Disconnect", KeyEvent.VK_D);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();

	menuItem = new JMenuItem("Exit", KeyEvent.VK_E);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, ActionEvent.ALT_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// Build the VIEW menu
	menu = new JMenu("View");
	menu.setMnemonic(KeyEvent.VK_V);
	menuBar.add(menu);

	menuItem = new JMenuItem(new ViewSearchAction("View Search Pane", "View Search Pane", new Integer(KeyEvent.VK_S)));
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.ALT_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Refresh Search Results", KeyEvent.VK_A);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.CTRL_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Clear Search Results", KeyEvent.VK_C);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();

	menuItem = new JMenuItem(new ViewDownloadedAction("View Downloaded Files", "View Downloaded Files", new Integer(KeyEvent.VK_V)));
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, ActionEvent.ALT_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Refresh Downloaded Files", KeyEvent.VK_D);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, ActionEvent.CTRL_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();

	menuItem = new JMenuItem(new ViewSharedAction("View Shared Files", "View Shared Files", new Integer(KeyEvent.VK_I)));
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, ActionEvent.ALT_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Refresh Shared Files", KeyEvent.VK_F);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, ActionEvent.CTRL_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// Build the OPTIONS menu
	menu = new JMenu("Options");
	menu.setMnemonic(KeyEvent.VK_O);
	menuBar.add(menu);

	menuItem = new JMenuItem("Set Download Directory", KeyEvent.VK_D);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.ALT_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem("Set Shared Directory", KeyEvent.VK_S);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.ALT_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	// Build the Help menu
	menu = new JMenu("Help");
	menu.setMnemonic(KeyEvent.VK_H);
	menuBar.add(menu);

	menuItem = new JMenuItem("Contents", KeyEvent.VK_C);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, ActionEvent.ALT_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();

	menuItem = new JMenuItem("About Interweave", KeyEvent.VK_A);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F2, ActionEvent.ALT_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	f.pack();
	f.setVisible(true);
	return f;
    }

    private JTabbedPane makeTabPanel() {
        JTabbedPane p = new JTabbedPane();
        Component searchPanel = makeSearchPanel();

        p.addTab("Search", null, searchPanel, "Search for files and display search results");
        p.setSelectedIndex(0);

        Component downloadedPanel = makeDownloadedPanel();
        p.addTab("Downloaded Files", null, downloadedPanel, "Displays files in the download directory");

        Component sharedPanel = makeSharedPanel();
        p.addTab("Shared Files", null, sharedPanel, "Displays files in the shared directory");

	return p;
    }

    private Component makeSearchPanel() {
	String[] primaryFieldTypes = { "Filename", "Login", "Keyword" };
	//	String[] secondaryFieldTypes = { "Filename, "Keyword", "Last Modified Before...", "Last Modified After..., "Last Modified On...", "Size", "Type" };

	searchFieldsPanelContainer = new JPanel();

	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	searchFieldsPanelContainer.setLayout(gridbag);
	c.insets = new Insets(10,10,10,10);  //top padding

	// Create the primary field type label	
	c.gridx = 0;
	c.gridy = 0;
	primaryLabel = new JLabel("Primary Field Type:");
	gridbag.setConstraints(primaryLabel, c);
	searchFieldsPanelContainer.add(primaryLabel);

	// Create the primary field type combo box
	c.gridx = 0;
	c.gridy = 1;
	primaryFieldList = new JComboBox(primaryFieldTypes);
	gridbag.setConstraints(primaryFieldList, c);
	primaryFieldList.setSelectedIndex(0);
	searchFieldsPanelContainer.add(primaryFieldList);

	// Create the primary field search string label
	c.gridx = 1;
	c.gridy = 0;
	primaryStrLabel = new JLabel("Primary Field Search String:");
	gridbag.setConstraints(primaryStrLabel, c);
	searchFieldsPanelContainer.add(primaryStrLabel);

	// Create the textfield for search string entry
	c.gridx = 1;
	c.gridy = 1;
	primaryTextField = new JTextField(40);
	gridbag.setConstraints(primaryTextField, c);
	primaryTextField.addActionListener(this);
	searchFieldsPanelContainer.add(primaryTextField);

	// Create a blank panel for the remainder of space
	c.weighty = 1.0; // request any extra vertical space for the blank panel
	c.gridx = 0; // aligned left
	c.gridwidth = 2; // 2 columns wide
	c.gridy = 2; // third row
	blankPanel = makeTextPanel("");
	gridbag.setConstraints(blankPanel, c);
	searchFieldsPanelContainer.add(blankPanel);

	// Create the search fields scroll pane and put the search field panel in it
	searchFieldsScrollPane = new JScrollPane(searchFieldsPanelContainer);

	// Create the search results panel
	searchResultsPane = makeSearchResultPanel();

	// Create a split pane with the two search panes in it
	splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				   searchFieldsScrollPane, searchResultsPane);
	splitPane.setOneTouchExpandable(true);
	splitPane.setDividerLocation(200);

	//Provide minimum sizes for the two components in the split pane
	Dimension minimumSize = new Dimension(400, 200);
	searchFieldsScrollPane.setMinimumSize(minimumSize);
	searchResultsPane.setMinimumSize(minimumSize);

        //Provide a preferred size for the split pane
        splitPane.setPreferredSize(new Dimension(700, 500));

	return splitPane;
    }

    private JScrollPane makeSearchResultPanel() {
	if (_searchResults == null) {
	    return null;
	}

	JPanel p = new JPanel();
	p.setLayout(new BorderLayout());
	downloadButton = new JButton("Download Selected");
	downloadButton.setEnabled(false);
	downloadButton.addActionListener(this);
	p.add(downloadButton, BorderLayout.SOUTH);

	InterweaveSearchResult[] data = new InterweaveSearchResult[_resultCount];
	for (int i = 0; i < _resultCount; i++) {
	    data[i] = (InterweaveSearchResult)_searchResults.get(new Integer(i));
	}
        searchResultTableModel = new SearchResultTableModel(data);
        sorter = new TableSorter((SearchResultTableModel)searchResultTableModel);
        searchResultTable = new JTable(sorter);
        sorter.addMouseListenerToHeaderInTable(searchResultTable);
        searchResultTable.setPreferredScrollableViewportSize(new Dimension(500, 70));

	searchResultTable.setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        javax.swing.ListSelectionModel rowSM = searchResultTable.getSelectionModel();
        rowSM.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent e) {
                //Ignore extra messages.
                if (e.getValueIsAdjusting()) return;
                
                ListSelectionModel lsm =
                    (ListSelectionModel)e.getSource();
                if (lsm.isSelectionEmpty()) {
		    return;
                } else {
		    downloadButton.setEnabled(true);
                }
            }
        });

	p.add(searchResultTable, BorderLayout.CENTER);
        return new JScrollPane(p);
    }

    private Component makeDownloadedPanel() {
	if (_downloadDir == null) {
	    return makeTextPanel("Downloaded Files");
	}

        downloadTableModel = new FileTableModel(_downloadDir.listFiles(new InterweaveFileFilter()));
        sorter = new TableSorter((FileTableModel)downloadTableModel);
        downloadTable = new JTable(sorter);
        sorter.addMouseListenerToHeaderInTable(downloadTable);
        downloadTable.setPreferredScrollableViewportSize(new Dimension(500, 70));
        return new JScrollPane(downloadTable);
    }

    private Component makeSharedPanel() {
	if (_sharedDir == null) {
	    return makeTextPanel("Shared Files");
	}

        sharedTableModel = new FileTableModel(_sharedDir.listFiles(new InterweaveFileFilter()));
        sorter = new TableSorter((FileTableModel)sharedTableModel);
        sharedTable = new JTable(sorter);
        sorter.addMouseListenerToHeaderInTable(sharedTable);
        sharedTable.setPreferredScrollableViewportSize(new Dimension(500, 70));
	sharedTable.setRowSelectionAllowed(true);
        return new JScrollPane(sharedTable);
    }

    protected Component makeTextPanel(String text) {
        JPanel panel = new JPanel(false);
        JLabel filler = new JLabel(text);
        filler.setHorizontalAlignment(JLabel.CENTER);
        panel.setLayout(new GridLayout(1, 1));
        panel.add(filler);
        return panel;
    }

    class ViewSearchAction extends AbstractAction {
	public ViewSearchAction(String text, String desc, Integer mnemonic) {
	    super(text);
	    putValue(SHORT_DESCRIPTION, desc);
	    putValue(MNEMONIC_KEY, mnemonic);
	}
	
	public void actionPerformed(ActionEvent e) {
	    tabPanel.setSelectedIndex(0);
	}
    }
    
    class ViewDownloadedAction extends AbstractAction {
	public ViewDownloadedAction(String text, String desc, Integer mnemonic) {
	    super(text);
	    putValue(SHORT_DESCRIPTION, desc);
	    putValue(MNEMONIC_KEY, mnemonic);
	}
	
	public void actionPerformed(ActionEvent e) {
	    tabPanel.setSelectedIndex(1);
	}
    }

    class ViewSharedAction extends AbstractAction {
	public ViewSharedAction(String text, String desc, Integer mnemonic) {
	    super(text);
	    putValue(SHORT_DESCRIPTION, desc);
	    putValue(MNEMONIC_KEY, mnemonic);
	}
	
	public void actionPerformed(ActionEvent e) {
	    tabPanel.setSelectedIndex(2);
	}
    }

    /******************************************* END GUI METHODS **************************************/




    public void destroy() throws Exception {
    // TODO
    }

    //  methods for ProfilableIF
    /* Specified by seda.sandStorm.api.ProfilableIF */
    public int profileSize()
    {
	return this_sink.size();
    }
    //  end methods for ProfilableIF

    public static String printGuid (SecureHash guid) {
        String out="";
	int [] gdigits = RoutingTable.digits(guid);
	for (int i = 0; i < gdigits.length; ++i) {
	    out += Integer.toHexString (gdigits [i]);
	}   
	return out;
    }

    private Thread inputThread = null;
    private File _sharedDir; // user-set directory where shared files reside
    private File _downloadDir; // user-set directory where downloaded files are placed
    private HashMap _searchResults; // hash resultCount -> InterweaveSearchResult
    private LinkedList _publications; // linked list of InterweavePublications
    private int _resultCount;
    private int _curSearchResultCount;
    private String _login;
    private String _state;

    // GUI
    private JFrame _JInterweaveFrame;
    private JLabel status;
    private JMenuBar menuBar;
    private JMenu menu;
    private JMenuItem menuItem;
    private final JFileChooser fc = new JFileChooser();
    private JTabbedPane tabPanel;

    // GUI: search tab
    private JSplitPane splitPane;
    private JScrollPane searchFieldsScrollPane;
    private JPanel searchFieldsPanelContainer;
    private JPanel searchFieldsPanel1;
    private JPanel searchFieldsPanel2;
    private Component blankPanel;
    private JLabel primaryLabel;
    private JLabel primaryStrLabel;
    private JComboBox primaryFieldList;
    private JTextField primaryTextField;
    private JButton downloadButton;
    private JScrollPane searchResultsPane;
    private JPanel searchResultsPanel;
    private SearchResultTableModel searchResultTableModel;
    private JTable searchResultTable;

    // GUI: downloaded, shared, and search tables
    private FileTableModel downloadTableModel, sharedTableModel;
    private TableSorter sorter;
    private JTable downloadTable, sharedTable;
	    
    private boolean DEBUG;
    private SecureHash _client_guid;
    
    private SinkIF this_sink;
    private NodeId _self_node_id;
    private Classifier _classifier;
        
}

class DirectoryFilter extends javax.swing.filechooser.FileFilter { // as opposed to java.io.FileFilter
    
    public boolean accept(File f) {
        if (f.isDirectory()) {
            return true;
        }

        return false;
    }
    
    // The description of this filter
    public String getDescription() {
        return "Directories";
    }
}

class InterweaveFileFilter implements FileFilter { // as opposed to java.io.FileFilter
    
    public boolean accept(File f) {
        if (!f.isDirectory() && !f.isHidden()) {
            return true;
        }

        return false;
    }
}

class FileTableModel extends javax.swing.table.AbstractTableModel {
    private String[] columnNames = { "Filename", "Size", "Last Modified" };
    private File[] data;

    public FileTableModel(File[] data) {
	this.data = data;
    }

    public int getColumnCount() {
	return columnNames.length;
    }
        
    public int getRowCount() {
	return data.length;
    }

    public String getColumnName(int col) {
	return columnNames[col];
    }

    public Object getValueAt(int row, int col) {
	if (col == FILENAME) {
	    return (Object)data[row].getName();
	} else if (col == SIZE) {
	    return (Object)(new Long(data[row].length()));
	} else if (col == LASTMOD) {
	    return (Object)(new Date(data[row].lastModified()));
	}
	return null;
    }

    public boolean isCellEditable(int row, int col) {
	return false;
    }

    public void refresh(File[] data, javax.swing.event.TableModelEvent e) {
	this.data = data;
	fireTableChanged(e);
    }

    public static final int FILENAME = 0;
    public static final int SIZE = 1;
    public static final int LASTMOD = 2;
    public static final int KEYWORDS = 3;
}

class SearchResultTableModel extends javax.swing.table.AbstractTableModel {
    private String[] columnNames = { "Peer", "Filename", "Login", "Last Modified", "Size", "Type" };
    private InterweaveSearchResult[] data;

    public SearchResultTableModel(InterweaveSearchResult[] data) {
	this.data = data;
    }

    public int getColumnCount() {
	return columnNames.length;
    }
        
    public int getRowCount() {
	return data.length;
    }

    public String getColumnName(int col) {
	return columnNames[col];
    }

    public Object getValueAt(int row, int col) {
	if (col == PEER) {
	    return (Object)data[row].peer;
	} else if (col == FILENAME) {
	    return (Object)data[row].filename;
	} else if (col == LOGIN) {
	    return (Object)data[row].login;
	} else if (col == LASTMODIFIED) {
	    return (Object)(new Long(data[row].lastModified));
	} else if (col == SIZE) {
	    return (Object)(new Long(data[row].size));
	} else if (col == TYPE) {
	    return (Object)data[row].type;
	}
	return null;
    }

    public boolean isCellEditable(int row, int col) {
	return false;
    }

    public void refresh(InterweaveSearchResult[] data, javax.swing.event.TableModelEvent e) {
	this.data = data;
	fireTableChanged(e);
    }

    public static final int PEER = 0;
    public static final int FILENAME = 1;
    public static final int LOGIN = 2;
    public static final int LASTMODIFIED = 3;
    public static final int SIZE = 4;
    public static final int TYPE = 5;
}

