package ostore.tapestry.interweave;
import java.io.*;
import ostore.tapestry.api.*;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import ostore.util.OutputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.InputBuffer;

/**
 * InterweaveFileMsg.java
 *
 * The message that carries a requested file being sent to SecureHash TO
 *
 * @author Benjamin K. Poon
 */
public class InterweaveFileMsg
    extends TapestryRouteMsg implements QuickSerializable {

    public String filename;
    File file;

    public InterweaveFileMsg (SecureHash to, String filename, File file) {
	super(to);
	this.filename = filename;
	this.file = file;
    }

    public InterweaveFileMsg (InputBuffer buffer) throws QSException {
	super(buffer);
	filename = buffer.nextString();
	long length = buffer.nextLong();
	FileOutputStream fos = null;

	try {
	    file = File.createTempFile(".temp", ".tmp");
	} catch (Exception e) {
	    System.out.println("[InterweaveFileMsg] Error: could not create temp file.");
	}

	try {
	    fos = new FileOutputStream(file);
	    for (long i = 0; i < length; i++) {
		fos.write(buffer.nextByte());
	    }
	} catch (Exception e) {
	    System.out.println("[InterweaveFileMsg] Error: could not write to temp file.");
	}
    }
    
    public void serialize (OutputBuffer buffer) {
	super.serialize (buffer);
	buffer.add(filename);
	buffer.add(file.length());

	FileInputStream fis = null;

	// open the file for input
	try {
	    fis = new FileInputStream(file);
	} catch (Exception e) {
	    System.out.println("[InterweaveFileMsg] Error: corrupt file.");
	}

	// write file to buffer
	try {
	    for (long i = 0; i < file.length(); i++) {
		buffer.add((byte)fis.read());
	    }
	} catch (Exception e) {
	    System.out.println("[InterweavefileMsg] Error: file could not be sent.");
	}
    }
    
    public Object clone () throws CloneNotSupportedException {
	InterweaveFileMsg result = (InterweaveFileMsg) super.clone();
	result.filename = filename;
	result.file = file;
	return result;
    }

    public String toString() {
	return "(InterweaveFileMsg: filename="+filename+")";
    }

}
