/**
 * LastModifiedTag.java
 *
 * @author	Benjamin K. Poon
 * @version	$Id: LastModifiedTag.java,v 1.1 2002/11/20 09:45:09 bpoon1 Exp $
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.tapestry.interweave;

import ostore.util.TypeTable;
import ostore.util.QSException;
import ostore.util.QuickSerializable;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QuickSerializable;
import ostore.tapestry.api.TapestryTag;

/**
 * <CODE>LastModifiedTag</CODE> is the {@link ostore.tapestry.api.TapestryTag}
 * for filename objects.  
 *
 * @see ostore.tapestry.api.TapestryTag
 *
 * @author	Benjamin K. Poon
 * @version	$Id: LastModifiedTag.java,v 1.1 2002/11/20 09:45:09 bpoon1 Exp $
 */

public class LastModifiedTag implements TapestryTag
{
  ////////////////////       CONSTANTS	////////////////////
  ////////////////////    STATIC VARIABLES 	////////////////////
  ////////////////////       VARIABLES	////////////////////
    long lastModified;
  ////////////////////  STATIC CONSTRUCTORS	////////////////////
  ////////////////////      CONSTRUCTORS	////////////////////

  /**
   * Constructor:
   * Creates a new <code>LastModifiedTag</code>. */
  public LastModifiedTag(long lastModified) {
      this.lastModified = lastModified;
  }
    
  /**
   * Constructs a <CODE>LastModifiedTag</CODE> from its
   * {@link ostore.util.QuickSerializable#serialize serialized} form.
   *
   * @param buffer {@link ostore.util.QuickSerializable#serialize serialized}
   *                form of object.  */
  public LastModifiedTag(InputBuffer buffer) throws QSException{}

  ////////////////////     STATIC METHODS 	////////////////////
  //////////////////// 	METHODS		////////////////////

  /**
   * Specified by ostore.util.QuickSerializable */
  public void serialize(OutputBuffer buffer){ /* do nothing */}

    

  /**
   * Compares this LastModifiedTag with another tag, and returns
   * true iff they are the same type of tag.
   * @param the object with which to compare this LastModifiedTag.
   * @return true iff this and other are instanceof LastModifiedTag. */
  public boolean equals(Object other)
  {
    return (other instanceof LastModifiedTag &&
	    (lastModified == ((LastModifiedTag)other).lastModified));
  }

  /**
   * Calculates a Java hash code for this LastModifiedTag object
   * (by returning its type_code)
   * @return the Java hash code for this LastModifiedTag. */
  public int hashCode()
  {
    return (int)TypeTable.type_code(this);
  }

  /**
   * Specified by java.lang.Object */
    public String toString () { return "(LastModifiedTag)"; }
  ////////////////////     INNER CLASSES	////////////////////
}
