/**
 * SizeQuery.java
 *
 * @author   Benjamin K. Poon
 * @version  $Id: SizeQuery.java,v 1.1 2002/11/20 09:45:09 bpoon1 Exp $
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.tapestry.interweave;

import ostore.util.Carp;
import ostore.util.Debug;
import ostore.util.Types;
import ostore.util.DebugFlags;
import ostore.util.SecureHash;
import ostore.util.QSException;
import ostore.util.QuickSerializable;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.VerifiableBlockTag;
import ostore.inner.InnerRingTag;
import ostore.tapestry.api.TapestryTag;
import ostore.tapestry.api.TapestryQuery;
import ostore.tapestry.api.TapestryQueryResult;
import ostore.tapestry.api.TapestryQueryResultMatch;
import ostore.tapestry.api.TapestryQueryResultNoMatch;
import ostore.tapestry.api.TapestryQueryState;

/**
 * <CODE>SizeQuery</CODE> is a 
 * {@link ostore.tapestry.api.TapestryQuery} used in a  
 * {@link ostore.tapestry.api.TapestryLocateMsg}
 * used to find objects identified by a {@link SizeTag}.
 *
 * <CENTER><H4>USAGE</H4></CENTER>
 * <P>
 * To receive an object identified by a {@link SizeTag}, dispatch a 
 * {@link ostore.tapestry.api.TapestryLocateMsg} that contains a 
 * <CODE>SizeQuery</CODE>.
 * </P>
 *
 * @see SizeTag
 * @see ostore.tapestry.api.TapestryTag
 * @see ostore.tapestry.api.TapestryQuery
 * @see ostore.tapestry.api.TapestryLocateMsg
 *
 * @author   Benjamin K. Poon
 * @version  $Id: SizeQuery.java,v 1.1 2002/11/20 09:45:09 bpoon1 Exp $
 */
public class SizeQuery implements TapestryQuery
{
  ////////////////////       CONSTANTS	////////////////////
  private static final boolean DEBUG = false;


  ////////////////////    STATIC VARIABLES 	////////////////////
  ////////////////////       VARIABLES	////////////////////

  ////////////////////  STATIC CONSTRUCTORS	////////////////////
  static
  {
    if(DEBUG) Debug.turnOn("SizeQuery");
  }

  ////////////////////      CONSTRUCTORS	////////////////////

  /**
   * Creates a new <code>SizeQuery</code>. */
  public SizeQuery() {}
    
  /**
   * Constructs a <CODE>SizeQuery</CODE> from its
   * {@link ostore.util.QuickSerializable#serialize serialized} form.
   *
   * @param buffer {@link ostore.util.QuickSerializable#serialize serialized}
   *                form of object.  */
  public SizeQuery(InputBuffer buffer) throws QSException{}
    
  ////////////////////     STATIC METHODS 	////////////////////
  //////////////////// 	METHODS		////////////////////

  /** Specified by ostore.util.QuickSerializable */
  public void serialize(OutputBuffer buffer){ /* do nothing */}    

   

  /* Specified by ostore.tapestry.api.TapestryQuery */
  public TapestryQueryResult query(SecureHash node_guid,
                                   SecureHash bp_guid,
				   TapestryTag tag,
				   TapestryQueryState state )
  {
    if(tag instanceof SizeTag)
    {
      if(DEBUG) Debug.printtagln("SizeQuery.query",
				 "tag is instanceof SizeTag" +
				 " we are about to return" +
				 " TapestryQueryResultMatch for nodeGuid==" +
				 bp_guid + " and stop the" +
				 " RequestFragMsg of type TapestryLocateMsg."+
				 "  Obj==" + this);
      SecureHash[] m = { bp_guid };
      return 
	new TapestryQueryResultMatch(m, true /* stop */);
    }
    else
    {
      if(DEBUG) Debug.printtagln("SizeQuery.query",
				 "tag is NOT instanceof SizeTag." +
				 "  Instead tag instanceof " +
				 tag + ".  Obj==" + this);
      return 
	new TapestryQueryResultNoMatch();
    }
  }

  /** Specified by java.lang.Object */
  public String toString()
  {
    String str = new String();

    str += "<SizeQuery";
    str += " " + super.toString();
    str += ">";

    return str;
  }    
  ////////////////////     INNER CLASSES	////////////////////
}
