/**
 * BlankBuilder.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.tapestry.patchwork;

import soss.core.BuilderIF;
import ostore.util.Carp;
import seda.sandStorm.main.SandstormConfig;
import seda.sandStorm.api.StageNameAlreadyBoundException;
import java.io.IOException;
import java.util.StringTokenizer;

/**
 * A configuration builder for empty nodes (plaeholders).
 * Modelled after <code>ostore.bloom.DualBuilder</code>.
 *
 * @author	Jeremy Stribling
 * @version	$Id: BlankBuilder.java,v 1.1 2002/08/28 00:09:00 strib Exp $
 */
public class BlankBuilder implements BuilderIF
{
  ////////////////////       CONSTANTS	////////////////////
  ////////////////////    STATIC VARIABLES 	////////////////////
  ////////////////////       VARIABLES	////////////////////
  ////////////////////  STATIC CONSTRUCTORS	////////////////////
  ////////////////////      CONSTRUCTORS	////////////////////

  /**
   * Set global parameters. */
  public BlankBuilder(String[] args)
  {

  }

  ////////////////////     STATIC METHODS 	////////////////////
  //////////////////// 	METHODS		////////////////////

  /**
   * Create a configuration object for a single virtual machine. */
  public SandstormConfig build_node(String[] args)
      throws StageNameAlreadyBoundException, IOException
  {

      if( args.length != 1 ) {
	  Carp.die( "args.length != 1" );
      }

      SandstormConfig result = new SandstormConfig();
      
      // BlankStage
      String[] init_args = {
	  "node_id=" + args[0]
      };
      result.addStage("BlankStage", 
		      "ostore.tapestry.patchwork.BlankStage",
		      init_args);
      

      // tell the simulator no network stage
      result.putBoolean( "no_sim_network", true );
      return result;
  }  
    ////////////////////     INNER CLASSES	////////////////////
}

