/**
 * PatchworkAlarmMsg.java
 *
 * @author Jeremy Stribling
 * @version    $Id: PatchworkAlarmMsg.java,v 1.2 2002/07/22 20:54:53 srhea Exp $
 */

package ostore.tapestry.patchwork;

import seda.sandStorm.api.QueueElementIF;
import ostore.util.NodeId;
import java.util.ArrayList;

/**
 * This represents a command to the <code>PatchworkStage</code>
 * indicating that it's time to send out a probe of a certain type
 * to the appropriate nodes.
 *
 * @author Jeremy Stribling
 * @version    $Id: PatchworkAlarmMsg.java,v 1.2 2002/07/22 20:54:53 srhea Exp $
 */
public class PatchworkAlarmMsg implements QueueElementIF {

    public static final int LATENCY = 0;
    public static final int BANDWIDTH = 1;
    public static final int LOSS = 2;
    public static final int RECALC_MODEL = 3;
    public static final int RESPOND = 4;

    public int type;

    public int level;

    public NodeId node;

    public ArrayList nodes;

    public PatchworkAlarmMsg( int type ) {

	if( (type != LATENCY) && (type != BANDWIDTH) && 
	    (type != LOSS) && (type != RECALC_MODEL) && (type != RESPOND) ) {
	  throw new RuntimeException( "Invalid type (" + type + 
				      ") for PatchworkAlarmMsg" ); 
	} else {
	  this.type = type;
	}
	level = -1;
	nodes = new ArrayList();
	//node = null;
    }

    public PatchworkAlarmMsg( int type, int level ) {

	this( type );

	if( level < 0 ) {
	  throw new RuntimeException( "Invalid level (" + level + 
				      "); must be > 0" ); 
	}
	this.level = level;

    }


    public PatchworkAlarmMsg( int type, NodeId node ) {

	this( type );
	addNode( node );

    }

    public void addNode( NodeId node ) {

	if( node == null ) {
	  throw new RuntimeException( "The node cannot be null" );
	}
	nodes.add( node );

    }

    public String toString() {
	return "(PatchworkAlarmMsg, type " + type + 
	       " " + super.toString() + ")";
    }
}

