/**
 * PatchworkRegressionStage.java
 *
 * @author	Jeremy Stribling
 * @version	$Id: PatchworkRegressionStage.java,v 1.7 2002/09/28 23:00:23 strib Exp $
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ostore.tapestry.patchwork;

import ostore.util.InvalidConfigFileException;

import ostore.util.Carp;
import ostore.util.Debug;
import ostore.util.DebugFlags;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.tapestry.impl.RoutingTable;
import ostore.util.NodeId;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.UnknownEventException;
import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.api.SinkFullException;   
import seda.sandStorm.api.StagesInitializedSignal; 
import java.util.Random;
import java.util.LinkedList;

/**
 * @author 	Jeremy Stribling
 * @version	$Id: PatchworkRegressionStage.java,v 1.7 2002/09/28 23:00:23 strib Exp $
 */  

public class PatchworkRegressionStage implements EventHandlerIF
{
    ////////////////////       CONSTANTS	////////////////////

    private Classifier classifier;

    private SinkIF my_sink;
    private RoutingTable routing_table;

    private int runmode = 0;

    private NodeId self_node_id;

    private NodeId otherNode;

    // from routing table
    private int digits_per_guid;
    private int digit_values;

    private PatchworkModel model;

    ////////////////////  STATIC CONSTRUCTORS	////////////////////
    ////////////////////      CONSTRUCTORS	////////////////////
    ////////////////////     STATIC METHODS 	////////////////////
    //////////////////// 	METHODS		////////////////////

    /**
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void handleEvents(QueueElementIF elemarr[]) 
	throws EventHandlerException
    {

	for( int i=0; i<elemarr.length; i++ ) {
	    handleEvent( elemarr[i] );
	}
    }


    /** 
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void handleEvent(QueueElementIF element) 
	throws EventHandlerException
    {

	if( element instanceof StagesInitializedSignal ) {

	    model = new PatchworkModel();
	    
	} else if( element instanceof PatchworkDeltaResponseMsg ) {

	    model.add_delta( ((PatchworkDeltaResponseMsg) element).delta );
	    double lossRate;

	    switch( runmode ) {
	    case 0:
		// do nothing, wait for fault to fully activate
		runmode++;
		break;
	    case 1:
		// Make sure there's not more than 10% loss,
		// because no faults have been injected yet.
		lossRate = 
		   model.lookupDouble(routing_table.node_id_to_guid(otherNode),
					       PatchworkModel.LOSS );
		if( lossRate > 20 ) {
		    System.err.println( "Loss rate is " + lossRate +
					"% before any faults.  Exiting." );
		    System.exit(-1);
		} else {
		    System.err.println( "Loss rate is good before any " +
					" faults.  Injecting 100% fault." );
		    PatchworkFaultMsg pfm = new PatchworkFaultMsg( otherNode,
								   100, true );
		    runmode++;
		    dispatch( pfm );
		}
		break;
	    case 2:
		// do nothing, wait for fault to fully activate
		runmode++;
		break;
	    case 3:
		// do nothing, wait for fault to fully activate
		runmode++;
		break;
	    case 4:
		// make sure it's not less than 90%
		lossRate = 
		   model.lookupDouble(routing_table.node_id_to_guid(otherNode),
					       PatchworkModel.LOSS );
		if( lossRate < 80 ) {
		    System.err.println( "Loss rate is " + lossRate +
					"% after a 100% fault.  Exiting." );
		    System.exit(-1);
		} else {
		    System.err.println( "Loss rate is good after 100% " +
					" faults.  Injecting 50% fault." );
		    PatchworkFaultMsg pfm = new PatchworkFaultMsg( otherNode,
								   50, true );
		    runmode++;
		    dispatch( pfm );
		}
		break;
	    case 5:
		// do nothing, wait for fault to fully activate
		runmode++;
		break;
	    case 6:
		// do nothing, wait for fault to fully activate
		runmode++;
		break;
	    case 7:
		// make sure it's between 25% and 75%
		lossRate =
		   model.lookupDouble(routing_table.node_id_to_guid(otherNode),
					       PatchworkModel.LOSS );
		if( lossRate < 25 || lossRate > 75 ) {
		    System.err.println( "Loss rate is " + lossRate +
					"% after a 50% fault.  Exiting." );
		    System.exit(-1);
		} else {
		    System.err.println( "Loss rate is good after 50% " +
					" faults." );
		    System.err.println( "All tests successful!" );
		    System.exit(0);
		}
	    }


        } else if( element instanceof RoutingTable.Available ) {
	
	    RoutingTable.Available rta = (RoutingTable.Available) element;
	    routing_table = rta.table;
	    digits_per_guid = routing_table.digits_per_guid();
	    digit_values = routing_table.digit_values();

	    // find the first node that's not me:
FIRSTFOR:   for( int i = 0; i < digits_per_guid; i++ ) {
		for( int j = 0; j < digit_values; j++ ) {	    
		    otherNode = routing_table.read( i, j );
		    if( (otherNode != null) && 
			!(otherNode.equals(self_node_id)) ) {
			break FIRSTFOR;
		    }
		}
	    }

	} else {

	  Carp.error( "Dropping unexpected message: " + element );

	}

    }

    /** 
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void init(ConfigDataIF config) throws Exception
    {

	self_node_id = new NodeId (config.getString ("node_id"));

	//////////////////
	// initialize filters

	classifier = Classifier.getClassifier( 
			new NodeId( config.getString( "node_id" ) ) );
	my_sink = config.getStage().getSink();
	Filter filter;

	// Subscribe to all my favorite events

	String[] eventNames = {
	    "seda.sandStorm.api.StagesInitializedSignal",	
	    "ostore.tapestry.impl.RoutingTable$Available",
	    "ostore.tapestry.patchwork.PatchworkDeltaResponseMsg"
	};

	for( int i = 0; i < eventNames.length; i++ ) {
	  filter = new Filter();
	  Class eventClass;
          try {
            eventClass = Class.forName( eventNames[i] );
	    if( !filter.requireType( eventClass ) || 
	        !classifier.subscribe( filter, my_sink ) ) {

	      Carp.die( "Could not require class " + eventNames[i] ); 

	    } 

          } catch ( ClassNotFoundException cnfe ) {
            Carp.die( cnfe );
          }

        }

    }

    /**
     * An internal method used for dispatching events
     */
    private void dispatch( QueueElementIF event ) {
                                                
      try {
        classifier.dispatch( event );
      } catch ( SinkFullException sfe ) {
        //TODO: Better way to deal with SinkFull
        try {
          Thread.currentThread().sleep( 5000 );
        } catch( InterruptedException ie ) {
          //ignore
        }
      }                                         
      
    }

    /** 
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void destroy()
    {
    }

}
