/**
 * RepairMonitorStage.java
 *
 * @author	Jeremy Stribling
 * @version	$Id: RepairMonitorStage.java,v 1.43 2003/05/26 21:28:06 strib Exp $
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ostore.tapestry.patchwork;

import ostore.apps.gnutella_trace.GnutellaPDF;
import ostore.apps.gnutella_trace.GnutellaHourPDF;
import ostore.util.InvalidConfigFileException;
import ostore.util.Carp;
import ostore.util.Debug;
import ostore.util.DebugFlags;
import ostore.util.StatRecorder;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.network.NetworkMessage;
import ostore.network.patchwork.InjectFaultMsg;
import ostore.tapestry.api.TapestryPublishMsg;
import ostore.tapestry.impl.FullTestMsg;
import ostore.tapestry.impl.Router;
import ostore.tapestry.impl.RoutingTable;
import ostore.tapestry.impl.TapestryVerifier;
import ostore.tapestry.impl.TapestryReadyMsg;
import ostore.tapestry.impl.VerifyRespMsg;
import ostore.util.NodeId;
import ostore.util.Clock;
import ostore.util.TypeTable;
import ostore.util.QuickSerializable;
import ostore.util.QSException;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.SecureHash;
import ostore.util.SHA1Hash;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.UnknownEventException;
import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SingleThreadedEventHandlerIF;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.api.SinkException;   
import seda.sandStorm.api.SinkFullException;   
import seda.sandStorm.api.StagesInitializedSignal; 
import seda.sandStorm.core.BufferElement;
import seda.sandStorm.lib.aDisk.AFile;
import seda.sandStorm.lib.aDisk.AFileWriteRequest;
import seda.sandStorm.lib.aDisk.AFileIOCompleted;
import seda.sandStorm.main.SandstormConfig;
import java.io.FileWriter;
import java.lang.reflect.Constructor;
import java.util.TreeMap;
import java.util.Iterator;
import java.util.Random;
import java.util.LinkedList;
import java.util.StringTokenizer;
import soss.core.BuilderIF;

/**
 * @author 	Jeremy Stribling
 * @version	$Id: RepairMonitorStage.java,v 1.43 2003/05/26 21:28:06 strib Exp $
 */  
public class RepairMonitorStage 
    implements SingleThreadedEventHandlerIF, EventHandlerIF
{

    private static final boolean DEBUG = true;

    private boolean sim_running;
    private boolean fail_immed;
    private boolean allow_parallel;
    private boolean use_old_guid;
    private boolean all_nodes;
    private Random my_rng;
    private PDF pdf;
    private CDF cdf;
    private PDF life_pdf;
    private CDF life_cdf;
    private PDF cluster_pdf;
    private CDF cluster_cdf;
    private PDF cluster_life_pdf;
    private CDF cluster_life_cdf;
    private int num_failures;
    private int actual_successes;
    private int failures_dispatched;
    private NodeId self_node_id;
    private long time_to_run;
    private boolean timed;
    private long repair_timeout;
    private boolean randomized_fail_pdf = false;
    private boolean randomized_life_pdf = false;
    private boolean periodic_bw;
    private int fail_start_time;
    private long fail_stop_time;
    private int start_time;
    private final static int PERIODIC_BW_PERIOD = 15000;

    private double failrate;
    private double liferate;
    private int failinterval;
    private int lifeinterval;

    private Classifier classifier;
    private SinkIF my_sink;

    // how often to verify Tapestry (1 hour)
    private static final long VERIFY_INTERVAL = 3600000;
    private boolean verify_tapestry;
    private boolean verifying;
    private int num_verify;
    private TapestryVerifier verifier;
    private Object verify_timeout;

    // the messages to watch for
    private String messages[];

    private String _result_log_filename;
    private boolean use_log;
    private AFile  _result_log_fd;

    private long time_started;

    // AIO file stuff
    private boolean ending = false;
    private AFileWriteRequest last;

    // the percentage of ptr_deleted message to expect
    private double PERCENT_TO_FAILURE = .70;

    // per hop stat recorders
    private StatRecorder perHopStats[];

    // data structures
    private TreeMap node_infos;  // NodeId -> NodeInfo
    private LinkedList undeletables; // NodeInfos
    private LinkedList failed_nodes; // NodeInfos
    private LinkedList detaching; // NodeInfos
    private LinkedList attaching; // NodeInfos
    private LinkedList waiting_to_detach; // NodeInfos
    private LinkedList waiting_to_attach; // NodeInfos
    private LinkedList waiting_to_fail; // NodeInfos
    private LinkedList clusters; // ClusterInfos

    // the file containing the name of the template file for new nodes
    private String node_cfg_file;
    // the command to configure a new node
    private String cfg_cmd;

    /**
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void handleEvents(QueueElementIF elemarr[]) 
	throws EventHandlerException
    {
	for( int i=0; i<elemarr.length; i++ ) {
	    handleEvent( elemarr[i] );
	}
    }

    /** 
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void handleEvent(QueueElementIF element) 
	throws EventHandlerException
    {
      try
      {
	if( element instanceof StagesInitializedSignal ) {
	    
	    handle_init();

	} else if( element instanceof TapestryReadyMsg ) {
	    
	    handle_ready( (TapestryReadyMsg) element );

	} else if( element instanceof RepairAlarmMsg ) {

	    handle_alarm( (RepairAlarmMsg) element );

	} else if( element instanceof RMLocateMsg ) {

	    handle_locate( (RMLocateMsg) element );

	} else if( element instanceof PatchworkRevCountMsg ) {

	    handle_rev_count( (PatchworkRevCountMsg) element );

	} else if( element instanceof PtrDeletedMsg ) {

	    handle_ptr_deleted( (PtrDeletedMsg) element );

	} else if( element instanceof AttachMessage ) {

	    handle_attach( (AttachMessage) element );

	} else if( element instanceof DetachMessage ) {

	    handle_detach( (DetachMessage) element );

	} else if( element instanceof VerifyRespMsg ) {

	    handle_verify_resp( (VerifyRespMsg) element );

	} else if (element instanceof AFileIOCompleted) {

	    handle_file_io_completed((AFileIOCompleted) element);

	} else {

	  Carp.error( "Dropping unexpected message: " + element );

	}
      } catch(Exception e) {
	  Carp.dieGracefully(e, self_node_id);
      }

    }

    private void handle_init() {

	if( sim_running ) {
	    soss.network.Network.instance().clear_byte_counts();
	}
	if( periodic_bw ) {
	    dispatch_later( new RepairAlarmMsg(RepairAlarmMsg.PERIODIC_BW),
			    PERIODIC_BW_PERIOD );
	}
    }

    /**
     * This means Tapestry is up and ready to go.
     * Basically, all there is to do here is to publish ourselves as 
     * the RepairMonitor, and wait for the LocateMsgs roll in
     */
    private void handle_ready( TapestryReadyMsg msg ) {

	time_started = Clock.current_date(self_node_id).getTime();
	// Increase size by one because we start index at one.
	// that is any node zero hop/level away is selfnode.
	perHopStats = new StatRecorder[ RoutingTable.DIGITS_PER_GUID+1 ];
	for( int i = 0; i < perHopStats.length; i++ ) {
	    perHopStats[i] = new StatRecorder( "Hop " + i + " stats" );
	}
	undeletables = new LinkedList();
	failed_nodes = new LinkedList();
	detaching = new LinkedList();
	attaching = new LinkedList();
	waiting_to_detach = new LinkedList();
	waiting_to_attach = new LinkedList();
	waiting_to_fail = new LinkedList();
	clusters = new LinkedList();

	SecureHash rmHash = new SHA1Hash( "RepairMonitor" );
	dispatch( new TapestryPublishMsg( rmHash, null ) );

	// set an alarm, so that we'll start making failures 
	// after things settle down a bit.
	// if it's all zeros, don't even bother
	start_time = fail_start_time;
	if( start_time < 0 ) {
	    start_time = 60000;
	}
	// adjust the stop time to be relative to the starting time
	if( fail_stop_time < Long.MAX_VALUE ) {
	    fail_stop_time += time_started + start_time;
	}
	if( !(pdf instanceof AllZerosPDF) ) {
	    RepairAlarmMsg rm = 
		new RepairAlarmMsg( RepairAlarmMsg.GENERATE_FAILURES );
	    dispatch_later( rm, start_time );
	}
	if( !(cluster_pdf instanceof AllZerosPDF) ) {
	    RepairAlarmMsg rm = 
		new RepairAlarmMsg( RepairAlarmMsg.GENERATE_CORR_FAILURES );
	    dispatch_later( rm, start_time );
	}
	// also, if we're Tapestry verifying, set the first alarm.
	if( verify_tapestry ) {
	    dispatch_later( new RepairAlarmMsg(RepairAlarmMsg.VERIFY_TAPESTRY),
			    VERIFY_INTERVAL );
	}

    }

    /**
     * When we get an alarm, we need to see if any failures are to be
     * generated, given that we haven't reached a timeout or already
     * sent out all the failures.
     */
    private void handle_alarm( RepairAlarmMsg msg ) {

	// check the timeout first off
	if( timed && ( Clock.current_date(self_node_id).getTime() -
			time_started ) >= time_to_run ) {
	    last = writeToLog( time_to_run + " time is up.  Exiting.\n" );
	    updateSuccesses(true);
	    // just exit if we're not waiting for a log response
	    if( !use_log ) {
		System.exit(0);
	    }
	    return;
	}

	// check the type:
	switch( msg.type ) {
	case RepairAlarmMsg.GENERATE_FAILURES:
	    generate_failures_and_fixes();
	    break;
	case RepairAlarmMsg.GENERATE_CORR_FAILURES:
	    generate_corr_failures();
	    break;
	case RepairAlarmMsg.REPAIR_TIMEOUT:
	    handle_repair_timeout( msg.failed_node );
	    break;
	case RepairAlarmMsg.VERIFY_TAPESTRY:
	    handle_verify_tapestry();
	    break;
	case RepairAlarmMsg.PERIODIC_BW:
	    handle_periodic_bw();
	    break;
	case RepairAlarmMsg.VERIFY_TIMEOUT:
	    handle_verify_timeout();
	    break;
	default:
	    printMsg( "Unknown alarm type: " + msg.type );
	}

    }

    /**
     * Flip a biased coin for every node, based on their life and the CDF,
     * and see if anyone needs to fail.  Also can bring detached nodes back,
     * depending on the policy.
     */
    private void generate_failures_and_fixes() {

	if( failures_dispatched < num_failures && 
	    Clock.current_date(self_node_id).getTime() < fail_stop_time ) {

	    // get the list of all deletable nodes
	    Iterator deletable = node_infos.values().iterator();
	    while( deletable.hasNext() ) {

		NodeInfo curr_node = (NodeInfo) deletable.next();

		// if the node hasn't already failed
		if( !curr_node.failed ) {
		    // then see if it's supposed to fail now
		    if( curr_node.rng.nextDouble() < 
			getFailThreshold( pdf, cdf, curr_node.life ) ) {

			// send it the first round failure info
			fail_first_round( curr_node );

			// if we're failing nodes immediately, 
			// go into the second round
			if( fail_immed ) {
			    fail_second_round( curr_node );
			}

		    }

		} else {

		    // if the node has failed and detached, we might want
		    // to bring it back to life.
		    if( curr_node.detached && 
			(curr_node.rng.nextDouble() < 
			 getFailThreshold( life_pdf, life_cdf, 
					   curr_node.life ) ) ) {

			// bring back to life somehow
			reanimate( curr_node );

		    }

		}

		// increase this node's current life 
		// (but not more than the max)
		if( curr_node.life + 1 < pdf.getPDFSize()) {
		    curr_node.life++;
		}

	    }

	}

	
	RepairAlarmMsg ram = 
	    new RepairAlarmMsg(RepairAlarmMsg.GENERATE_FAILURES);
	// HACK for making churn stop and then start again with
	// different parameters:
	if( Clock.current_date(self_node_id).getTime() >= fail_stop_time ) {
	    fail_stop_time += fail_stop_time + (start_time-time_started) - 
		(time_started+start_time);
	    // double the rate, and halve the lifetime
	    makeProbPDF( (ProbPDF) pdf, failrate*2.0, failinterval/2 );
	    makeProbPDF( (ProbPDF) life_pdf, liferate*2.0, lifeinterval );
	    printMsg( "Restarting churn in " + (start_time-time_started) );
	    // move everyone to the undeletable list
	    // get the list of all deletable nodes
	    Iterator deletable = node_infos.values().iterator();
	    while( deletable.hasNext() ) {
		NodeInfo curr_node = (NodeInfo) deletable.next();
		if( !curr_node.failed ) {
		    undeletables.add( deletable.next() );
		    deletable.remove();
		}
	    }
	    dispatch_later( ram, (start_time-time_started) );
	    return;
	}
	// end HACK
	
	// set the next alarm
	// note that this should be set even if all failures have 
	// been dispatched, since we need to check the timeout still
	dispatch_later( ram, (long) cdf.getCDFIncrement() );

    }

    /**
     * Flip a biased coin for every cluster, based on their life and the CDF,
     * and see if any cluster needs to fail.
     */
    private void generate_corr_failures() {

	if( failures_dispatched < num_failures ) {

	    for( int i = 0; i < clusters.size(); i++ ) {
		ClusterInfo ci = (ClusterInfo) clusters.get(i);
		// if the cluster hasn't already failed
		if( !ci.failed ) {
		    // then see if it's supposed to fail now
		    if( ci.rng.nextDouble() < 
			getFailThreshold( cluster_pdf, cluster_cdf, 
					  ci.life ) ) {

			printMsg( "Failing cluster " + ci );

			// the cluster failed, so let's fail each node in it

			for( int j = 0; j < ci.node_infos.size(); j++ ) {

			    NodeInfo curr_node = 
				(NodeInfo) ci.node_infos.get(j);

			    // if it's already failed, skip it
			    if( curr_node.failed ) {
				continue;
			    }

			    // send it the first round failure info
			    fail_first_round( curr_node );
			    
			    // if we're failing nodes immediately, 
			    // go into the second round
			    if( fail_immed ) {
				fail_second_round( curr_node );
			    }
			}
			
			// now that it's failed, rmeove it from the list
			ci.failed = true;
			clusters.remove(i);
			// subtract one from i because of the remove
			i--;

		    }
		}

		// increase this node's current life 
		// (but not more than the max)
		if( ci.life + 1 < cluster_pdf.getPDFSize()) {
		    ci.life++;
		}

	    }

	}

	// set the next alarm
	RepairAlarmMsg rm = 
	    new RepairAlarmMsg( RepairAlarmMsg.GENERATE_CORR_FAILURES );
	dispatch_later( rm, (long) cluster_cdf.getCDFIncrement() );

    }

    /**
     * Start the first phase of failure, which informs the node it is failing,
     * but does not actually have it detach yet.
     */
    private void fail_first_round( NodeInfo failed_node ) {

	// set it as failed now so it can't be failed twice
	failed_node.failed = true;

	// if we're not allowing for parallelism, don't even let
	// a simulated fault happen if there is already one in the system
	if( !allow_parallel && !failed_nodes.isEmpty() ) {
	    waiting_to_fail.add( failed_node );
	    return;
	}

	// send out an "all nodes" fault message to the node
	InjectFaultMsg pfm = new InjectFaultMsg(failed_node.node_id, 
						100, true, true );
	failures_dispatched++;
	// set the initial ptrs_remaining to something huge, so that
	// failures that show up before the RevCount comes back are
	// useful
	failed_node.ptrs_remaining = 1000;
	failed_node.failure_time = Clock.current_date(self_node_id).getTime();

	// if we're going to time it out, set the timer now
	if( repair_timeout >= 0 ) {
	    failed_node.timeout_token = 
		dispatch_later( new RepairAlarmMsg(failed_node), 
				repair_timeout );
	}

	failed_nodes.add( failed_node );
	dispatch( pfm );
	printMsg( "Dispatching failure to " + failed_node + 
		  ", it was less than " + cdf.getCDFEntry(failed_node.life ) + 
		  " at life " + failed_node.life + " at time " +
		  failed_node.failure_time );
	    
    }

    /**
     * Start the second phase of failure, which informs the node to detach.
     */
    private void fail_second_round( NodeInfo failed_node ) {

	// if we allow parallelism, or there is no one else attaching or
	// detaching, then tell this node to fail right away
	// also put on the wait list if we're verifying
	if( !verifying && 
	    (allow_parallel || (attaching.isEmpty() && detaching.isEmpty()))) {

	    make_detach( failed_node );

	} else {

	    // otherwise, put it in line for detaching
	    waiting_to_detach.add( failed_node );

	}

    }

    /**
     * Tell a node it should detach from Tapestry forthwith
     */
    private void make_detach( NodeInfo failed_node ) {

	// send out an "all nodes" fault message to the node
	DetachMessage dm = new DetachMessage(failed_node.node_id, true);
	printMsg( "Telling " + failed_node + " to detach." );
	detaching.add( failed_node );
	dispatch( dm );

    }

    /**
     * Begins the process of telling a node to attach
     */
    private void reanimate( NodeInfo birth_node ) {

	// set this to false, otherwise it may try to reattach several times
	birth_node.detached = false;

	// if we allow parallelism, or there is no one else attaching or
	// detaching, then tell this node to attach right away
	// also put on the wait list if we're verifying
	if( !verifying && 
	    (allow_parallel || (attaching.isEmpty() && detaching.isEmpty()))) {

	    make_attach( birth_node );

	} else {

	    // otherwise, put it in line for detaching
	    waiting_to_attach.add( birth_node );

	}	

    }

    /**
     * Tell a node to attach itself to Tapestry
     */
    private void make_attach( NodeInfo new_node ) {

	if( node_cfg_file == null ) {
	    printMsg( "WARNING: node_cfg_file was null, so not attaching " +
		      "new node " + new_node );
	    return;
	}

	String command_env[];

	// look for remote ostore user and run dir
	if( !sim_running ) {
	    StringTokenizer st = new StringTokenizer( cfg_cmd, ";" );
	    if( st.countTokens() != 6 ) {
		Carp.die( "Wrong number of semicolons in command: " + 
			  cfg_cmd + ", says there are " + st.countTokens() );
	    }
	    String first = st.nextToken();
	    String second = st.nextToken();
	    String third = st.nextToken();
	    String fourth = st.nextToken();

	    int findex = first.indexOf( "REMOTE" );
	    String remoteuser = first.substring( findex );
	    int sindex = second.indexOf( "SSH" );
	    String sshagent = second.substring( sindex );
	    int tindex = third.indexOf( "SSH" );
	    String sshauth = third.substring( tindex );
	    int foindex = fourth.indexOf( "REMOTE" );
	    String remoterun = fourth.substring( foindex );

	    printMsg( "export variables: " + remoteuser + " and " +
		      remoterun + " and " + sshagent + " and " + 
		      sshauth );

	    // construct a command
	    String cenv[] = {"NODENAME=NodeX", 
			     "CFGNAMEFILE=" + node_cfg_file,
			     "HOSTNAME=" + 
			     new_node.node_id.address().getHostAddress(),
			     "PORT=" + new_node.node_id.port(),
			     remoteuser, remoterun, sshagent, sshauth};
	    command_env = cenv;

	} else {
	    // construct a command
	    String cenv[] = {"NODENAME=NodeX", 
			     "CFGNAMEFILE=" + node_cfg_file,
			     "HOSTNAME=" + 
			     new_node.node_id.address().getHostAddress(),
			     "PORT=" + new_node.node_id.port()};
	    command_env = cenv;
	}

	String suffix;
	String ucfg = node_cfg_file + "." + System.currentTimeMillis();
	if( sim_running ) {
	    // in simulation, we just want the cfg file, that's all
	    suffix = " > " + ucfg;
	} else {
	    // not in simulation, then don't wait
	    suffix = " &";
	}

	String command = cfg_cmd + suffix;
	printMsg( "Executing command: " + command );
	String executor_file = ucfg + ".executor"; 

	// make a system call to execute
	try {
	    // we need to actually write a script on the fly here to execute
	    // this, because Java process don't understand I/O redirection
	    FileWriter executor = new FileWriter( ucfg + ".executor" );	    
	    executor.write( "#! /bin/sh\n" );
	    executor.write( command + "\n" );
	    executor.flush();
	    executor.close();
	    Process proc = 
		Runtime.getRuntime().exec( "chmod a+x " + executor_file );
	    proc.waitFor();
	    proc = Runtime.getRuntime().exec( executor_file, command_env );
	    proc.waitFor();
	    proc = Runtime.getRuntime().exec( "rm -f " + executor_file );
	    proc.waitFor();
	} catch( Exception e ) {
	    Carp.die( "Got exception " + e + " while trying to execute: " +
		      command );
	}

	// if we're not running under simulation, no need to wait

	if( sim_running ) {
	    // just pass the cfg file into soss and let it do the work
	    soss.core.Main.createNode( new_node.node_id, ucfg );
	    try {
		(new java.io.File( ucfg )).delete();
	    } catch( Exception e ) {
		Carp.die( "Couldn't delete ucfg " + ucfg );
	    }
	}
	
	/**
	AttachMessage am = new AttachMessage( new_node.node_id, 
					      !use_old_guid );
	attaching.add( new_node );
	printMsg( "Telling " + new_node + " to attach itself." );
	dispatch( am );
	**/

    }


    /**
     * Return the failure threshold for a particular pdf/cdf set,
     * given an amount of time the node has been in a given state.
     */
    private double getFailThreshold( PDF pdf, CDF cdf, int life ) {

	return (pdf.getPDFEntry( life ) / (1.0 - cdf.getCDFEntry( life )));

    }
    
    /**
     * Someone else wants to subscribe to this monitor.
     * Add them to our data structures, if they want to be monitored.
     * Also, send them our address so they can talk to us directly.
     */
    private void handle_locate( RMLocateMsg msg ) {

	printMsg( "Got LocateMsg from " + msg.src + 
		  ", deletable=" + msg.deletable + " at time " + 
		  Clock.current_date(self_node_id).getTime());

	// if they aren't deletable or, in the case of a churn,  
	// they arrive after the churn starts
	if( msg.deletable &&
	    Clock.current_date(self_node_id).getTime() > fail_start_time ) {

	    // we'll store the guid and the node id
	    // also, give the node a random number generator, and a life
	    NodeInfo new_node = new NodeInfo( msg.src, msg.srcGuid );
	    if( !randomized_fail_pdf ) {
		new_node.setInitialLife();
	    }
	    // make sure we don't know about this person yet
	    if( node_infos.get( msg.src ) != null ) {
		// print a warning
		printMsg( "An entry for " + msg.src + 
			  " already exists; overwriting." );
	    }

	    node_infos.put( msg.src, new_node );
	    // add it to a cluster list
	    boolean added = false;
	    for( int i = 0; i < clusters.size(); i++ ) {
		ClusterInfo ci = (ClusterInfo) clusters.get(i);
		if( ci.addNode( new_node ) ) {
		    added = true;
		    printMsg( "Added " + new_node + " to existing cluster: "
			      + ci );
		    break;
		}
	    }
	    // if it didn't match any cluster, make a new one
	    if( !added ) {
		ClusterInfo ci = new ClusterInfo();
		ci.setInitialLife();
		ci.addNode( new_node );
		clusters.add( ci );
		printMsg( "Made new cluster for " + new_node );
	    }

	} else {

	    NodeInfo new_node = new NodeInfo( msg.src, msg.srcGuid );
	    undeletables.add( new_node );
	    // do not delete or start this guy, ever.
	    node_infos.remove( msg.src );

	}

	// now respond, and tell them your name
	dispatch( new RMFoundMsg( msg.src ) );

    }

    /**
     * A node that has been instructed to fail responded to us with the
     * number of other nodes that point to it.  Make a note of it.
     * If no one points to it, you might as well just fail right away.
     */
    private void handle_rev_count( PatchworkRevCountMsg msg ) {

	// make sure the node is really failed 
	// (don't look in undeletables, that would be dumb)
	NodeInfo ninfo = (NodeInfo) node_infos.get( msg.peer );
	if( ninfo == null && (ninfo = findInFailed( msg.peer )) == null ) {
	    Carp.die( "RevCount message from a node " + msg.peer + 
		      " for which there is no info." );
	} else if( !ninfo.failed || !failed_nodes.contains( ninfo ) ) {
	    Carp.die( "RevCount message from a node " + ninfo + 
		      " which seems not to be failed." );
	}

	ninfo.ptrs_remaining = (int) (msg.num_ptrs*PERCENT_TO_FAILURE);
	
	printMsg( msg.num_ptrs + " pointers for node " + ninfo );
	
	// for now, just fake a really big # of ptrs for
	// the repair test, and let the timeout take care of it
	if( repair_timeout >= 0 && msg.num_ptrs <= 0 ) {
	    ninfo.ptrs_remaining = 100;
	}

	// if there are no ptrs, consider it repaired right away
	if( ninfo.ptrs_remaining == 0 ) {
	    failure_finished( ninfo );
	}

    }

    /**
     * A node has been detected as failed by someone else.  Update 
     * statistics and if that's everyone, consider it detected.
     */
    private void handle_ptr_deleted( PtrDeletedMsg msg ) {

	// first look up the info for the failed node
	NodeInfo ninfo = null;
	for( int i = 0; i < failed_nodes.size(); i++ ) {
	    NodeInfo currNinfo = (NodeInfo) failed_nodes.get(i);
	    if( (msg.badGuyGuid != null && 
		 msg.badGuyGuid.equals( currNinfo.guid )) ||
		(msg.badGuyNode != null &&
		 msg.badGuyNode.equals( currNinfo.node_id )) ) {
		ninfo = currNinfo;
		break;
	    }
	}

	NodeInfo peer_ninfo = findNodeInfo( msg.peer );

	if( peer_ninfo == null ) {
	    printMsg( "Could not find NodeInfo for " + msg.peer +
		      ".  Ignoring." );
	    return;
	}

	// if no hopCount was specified, we should be able to figure it out?
	if( msg.hopCount < 0 && ninfo != null && ninfo.guid != null ) {
	    msg.hopCount = Router.guidCompare( peer_ninfo.guid, 
					       ninfo.guid );
	} else if( ninfo != null && ninfo.guid == null ) {
	    msg.hopCount = 0;
	} else if( msg.hopCount < 0 && msg.badGuyGuid != null ) {
	    msg.hopCount = Router.guidCompare( peer_ninfo.guid, 
					       msg.badGuyGuid );
	}

	// if we couldn't find the info in failed_nodes, then we
	// didn't cause this failure, so just ignore.
	if( ninfo == null ) {

	    // try to figure out the guid if we don't have it
	    if( msg.badGuyNode != null ) {

		ninfo = findNodeInfo( msg.badGuyNode );
		
		if( msg.hopCount < 0 && msg.badGuyGuid == null && 
		    ninfo != null ) {
		    msg.hopCount = Router.guidCompare( peer_ninfo.guid, 
						       ninfo.guid );
		}
	    }

	    writeToLog( "UNEXFAILURE: Could not find failure info for node " +
			(ninfo != null ? ninfo.toString() : 
			 ("(" + msg.badGuyNode + "/" + msg.badGuyGuid +")")) + 
			", although " + peer_ninfo + 
			" sent a PtrDeletedMsg at time " + 
			Clock.current_date(self_node_id).getTime() + 
			" with hops " + msg.hopCount + ".  Ignoring.\n" );
	    return;
	}

	long timeDone = Clock.current_date(self_node_id).getTime() -
	    ninfo.failure_time;
	ninfo.ptrs_remaining--;

	// notice that we don't record stats, even on a per hop basis,
	// for nodes that are done failing.	
	printMsg( "pointer to " + ninfo + " at hop/level " + msg.hopCount + 
		  " deleted by " + 
		  ( peer_ninfo == null ? msg.peer.toString() : 
		                         peer_ninfo.toString() ) );
	// now add the stats
	ninfo.stats.add(timeDone);
	perHopStats[msg.hopCount].add(timeDone);

	// and if we've reached 0, finish off this failure
	if( ninfo.ptrs_remaining <= 0 ) {
	    failure_finished( ninfo );
	} 

    }

    private NodeInfo findNodeInfo( NodeId node_id ) {

	NodeInfo ninfo = (NodeInfo) node_infos.get( node_id );
	
	if( ninfo == null ) {
	    // maybe it's an undeletable:
	    for( int i = 0; i < undeletables.size(); i++) {
		NodeInfo currNinfo = (NodeInfo) undeletables.get(i);
		if( currNinfo.node_id.equals( node_id ) ) {
		    ninfo = currNinfo;
		    break;
		}
	    }
	}

	return ninfo;

    }

    private NodeInfo findInFailed( NodeId node_id ) {
	
	for( int i = 0; i < failed_nodes.size(); i++ ) {
	    NodeInfo currNinfo = (NodeInfo) failed_nodes.get(i);
	    if( sim_running && currNinfo.node_id.port() == node_id.port() ) {
		return currNinfo;
	    } else if( currNinfo.node_id.equals( node_id ) ) {
		return currNinfo;
	    }
	}

	return null;

    }

    private void handle_attach( AttachMessage msg ) {

	// first, make sure we were aware that this node was attaching
	NodeInfo ninfo = null;
	for( int i = 0; i < attaching.size(); i++ ) {
	    NodeInfo currNinfo = (NodeInfo) attaching.get(i);
	    if( msg.peer.equals( currNinfo.node_id ) ) {
		ninfo = currNinfo;
		break;
	    }
	}

	// if no one we know, ignore
	if( ninfo == null ) {
	    printMsg( "WARNING! Got an AttachMessage from " + msg.peer + 
		      ", but we weren't expecting it.  Ignoring." );
	    return;
	}

	// remove from attaching
	attaching.remove( ninfo );

	// now we must make a new NodeInfo for this node, using the new
	// guid, but an initial life of 0.  We can replace the old one
	// safely, because even if fail_immed is on and other nodes are
	// still detecting it's deletion, the old NodeInfo remains in
	// the failed_nodes list.
	NodeInfo new_info = new NodeInfo( msg.peer, msg.myGuid );
	node_infos.put( msg.peer, new_info );

	printMsg( "Got AttachMessage for " + new_info );

	// start off the next in line
	// NOTE: make sure to switch to detach, so attach doesn't get a 
	// monopoly
	if( !waiting_to_detach.isEmpty() ) {
	    make_detach( (NodeInfo) waiting_to_detach.getFirst() );
	    waiting_to_detach.remove( waiting_to_detach.getFirst() );
	} else if( !waiting_to_attach.isEmpty() ) {
	    make_attach( (NodeInfo) waiting_to_attach.getFirst() );
	    waiting_to_attach.remove( waiting_to_attach.getFirst() );
	}

    }

    /**
     * Called when a detaching node has finished detaching, and is no longer
     * particpating in Tapestry.  Cleans up data structures for the node,
     * and handles other bookkeeping such as telling another node to attach
     * or detach.
     */
    private void handle_detach( DetachMessage msg ) {

	// first, make sure we were aware that this node was detaching
	NodeInfo ninfo = null;
	for( int i = 0; i < detaching.size(); i++ ) {
	    NodeInfo currNinfo = (NodeInfo) detaching.get(i);
	    if( msg.peer.equals( currNinfo.node_id ) ) {
		ninfo = currNinfo;
		break;
	    }
	}

	// we don't take commands
	if( msg.command ) {
	    return;
	}

	// if no one we know, ignore
	if( ninfo == null ) {
	    printMsg( "WARNING! Got a DetachMessage from " + msg.peer + 
		      ", but we weren't expecting it.  Ignoring." );
	    return;
	}

	printMsg( ninfo + " finished detachment." );

	ninfo.detached = true;
	// set life to 0, so that the life_cdf can determine when it 
	// should reattach
	// only do this if its not all nodes though, so that lifes
	// of nodes that were and weren't alive match up
	if( !all_nodes ) { 
	    ninfo.life = 0;
	}
	detaching.remove( ninfo );
	
	// bring the node up with a different ip next time
	if( sim_running && !use_old_guid && all_nodes ) {
	    // increment the ip address by 1:
	    NodeId oldid = ninfo.node_id;
	    NodeId newid = null;
	    String idstring = ninfo.node_id.toString();
	    int colindex = idstring.indexOf(':');
	    int lastperindex = idstring.lastIndexOf(".");
	    try {
		int lastnum = 
		    Integer.parseInt( idstring.substring( lastperindex+1,
							  colindex ) );
		lastnum++;
		newid = new NodeId(idstring.substring(0,lastperindex+1) 
				   + lastnum + 
				   idstring.substring(colindex));
	    } catch( Exception e ) {
		Carp.die( "Exception reading integer from " + idstring );
	    }
	    ninfo.node_id = newid;
	    node_infos.remove( oldid );
	    node_infos.put( newid, ninfo );
	    printMsg( "Changed node id from " + oldid + " to " + newid );
	}

	// start off the next in line
	// NOTE: make sure to switch to attach, so detach doesn't get a 
	// monopoly
	if( !waiting_to_attach.isEmpty() ) {
	    make_attach( (NodeInfo) waiting_to_attach.getFirst() );
	    waiting_to_attach.remove( waiting_to_attach.getFirst() );
	} else if( !waiting_to_detach.isEmpty() ) {
	    make_detach( (NodeInfo) waiting_to_detach.getFirst() );
	    waiting_to_detach.remove( waiting_to_detach.getFirst() );
	}

	// now, we should reattach this node right away if the life_cdf tells 
	// us to.
	if( life_pdf.getPDFEntry(0) >= 1.0 ) {
	    reanimate( ninfo );
	} 

    }

    /**
     * A node failed, but the prerequisite # of nodes did not detect it as
     * failed before the timeout occured.
     */
    private void handle_repair_timeout( NodeInfo node ) {

	if( node == null ) {
	    printMsg( "WARNING! A null node was passed into " + 
		      "handle_repair_timeout.  Ignoring it." );
	    return;
	}

	printMsg( "Node " + node + " timed out, so we're considering it " +
		  "repaired." );
	// finish up
	node.timeout_token = null;
	failure_finished( node );
	
    }

    /**
     * Called when a nodes failure has been sufficiently detected, or
     * a timeout has occured.  Cleans up state, gathers and logs stats,
     * and makes the node detach if needed.
     */
    private void failure_finished( NodeInfo node ) {

	// first, remove from list of failed nodes.
	if( failed_nodes.contains( node ) ) {
	    failed_nodes.remove( node );
	} else {
	    printMsg( "WARNING! Failed node " + node + " not found in " +
		      "list of failed nodes.  Ignoring." );
	    return;
	}

	// next, cancel the repair timeout, if it hasn't happened yet
	if( node.timeout_token != null ) {
	    classifier.cancel( node.timeout_token );
	}

	// now, write this node's stats to the log
	double mean = node.stats.getMean();
	if( mean != 0.0 ) { 
	    writeToLog( "REPAIR " + mean + 
			" " + node.guid + "\n" );
	    updateSuccesses();
	} else {
	    // take away one of the failures, because no one ever
	    // detected this guy, so it didn't count
	    failures_dispatched--;
	}

	// finally, if the node needs to be detached still, go ahead.
	if( !fail_immed ) {
	    fail_second_round( node );
	}

	// if someone else is waiting to fail, do it
	if( !allow_parallel && !waiting_to_fail.isEmpty() ) {
	    fail_first_round( (NodeInfo) waiting_to_fail.getFirst() );
	    waiting_to_fail.remove( waiting_to_fail.getFirst() );
	}

    }

    /**
     * If we're writing to an internal log, we'll be getting these
     * IO confirmations back.  When we get our last one back, we'll know
     * it's finally time to exit.
     */
    private void handle_file_io_completed( AFileIOCompleted msg ) {

	if( ending ) {
	    if( last == msg.getRequest() ) {
		System.exit(0);
	    }
	}

    }

    /**
     * Print out bandwidth numbers.
     */
    private void handle_periodic_bw() {

	if( sim_running ) {
	    System.out.println( "BANDWIDTH START " +
				Clock.current_date(self_node_id).getTime() );
	    soss.network.Network.instance().print_byte_counts();
	    soss.network.Network.instance().clear_byte_counts();
	    System.out.println( "BANDWIDTH STOP" );
	}
	dispatch_later( new RepairAlarmMsg(RepairAlarmMsg.PERIODIC_BW),
			PERIODIC_BW_PERIOD );
    }

    /**
     * Request routing info from all active nodes.  If there are fluctuations
     * in progress though, wait a few seconds then try again.
     */
    private void handle_verify_tapestry() {

	// if no one is coming or going, grab the lock
	// else, wait 10 seconds
	if( attaching.isEmpty() && detaching.isEmpty() ) {
	    verifying = true;
	} else {
	    // try again
	    dispatch_later(new RepairAlarmMsg(RepairAlarmMsg.VERIFY_TAPESTRY),
			    10000);
	    return;
	}

	// send out requests for routing info to all nodes
	printMsg( "Beginning to verify Tapestry" );
	num_verify = 0;
	if( verifier == null ) {
	    verifier = new TapestryVerifier();
	} else {
	    verifier.reset();
	}
	SecureHash msg_id = new SHA1Hash( "no_id" );
	Iterator deletable = node_infos.values().iterator();
	while( deletable.hasNext() ) {
	    
	    NodeInfo curr_node = (NodeInfo) deletable.next();
	    
	    // if the node hasn't already detached (still in Tapestry)
	    if( !curr_node.detached ) {
		num_verify++;
		dispatch( new FullTestMsg( curr_node.node_id, 
					   FullTestMsg.VERIFY_REQUEST, msg_id,
					   0, null ) );
	    }
	}

	for( int i = 0; i < undeletables.size(); i++ ) {
	    num_verify++;
	    dispatch(new FullTestMsg(((NodeInfo) undeletables.get(i)).node_id, 
				     FullTestMsg.VERIFY_REQUEST, msg_id,
				     0, null));  
	}

	// set a timeout to occur after a reasonable amount of time has passed
	verify_timeout = 
	    dispatch_later( new RepairAlarmMsg(RepairAlarmMsg.VERIFY_TIMEOUT),
			    VERIFY_INTERVAL/10 );

    }

    /**
     * Stores the routing info for a node when in verification mode.
     * When all responses are in, tries to verify that Tapestry is correct
     */
    private void handle_verify_resp( VerifyRespMsg msg ) {

	// make sure we're in the right mode
	if( !verifying || num_verify <= 0 ) {
	    printMsg( "WARNING! Got unexpected VerifyResp from " +
		      msg.peer + ".  Ignoring." );
	    return;
	}

	printMsg( "Got VerifyResp from " + msg.peer );

	num_verify--;
	verifier.addData( msg.getSrc(), msg.getRoutingTable(),
			  msg.getRevpointers() );

	/*
	printMsg( "Verify info for: " + msg.getSrc() + "\n" + 
		  msg.getRoutingTable() + "\n");
	ostore.tapestry.impl.Router.dumpRevpointers( msg.getRevpointers(), 5 );
	printMsg( "-------------------------------------------\n" );
	*/

	// if we've got all the responses in . . .
	if( num_verify == 0 ) {

	    verify_timeout = null;
	    if( verifier.verify() ) {

		printMsg( "Verification passed!" );

	    } else {

		// we failed.  print out why.
		printMsg( "Verification failed!" );
		verifier.printErrors();

	    }

	    resume_from_verification();

	}

    }

    /**
     * A timeout occured during verification, which means we didn't get
     * all of the responses we were expecting in a timely manner.
     */
    private void handle_verify_timeout() {

	// make sure we're in the right mode
	if( !verifying ) {
	    printMsg( "WARNING! Got unexpected Verify timeout.  Ignoring." );
	    return;
	}

	printMsg( "Verification timed out.  Still waiting on " +
		  num_verify + " responses.  Going on with our life." );
	resume_from_verification();

    }

    /**
     * Starts up anyone who's waiting for verification to finish
     */
    private void resume_from_verification() {

	verifying = false;
	if( !waiting_to_detach.isEmpty() ) {
	    make_detach( (NodeInfo) waiting_to_detach.getFirst() );
	    waiting_to_detach.remove( waiting_to_detach.getFirst() );
	} else if( !waiting_to_attach.isEmpty() ) {
	    make_attach( (NodeInfo) waiting_to_attach.getFirst() );
	    waiting_to_attach.remove( waiting_to_attach.getFirst() );
	}

	if( verify_timeout != null ) {
	    classifier.cancel( verify_timeout );
	}
	// set the next alarm
	dispatch_later( new RepairAlarmMsg( RepairAlarmMsg.VERIFY_TAPESTRY ),
			VERIFY_INTERVAL );

    }

    private void updateSuccesses() {
	updateSuccesses( false );
    }

    /**
     * This is called either when a node has been completely detected as fails,
     * or a timeout has been reached.  Will terminate the test if all
     * successes have been detected.
     *
     * @param force   force the test to end, even if we haven't finished
     *                detecting (usually indicated the timeout)
     */
    private void updateSuccesses( boolean force ) {
	actual_successes++;
	if( force || (actual_successes == num_failures) ) {

	    writeToLog( "HOPS " );
	    for( int i = 0; i < perHopStats.length; i++ ) {
		writeToLog( ":" +
			    perHopStats[i].getNumSamples() + " " +
			    perHopStats[i].getMedian() + " " +
			    perHopStats[i].getPercentile(0.05) + " " +
			    perHopStats[i].getPercentile(0.95) + " " +
			    perHopStats[i].getMean() + " " +
			    perHopStats[i].getStddev() + " " +
			    perHopStats[i].getMin() + " " +
			    perHopStats[i].getMax() );
	    }
	    writeToLog( "\n" );

	    for( int i = 0; i < messages.length; i++ ) {
		// if this is a message that's DynamicTapestry related, 
		// print out DYNAMIC afterwards.
		String dynamic = " not";
		if( messages[i].indexOf( "Dynamic" ) != -1 ) {
		    dynamic = " DYNAMIC";
		}
		writeToLog( "TYPE: " + 
			    TypeTable.type_code( messages[i] ) + 
			    dynamic + "\n" );
	    }
	    if( sim_running ) {
		soss.network.Network.instance().print_byte_counts();
	    }
	    long timeSpent = Clock.current_date(self_node_id).getTime()
		- time_started;
	    last = writeToLog( "NEWLINE " + timeSpent + "\n" );

	    // if no simulation or no log, we'll have to exit now
	    if( !use_log && !sim_running ) {
		System.exit(0);
	    } else {
		ending = true;
	    }

	}
	if( sim_running ) { 
	    dispatch( new soss.core.SimulationFinishedEvent(0) );
	}
    }

    /**
     * If using a log, write asynchronously to the log
     * otherwise, print to System.out.
     */
    private AFileWriteRequest writeToLog( String result ) {

	if( use_log ) {

	    BufferElement     buffer = new BufferElement(result.getBytes(),
							 my_sink);
	    AFileWriteRequest write  = new AFileWriteRequest(buffer, my_sink);
	
	    try {
		_result_log_fd.enqueue(write);
	    }
	    catch (SinkException se) {
		System.err.println("WARNING! " +
				   "Sink exception while writing to log.");
	    }
	    return write;
	} else {
	    System.out.print( result );
	    return null;
	}

    }

    /** 
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void init(ConfigDataIF config) throws Exception
    {

	///////////////////
	// get config info
	sim_running = config.getBoolean( "simulator_running" );
	fail_immed = config.getBoolean( "fail_immed" );
	allow_parallel = config.getBoolean( "allow_parallel" );
	use_old_guid = config.getBoolean( "use_old_guid" );
	verify_tapestry = config.getBoolean( "verify_tapestry" );
	periodic_bw = config.getBoolean( "periodic_bw" );
	String random_str = config.getString( "random_seed" );
	String cdf_string = config.getString( "cdf_class" );
	String life_cdf_string = config.getString( "life_cdf_class" );
	String cluster_cdf_string = config.getString( "cluster_cdf_class" );
	String cluster_life_cdf_string = 
	    config.getString( "cluster_life_cdf_class" );
	node_cfg_file = config.getString( "node_cfg" );
	cfg_cmd = config.getString( "cfg_cmd" );
	num_failures = config.getInt( "num_failures" );
	self_node_id = new NodeId (config.getString ("node_id"));
	String tor = config.getString( "time_to_run" );
	String rt = config.getString( "repair_timeout" );
	failrate = config.getDouble( "failrate" );
	liferate = config.getDouble( "liferate" );
	failinterval = config.getInt( "failinterval" );
	lifeinterval = config.getInt( "lifeinterval" );
	all_nodes = config.getBoolean( "all_nodes_in_network" );
	fail_start_time = config.getInt( "fail_start_time" );
	fail_stop_time = (long) config.getInt( "fail_stop_time" );

	if( num_failures == -1 && tor == null ) {
	    printMsg( "No # of failures or timeout given, " +
		      "could run forever." );
	}

	if( fail_stop_time < 0 ) {
	    fail_stop_time = Long.MAX_VALUE;
	} 

	long random_seed;
	if( random_str == null ) {
	    random_seed = -1;
	} else {
	    random_seed = Long.parseLong( random_str );
	}

	Random starter_rng;
	if( random_seed == -1 ) {
	    starter_rng = new Random();
	} else {
	    starter_rng = new Random( random_seed + 
				      self_node_id.hashCode() );
	}
	long my_rng_starter = starter_rng.nextLong();
	printMsg( "the starter random number is " + my_rng_starter + 
		  ", random seed was " + random_seed );
	my_rng = new Random( my_rng_starter );

	if( tor == null ) {
	    time_to_run = -1;
	} else {
	    time_to_run = Long.parseLong( tor );
	}

	if( rt == null ) {
	    repair_timeout = -1;
	} else {
	    repair_timeout = Long.parseLong( rt );
	}

	if( time_to_run == -1 ) {
	    if( num_failures == -1 ) {
		Carp.die( "At least one of num_failures or " + 
			  "time_to_run must be specified" );
	    }
	} else {
	    timed = true;
	}

	actual_successes = 0;
	failures_dispatched = 0;
	messages = config.getStringList( "messages" );
	if( messages == null ) {
	    String tmp[] = {"ostore.tapestry.patchwork.PatchworkFaultMsg"};
	    messages = tmp;
	}

	if( cdf_string == null ) {
	    cdf_string = "ostore.tapestry.patchwork.SecondPDF";
	}
	if( life_cdf_string == null ) {
	    life_cdf_string = "ostore.tapestry.patchwork.AllOnesPDF";
	}
	if( cluster_cdf_string == null ) {
	    cluster_cdf_string = "ostore.tapestry.patchwork.AllZerosPDF";
	}
	if( cluster_life_cdf_string == null ) {
	    cluster_life_cdf_string = "ostore.tapestry.patchwork.AllZerosPDF";
	}

	try {
	    Object o = Class.forName( cdf_string ).newInstance();
	    if( !(o instanceof CDF) ) {
		Carp.die( "class " + cdf_string + " does not extend CDF" );
	    }
	    cdf = (CDF) o;

	    o = Class.forName( life_cdf_string ).newInstance();
	    if( !(o instanceof CDF) ) {
		Carp.die( "class " + life_cdf_string + 
			  " does not extend CDF" );
	    }
	    life_cdf = (CDF) o;

	    o = Class.forName( cdf_string ).newInstance();
	    if( !(o instanceof PDF) ) {
		Carp.die( "class " + cdf_string + 
			  " does not extend PDF" );
	    }
	    pdf = (PDF) o;

	    // checks here for randomized pdfs
	    if( pdf instanceof ProbPDF ) {
		randomized_fail_pdf = true;
		makeProbPDF( (ProbPDF) pdf, failrate, failinterval );
	    }

	    o = Class.forName( life_cdf_string ).newInstance();
	    if( !(o instanceof PDF) ) {
		Carp.die( "class " + life_cdf_string + 
			  " does not extend PDF" );
	    }
	    life_pdf = (PDF) o;

	    // checks here for randomized pdfs
	    if( life_pdf instanceof ProbPDF ) {
		randomized_life_pdf = true;
		makeProbPDF( (ProbPDF) life_pdf, liferate, lifeinterval );
	    }

	    // Clustering:

	    o = Class.forName( cluster_cdf_string ).newInstance();
	    if( !(o instanceof CDF) ) {
		Carp.die( "class " + cluster_cdf_string + 
			  " does not extend CDF" );
	    }
	    cluster_cdf = (CDF) o;

	    o = Class.forName( cluster_life_cdf_string ).newInstance();
	    if( !(o instanceof CDF) ) {
		Carp.die( "class " + cluster_life_cdf_string + 
			  " does not extend CDF" );
	    }
	    cluster_life_cdf = (CDF) o;

	    o = Class.forName( cluster_cdf_string ).newInstance();
	    if( !(o instanceof PDF) ) {
		Carp.die( "class " + cluster_cdf_string + 
			  " does not extend PDF" );
	    }
	    cluster_pdf = (PDF) o;

	    o = Class.forName( cluster_life_cdf_string ).newInstance();
	    if( !(o instanceof PDF) ) {
		Carp.die( "class " + cluster_life_cdf_string + 
			  " does not extend PDF" );
	    }
	    cluster_life_pdf = (PDF) o;


	} catch( ClassNotFoundException cnfe ) {
	    Carp.die( cnfe );
	}

	// deal with result log
        //  Configure the result log.
        _result_log_filename = config.getString("ResultLog");
	use_log = ( _result_log_filename == null ? false : true );
	if( use_log ) {
	    _result_log_fd = new AFile(_result_log_filename,
				       my_sink,  // completion queue
				       true,      // create if necessary
				       false);    // not read only
	}

	// if we want to be able to control all nodes in the network
	// (that is, bring brand new nodes up in the middle of the test)
	// we need to give them failed NodeInfos to start out with.
	// Don't worry, these will be overwritten for live nodes
	// that send in RMLocateMsgs.
	// TODO: How does this work in non-simulation?????
	node_infos = new TreeMap();
	if( all_nodes && sim_running ) {
	    int num_nodes = soss.network.Network.instance().num_nodes();
	    String nstring = config.getString( "node_id" );
	    String prefix = nstring.substring( 0, nstring.indexOf( ':' ) + 1 );
	    for( int i = 0; i < num_nodes; i++ ) {
		NodeId id = new NodeId( prefix + i );
		// guid value doesn't matter
		NodeInfo ninfo = new NodeInfo( id, new SHA1Hash() );
		ninfo.failed = true;
		ninfo.detached = true;
		node_infos.put( id, ninfo );
	    }
	}

	//////////////////
	// initialize filters

	classifier = Classifier.getClassifier( self_node_id );
	my_sink = config.getStage().getSink();
	Filter filter;

	// Subscribe to all my favorite events

	String[] eventNames = {
	    "seda.sandStorm.api.StagesInitializedSignal",
	    "ostore.tapestry.patchwork.RepairMonitorStage$RepairAlarmMsg",
	    "ostore.tapestry.impl.TapestryReadyMsg"
	};

	for( int i = 0; i < eventNames.length; i++ ) {
	  filter = new Filter();
	  Class eventClass;
          try {
            eventClass = Class.forName( eventNames[i] );
	    if( !filter.requireType( eventClass ) || 
	        !classifier.subscribe( filter, my_sink ) ) {

	      Carp.die( "Could not require class " + eventNames[i] ); 

	    } 

          } catch ( ClassNotFoundException cnfe ) {
            Carp.die( cnfe );
          }

        }

	String[] inb_eventNames = {
	    "ostore.tapestry.impl.VerifyRespMsg",
	    "ostore.tapestry.patchwork.RMLocateMsg",
	    "ostore.tapestry.patchwork.PatchworkRevCountMsg",
	    "ostore.tapestry.patchwork.RepairMonitorStage$PtrDeletedMsg",
	    "ostore.tapestry.patchwork.RepairMonitorStage$AttachMessage",
	    "ostore.tapestry.patchwork.RepairMonitorStage$DetachMessage"
	};


	// require inbounders
	for( int i = 0; i < inb_eventNames.length; i++ ) {
	  filter = new Filter();
	  Class eventClass;
          try {
            eventClass = Class.forName( inb_eventNames[i] );
	    if( !filter.requireType( eventClass ) ||
		!filter.requireValue ( "inbound", new Boolean(true) ) || 
	        !classifier.subscribe( filter, my_sink ) ) {

	      Carp.die( "Could not require class " + inb_eventNames[i] ); 

	    } 

          } catch ( ClassNotFoundException cnfe ) {
            Carp.die( cnfe );
          }

        }
	
    }

    private void makeProbPDF( ProbPDF pdf, double rate, int interval ) {

	pdf.setSeed( my_rng.nextLong() );
	if( rate != -1.0 ) {
	    pdf.setRate( rate );
	}
	if( interval != -1 ) {
	    pdf.setInterval( interval );
	}

    }

    /** 
     * Specified by seda.sandStorm.api.EventHandlerIF 
     */
    public void destroy()
    {
    }

    /**
     * An internal method used for dispatching events
     */
    private void dispatch( QueueElementIF event ) {

	//printMsg( "Dispatching event " + event );
                                                
      try {
        classifier.dispatch( event );
      } catch ( SinkFullException sfe ) {
        //TODO: Better way to deal with SinkFull
        try {
          Thread.currentThread().sleep( 5000 );
        } catch( InterruptedException ie ) {
          //ignore
        }
      }                                         
      
    }

    /**
     * An internal method used for dispatching events
     */
    private Object dispatch_later( QueueElementIF event, long millis ) {
                                                
        return classifier.dispatch_later( event, millis );

    }

    /** 
     * Print a RepairMonitor debugging message
     */
    private void printMsg( String msg ) {
	if( DEBUG ) {
	    Debug.printtagln( "RepairMonitor",  msg );
	}
    }

    private boolean areSameDomain( NodeInfo one, NodeInfo two ) {

	// the simulator knows for sure
	if( sim_running ) {

	    return soss.network.Network.instance().areSameDomain(one.node_id, 
								 two.node_id);
	    
	} else {

	    String host1 = one.node_id.address().getHostName();
	    String host2 = two.node_id.address().getHostName();

	    // Consider two hosts in the same domain iff the last 
	    // n-1 segments in the host names match
	    int start1 = host1.indexOf('.');
	    int start2 = host2.indexOf('.');
	    String domain1 = host1;
	    if( start1 != -1 ) {
		domain1 = host1.substring(start1+1);
	    }
	    String domain2 = host2;
	    if( start2 != -1 ) {
		domain2 = host2.substring(start2+1);
	    }
	    return domain1.equalsIgnoreCase( domain2 );
	}

    }

    ////////////////// INNER CLASSES /////////////////////////////////

    /**
     * Holds info about nodes, including guid, current life, etc.
     */
    private class NodeInfo implements Comparable {

	public NodeId node_id;
	public SecureHash guid;
	public Random rng;
	public int life;
	public boolean failed;
	public boolean detached;
	public Object timeout_token;
	public StatRecorder stats;
	public int ptrs_remaining;
	public long failure_time;

	public NodeInfo( NodeId nid, SecureHash guid ) {

	    node_id = nid;
	    this.guid = guid;
	    rng = new Random( my_rng.nextLong() + nid.hashCode() );
	    
	    // for now, set the initial life to 0
	    life = 0;
	    failed = false;
	    detached = false;
	    timeout_token = null;
	    stats = new StatRecorder( this + " stats" );

	}

	/**
	 * Set the life to an intelligently random value, based on 
	 * some function over the CDF
	 */
	public void setInitialLife() {
	    
	    // Iterate through the life of a node several times,
	    // in order to get a suitable random starting value.
	    int NUM_INTERVALS = 1000;
	    int JUMP_SIZE = 1;
	    if( cdf.getCDFSize() > NUM_INTERVALS ) {
		JUMP_SIZE = (int) (cdf.getCDFSize() / NUM_INTERVALS);
	    } 
	    for( ; NUM_INTERVALS > 0 ; NUM_INTERVALS-- ) {

		if( rng.nextDouble() < 
		    getFailThreshold( pdf, cdf, life ) ) {

		    // we failed, so restart at 0
		    life = 0;

		} else if( life + JUMP_SIZE < cdf.getCDFSize()) {
		    life += JUMP_SIZE;
		} else {
		    // if we can't jump any higher, assign the life the max 
		    // value
		    life = cdf.getCDFSize() - 1;
		}

	    }

	}

	public boolean equals( Object other ) {

	    if( other == null ) {
		return false;
	    }
	    NodeInfo nother = (NodeInfo) other;
	    return node_id.equals(nother.node_id) && guid.equals(nother.guid);

	}

	public int compareTo( Object other ) {
	    if( other == null ) {
		return 1;
	    }
	    if( equals( other ) ) {
		return 0;
	    } else {
		NodeInfo nother = (NodeInfo) other;
		int r = node_id.compareTo(nother.node_id);
		if( r == 0 ) {
		    r = guid.compareTo(nother.guid);
		}
		return r;
	    }

	}

	public String toString() {
	    return "(" + node_id + "/" + guid + ")";
	}

    }

    /**
     * Holds info about a cluster (domain) of nodes
     */
    private class ClusterInfo {

	public LinkedList node_infos;
	public Random rng;
	public int life;
	public boolean failed;

	public ClusterInfo() {

	    rng = new Random( my_rng.nextLong() );
	    node_infos = new LinkedList();
	    // for now, set the initial life to 0
	    life = 0;
	    failed = false;

	}

	/**
	 * Returns true iff this node belongs in this cluster,
	 * and it was added.  If the cluster is empty, always adds
	 * the node
	 */
	public boolean addNode( NodeInfo newNode ) {
	    // if someone's in this cluster already, make sure their
	    // domains match
	    if( node_infos.size() > 0 ) {
		if( !areSameDomain( newNode, (NodeInfo) node_infos.get(0) ) ) {
		    return false;
		}
	    }
	    if( !node_infos.contains( newNode ) ) {
		node_infos.add( newNode );
	    }
	    return true;
	}

	/**
	 * Set the life to an intelligently random value, based on 
	 * some function over the CDF
	 */
	public void setInitialLife() {
	    
	    // Iterate through the life of a node several times,
	    // in order to get a suitable random starting value.
	    int NUM_INTERVALS = 1000;
	    int JUMP_SIZE = 1;
	    if( cluster_cdf.getCDFSize() > NUM_INTERVALS ) {
		JUMP_SIZE = (int) (cluster_cdf.getCDFSize() / NUM_INTERVALS);
	    } 
	    for( ; NUM_INTERVALS > 0 ; NUM_INTERVALS-- ) {

		if( rng.nextDouble() < 
		    getFailThreshold( cluster_pdf, cluster_cdf, life ) ) {
		    
		    // we failed, so restart at 0
		    life = 0;

		} else if( life + JUMP_SIZE < cluster_cdf.getCDFSize()) {
		    life += JUMP_SIZE;
		} else {
		    // if we can't jump any higher, assign the life the max 
		    // value
		    life = cluster_cdf.getCDFSize() - 1;
		}

	    }

	}

	public String toString() {
	    String s = "{";
	    for( int i = 0; i < node_infos.size(); i++ ) {
		s += ((NodeInfo) node_infos.get(i)) + " "; 
	    }
	    s += "}";
	    return s;
	}

    }

    /**
     * A wakeup call to the RepairMonitor, telling it to perform a certain
     * action such as another failure generation round.
     */
    private class RepairAlarmMsg implements QueueElementIF {

	public static final int GENERATE_FAILURES = 0;
	public static final int REPAIR_TIMEOUT = 1;
	public static final int VERIFY_TAPESTRY = 2;
	public static final int VERIFY_TIMEOUT = 3;
	public static final int GENERATE_CORR_FAILURES = 4;
	public static final int PERIODIC_BW = 5;

	public int type;
	
	public NodeInfo failed_node;

	public RepairAlarmMsg( int type ) {
	    this.type = type;
	    failed_node = null;
	}

	public RepairAlarmMsg( NodeInfo failed_node ) {
	    type = REPAIR_TIMEOUT;
	    this.failed_node = failed_node;
	}

    }

    /**
     * A message to inform the RepairMonitor that a node has been 
     * detected as failed.
     */
    public static class PtrDeletedMsg extends NetworkMessage 
	implements QuickSerializable {

	public SecureHash badGuyGuid = null;
	public NodeId badGuyNode = null;
	public int hopCount;

	public PtrDeletedMsg (NodeId peer, SecureHash badGuy, int hopCount) {
	    
	    super (peer, false /* outbound */);
	    this.badGuyGuid = badGuy;
	    this.hopCount = hopCount;
	}

	public PtrDeletedMsg (NodeId peer, NodeId badGuy, int hopCount) {
	    
	    super (peer, false /* outbound */);
	    this.badGuyNode = badGuy;
	    this.hopCount = hopCount;
	}

	public PtrDeletedMsg (InputBuffer buffer) throws QSException {
	    super( buffer );
	    if( buffer.nextBoolean() ) {
		badGuyGuid = (SecureHash) buffer.nextObject();
	    } else {
		badGuyNode = (NodeId) buffer.nextObject();
	    }
	    this.hopCount = buffer.nextInt();
	}

	public void serialize (OutputBuffer buffer) {
	    super.serialize( buffer );
	    if( badGuyGuid != null ) {
		buffer.add( true );
		buffer.add( badGuyGuid );
	    } else {
		buffer.add( false );
		buffer.add( badGuyNode );
	    }
	    buffer.add( hopCount );
	}

	public Object clone () throws CloneNotSupportedException {
	    PtrDeletedMsg newone = (PtrDeletedMsg) super.clone ();
	    newone.badGuyGuid = badGuyGuid;
	    newone.badGuyNode = badGuyNode;
	    newone.hopCount = hopCount;
	    return newone;
	}
	
	public String toString () {
	    return "(PtrDeletedMsg, bad guy guid: " + badGuyGuid + 
		"bad guy node: " + badGuyNode + 
		" hops/level: " + hopCount + 
		super.toString () + ")";
	}
    }

    /**
     * A message indicating a command to detach, or the confirmation of a
     * successful detachment
     */
    public static class DetachMessage extends NetworkMessage 
	implements QuickSerializable {

	public boolean command = false;

	public DetachMessage (NodeId peer, boolean command) {
	    
	    super (peer, false /* outbound */);
	    this.command = command;

	}

	public DetachMessage (InputBuffer buffer) throws QSException {
	    super( buffer );
	    command = buffer.nextBoolean();
	}

	public void serialize (OutputBuffer buffer) {
	    super.serialize( buffer );
	    buffer.add( command );
	}

	public Object clone () throws CloneNotSupportedException {
	    DetachMessage newone = (DetachMessage) super.clone ();
	    newone.command = command;
	    return newone;
	}
	
	public String toString () {
	    return "(DetachMessage), (" + super.toString () + ")";
	}
    }

    /**
     * A message indicating a command to attach, or the confirmation of a
     * successful attachment
     */
    public static class AttachMessage extends NetworkMessage 
	implements QuickSerializable {

	public boolean command = false;
	public SecureHash myGuid = null;
	// whether or not to reattach with a new guid, rather than
	// reusing the old one
	public boolean new_guid = false;

	/**
	 * Attach confirmation
	 */
	public AttachMessage (NodeId peer, SecureHash myGuid) {
	    
	    super (peer, false /* outbound */);
	    this.myGuid = myGuid;

	}

	/**
	 * Tells a node to attach, and whether or not to use a new guid
	 */
	public AttachMessage (NodeId peer, boolean new_guid) {
	    
	    super (peer, false /* outbound */);
	    command = true;
	    this.new_guid = new_guid;
	}

	public AttachMessage (InputBuffer buffer) throws QSException {
	    super( buffer );
	    command = buffer.nextBoolean();
	    if( !command ) {
		myGuid = (SecureHash) buffer.nextObject();
	    } else {
		new_guid = buffer.nextBoolean();
	    }
	}

	public void serialize (OutputBuffer buffer) {
	    super.serialize( buffer );
	    buffer.add( command );
	    if( !command ) {
		buffer.add( myGuid );
	    } else {
		buffer.add( new_guid );
	    }
	}

	public Object clone () throws CloneNotSupportedException {
	    AttachMessage newone = (AttachMessage) super.clone ();
	    newone.command = command;
	    newone.new_guid = new_guid;
	    if( !command ) {
		try {
		    newone.myGuid = (SecureHash) myGuid.clone();
		} catch( Exception e ) {
		    throw new CloneNotSupportedException();
		}
	    }
	    return newone;
	}
	
	public String toString () {
	    return "(AttachMessage), (" + super.toString () + ")";
	}
    }

}
