/**
 * NonceMsg
 *
 * A class to be extended by messages that need to be fault tolerant,
 * such as messages critical to the robustness of dynamic integration.
 *
 * @author     Jeremy Stribling
 * @version $Id: NonceMsg.java,v 1.1 2003/03/03 19:34:50 strib Exp $ */

package ostore.util;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.QSException;

public abstract class NonceMsg 
extends NonceAckMsg implements QuickSerializable {

    private int my_nonce;

    public NonceMsg ( NodeId peer ) {
      super(peer);
    }

    public NonceMsg ( InputBuffer buffer ) throws QSException {
      super (buffer); 
      my_nonce = buffer.nextInt();
    }

    public void serialize ( OutputBuffer buffer ) {
      super.serialize (buffer);
      buffer.add( my_nonce );
    }

    public int getNonce() {
	return my_nonce;
    }

    public void setNonce( int nonce ) {
	my_nonce = nonce;
    }

    public String toString() {
	return "NonceMsg: " + getNonce();
    }
}

