/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * A quick serializable list.
 *
 * @author     Sean C. Rhea
 * @version    $Id: QSList.java,v 1.1 2003/03/23 20:55:25 srhea Exp $
 */
public class QSList implements QuickSerializable {

    public LinkedList list;

    public QSList (QSList l) {
	list = new LinkedList ();
	Iterator i = l.list.iterator ();
	while (i.hasNext ())
	    list.addLast (i.next ());
    }

    public QSList (LinkedList l) {
	list = l;
    }

    public QSList (InputBuffer buffer) throws QSException {
	list = new LinkedList ();
	int size = buffer.nextInt ();
	while (size-- > 0) 
	    list.addLast (buffer.nextObject ());
    }

    public void serialize (OutputBuffer buffer) {
	buffer.add (list.size ());
	Iterator i = list.iterator ();
	while (i.hasNext ()) {
	    QuickSerializable q = (QuickSerializable) i.next ();
	    buffer.add (q);
	}
    }

    public int hashCode () {
	int result = 0;
	Iterator i = list.iterator ();
	while (i.hasNext ()) {
	    Object x = i.next ();
	    result ^= x.hashCode ();
	}
	return result;
    }

    public boolean equals (Object other) {
	if (! (other instanceof QSList)) 
	    return false;
	QSList rhs = (QSList) other;
	if (list.size () != rhs.list.size ())
	    return false;
	Iterator i = list.iterator ();
	Iterator j = rhs.list.iterator ();
	while (i.hasNext ()) {
	    Object x = i.next ();
	    Object y = j.next ();
	    if (! x.equals (y))
		return false;
	}
	return true;
    }

    public String toString()
    {
	String result = "(QSList";
	Iterator i = list.iterator ();
	while (i.hasNext ()) {
	    Object x = i.next ();
	    result += " " + x;
	}
	return result + ")";
    }
}
