/**
 * QuickSerialzable.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package ostore.util;

/**
 * The serialization interface of OceanStore.  To inherit from this
 * interface, in addition to defining a <code>serialize</code> function, a
 * class should also define a deserialization constructor of the following
 * form
 *
 <pre>
     Constructor (InputBuffer buffer) throws QSException {
         ...
     }
 </pre>
 *
 * which is an inverse of the serialization function (loosely speaking).
 * The throws clause is optional.  
 *
 * @author	Sean C. Rhea
 * @version	$Id: QuickSerializable.java,v 1.10 2002/06/16 20:25:03 srhea Exp $
 */
public interface QuickSerializable {

    /**
     * Add the object to the <code>buffer</code>.
     *
     * @param buffer the output buffer to add the object to
     */
    void serialize (OutputBuffer buffer);
}

