/**
 * ostore_util_sha1_c.c
 *
 * @author   Hakim Weatherspoon
 * @version  $Id: ostore_util_sha1_c.h,v 1.2 2002/09/05 08:28:10 hweather Exp $
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 **/

/**
 * <CODE>ostore_util_sha1_c.h</CODE> is a <CODE>C</CODE>
 * header file that mimics <CODE>SHA1NativeIF.java</CODE>.
 *
 * The result of <CODE>ostore_util_sha1_c</CODE> is used by the 
 * {@link ostore.util.SHA1Hash} and {@link ostore.util.SHA1HashBuffer} 
 * class and by all entities which are SHA1 hashes.
 * (In the prototype, this means Guids and VHashes).
 *
 * @author   Hakim Weatherspoon
 * @version  $Id: ostore_util_sha1_c.h,v 1.2 2002/09/05 08:28:10 hweather Exp $
 **/
#ifndef _Included_ostore_util_sha1_h
#define _Included_ostore_util_sha1_h

#define SHA1_LENGTH 20

typedef unsigned long long ulong64;

struct sha1_state{
  ulong64 count;
  unsigned int W[80];
  unsigned int AA;
  unsigned int BB;  
  unsigned int CC;
  unsigned int DD;
  unsigned int EE;
};

/**
 * A nibble->char mapping for printing out bytes.
 */
unsigned char digits[] = {
  '0' , '1' , '2' , '3' , '4' , '5' , '6' , '7' , '8' , '9' ,
  'a' , 'b' , 'c' , 'd' , 'e' , 'f'
};

#define LOAD32H(x, y)                            \
     { x = ((unsigned long)(y)[0]<<24) | \
           ((unsigned long)(y)[1]<<16) | \
           ((unsigned long)(y)[2]<<8)  | \
           ((unsigned long)(y)[3]); }

#define STORE32H(x, y)                                                                     \
     { (y)[0] = (unsigned char)((x)>>24); \
       (y)[1] = (unsigned char)((x)>>16); \
       (y)[2] = (unsigned char)((x)>>8);  \
       (y)[3] = (unsigned char)(x); }

#define STORE64H(x, y)                                                                     \
   { (y)[0] = (unsigned char)((x)>>56); (y)[1] = (unsigned char)((x)>>48);     \
     (y)[2] = (unsigned char)((x)>>40); (y)[3] = (unsigned char)((x)>>32);     \
     (y)[4] = (unsigned char)((x)>>24); (y)[5] = (unsigned char)((x)>>16);     \
     (y)[6] = (unsigned char)((x)>>8); (y)[7] = (unsigned char)(x); }

#define ROL(x, y) (((x)<<((y)&31)) | ((x)>>(32-((y)&31))))

#define F0(x,y,z)  ( (x&y) | ((~x)&z) )
#define F1(x,y,z)  (x ^ y ^ z)
#define F2(x,y,z)  ((x & y) | (z & (x | y)))
#define F3(x,y,z)  (x ^ y ^ z)

void sha1_init(struct sha1_state* md);

/**
 * Update a byte.
 *
 * @param b	the byte
*/
void update1(int b, struct sha1_state* md);
void update2(const unsigned char* b, int off, int len, struct sha1_state* md);
int digest(unsigned char* hashvalue, int offset, int length, struct sha1_state* md);

void zeromem(void *dst, unsigned long len);

/**
 * Produce a <code>String</code> representation for the specified array of
 * <code>byte</code>s.  Print each <code>byte</code> as two hexadecimal
 * digits.
 *
 * @param data	The array to print
 * @param offset 	the start offset in <code>data</code>
 * @param length	the number of <code>byte</code>s to print
 */
void print_bytes (jbyte* data, int offset, int length);

#endif
