#!/usr/bin/perl -w
#
# author: Dennis Geels
# $Id: extract_stress.pl,v 1.2 2003/01/04 01:12:58 geels Exp $
#
# Collects link stress numberes from several logs.

use strict;

$| = 1;

sub usage {
    warn( "USAGE: extract_stress.pl <log1> ... <logN> \n" );
}

##################################################
# Variables

my $from_parent_sum = 0;
my $from_parent_stress_sum = 0;
my $to_child_sum = 0;

my $from_child_sum = 0;
my $from_child_stress_sum = 0;
my $to_parent_sum = 0;

my $bucket_width = 20;
my @from_parent_buckets;

##################################################
# Script

# Process arguments.
if( ! @ARGV ) {
    &usage() and exit(1);
}

my $file;
foreach $file (@ARGV) {
    open( LOG, "<$file" ) or die ("Can't open $file\n" );
    while( <LOG> ) {
	next unless /LinkStats/;

	if( /FROM PARENTS.*bytes: (\d+).*product: (\d+)/ ) {
	    $from_parent_sum += $1;
	    $from_parent_stress_sum += $2;
	} elsif( /FROM CHILDREN.*bytes: (\d+).*product: (\d+)/ ) {
	    $from_child_sum += $1;
	    $from_child_stress_sum += $2;
	} elsif( /TO CHILDREN.*bytes: (\d+)/ ) {
	    $to_child_sum += $1;
	} elsif( /TO PARENTS.*bytes: (\d+)/ ) {
	    $to_parent_sum += $1;
	} elsif( /FROM PARENTS.*distance: (.*)/ ) {
#	    print;
	    my @entries = split( /\s|:/, $1 );
#	    print "entries: @entries\n";
	    for( my $i=0;  $i < @entries; $i+=2 ) {
		my $distance = $entries[$i];
		my $bytes = $entries[$i+1];
		$from_parent_buckets[($distance/$bucket_width)] += $bytes;
	    }
	}
    }
    close( LOG );
}


print "DOWNSTREAM: $from_parent_sum/$to_child_sum, $from_parent_stress_sum\n";
print "UPSTREAM: $from_child_sum/$to_parent_sum, $from_child_stress_sum\n";
print "HISTOGRAM: \n";
for( my $i=0; $i < @from_parent_buckets; $i++ ) {
    print "\t", $i*$bucket_width, "-", ($i+1)*$bucket_width, " ms: ";
    if( $from_parent_buckets[$i] ) {
	printf "%f\n", ($from_parent_buckets[$i]/$from_parent_sum);
    } else {
	print "0\n";
    }
}
