/*
 * Copyright (c) 1997 by the University of Southern California
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation in source and binary forms for non-commercial purposes
 * and without fee is hereby granted, provided that the above copyright
 * notice appear in all copies and that both the copyright notice and
 * this permission notice appear in supporting documentation. and that
 * any documentation, advertising materials, and other materials related
 * to such distribution and use acknowledge that the software was
 * developed by the University of Southern California, Information
 * Sciences Institute.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about
 * the suitability of this software for any purpose.  THIS SOFTWARE IS
 * PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Other copyrights might apply to parts of this software and are so
 * noted when applicable.
 *
 * sgb2ns.c -- converter from sgb format to ns-2 configuration. 
 * $Id: sgb2hier.c,v 1.2 2002/11/04 22:45:18 srhea Exp $
 *
 * Daniel Zappala (daniel@isi.edu) wrote for converting GT-ITM's output to
 *   ns-1.
 * Polly Huang (phuang@catarina.usc.edu) and Haobo Yu (haoboy@isi.edu) 
 *   ported to ns-2.
 *
 * Maintainer: Haobo Yu (haoboy@isi.edu)
 *
 */

#include <stdio.h>
#include <strings.h>
#include <assert.h>
#include "gb_graph.h"
#include "gb_save.h"
#include "geog.h"

static void parse_node_name (char *name, int *dom, int *dnn, 
	int *snum, int *snn) {

    int i = 2, j = 0;
    char buf[200];
    while ((name [i] >= '0') && (name [i] <= '9')) {
	buf [j] = name [i];
	++i; ++j;
    }
    buf [j] = '\0';

    *dom = atoi (buf);

    assert (name [i] == '.');
    ++i;

    j = 0;
    while ((name [i] >= '0') && (name [i] <= '9')) {
	buf [j] = name [i];
	++i; ++j;
    }
    buf [j] = '\0';

    *dnn = atoi (buf);

    if (name [0] == 'S') {
	assert (name [i] == '/');
	++i;

	j = 0;
	while ((name [i] >= '0') && (name [i] <= '9')) {
	    buf [j] = name [i];
	    ++i; ++j;
	}
	buf [j] = '\0';

	*snum = atoi (buf);

	assert (name [i] == '.');
	++i;

	j = 0;
	while ((name [i] >= '0') && (name [i] <= '9')) {
	    buf [j] = name [i];
	    ++i; ++j;
	}
	buf [j] = '\0';

	*snn = atoi (buf);
    }
    else {
	*snum = -1;
	*snn = -1;
    }
}

main(argc,argv)
     int argc;
     char *argv[];
{
	int ch;
	int i, j, nlink;
	Vertex *v;
	Arc *a;
	Graph *g;
	FILE *fopen(), *fout;
	char m[420];

	if (argc != 3) {
		printf("sgb2ns <sgfile> <nsfile>\n\n");
		return;
	}
	fout = fopen(argv[2],"w");

	g = restore_graph(argv[1]);
	if (g == NULL) {
		printf("%s does not contain a correct SGB graph\n",argv[1]);
		return;
	}

	fprintf(fout, "%d\n", g->n);
	for (v = g->vertices,i=0; i < g->n; i++,v++) {
	    int dom, dnn, snum, snn;
	    parse_node_name (v->name, &dom, &dnn, &snum, &snn);
	    fprintf (fout, "%d %d %d %d %d\n", i, dom, dnn, snum, snn);
	}
			
	for (v = g->vertices,i=0; i < g->n; i++,v++) {

		int src_dom, src_dnn, src_snum, src_snn;

		for (a = v->arcs; a != NULL; a = a->next) {
			j = a->tip - g->vertices;
			if (j > i) {
				fprintf(fout, "%d %d %d\n", i, j, a->len);
				nlink++;
			}
		}
	}
}	

