/**
 * StandardDialog.java
 *
 * Copyright (c) 2002 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package Sonar.Dialog;

import javax.swing.*;
import java.awt.*;

/**
 * This is a dialog class to which only one Active Panel can be added.  
 * It also has two buttons: "Apply" and "Cancel". Applying will call
 * applyChanges() on the ActivePanel and then close the dialog. Canceling
 * will just close the dialog. 
 *
 * To use this dialog, create an activePanel, create a StandardDialog
 * call MyDialog.AddActivePanel(myActivePanel); and then call MyDialog.show();
 *
 * The end title of the activePanel becomes the title of the dialog
 * the modal variable makes the dialog modal or not
 *
 * @author	Kamin Whitehouse <kamin@cs.berkeley.edu>
 */
public class StandardDialog extends javax.swing.JDialog
{
	
    ActivePanel activePanel;

    //----------------------------------------------------------------------
    //CONSTRUCTORS
    public StandardDialog(Frame parent)
    {
	super(parent);
	// This code is automatically generated by Visual Cafe when you add
	// components to the visual environment. It instantiates and initializes
	// the components. To modify the code, only use code syntax that matches
	// what Visual Cafe can generate, or Visual Cafe may be unable to back
	// parse your Java file into its visual environment.
	//{{INIT_CONTROLS
	
	setDefaultCloseOperation(javax.swing.JFrame.DISPOSE_ON_CLOSE);
	setModal(true);	
	setTitle("DialogTitle");	
	getContentPane().setLayout(null);	
	setSize(405,305);	
	setVisible(false);	
	ApplyButton.setNextFocusableComponent(CancelButton);	
	ApplyButton.setText("Apply");	
	ApplyButton.setActionCommand("Apply");	
	getContentPane().add(ApplyButton);	
	ApplyButton.setBounds(84,264,105,30);	
	CancelButton.setText("Close");	
	CancelButton.setActionCommand("Close");	
	getContentPane().add(CancelButton);	
	CancelButton.setBounds(240,264,102,30);	
	//}}
	
	//{{REGISTER_LISTENERS
	SymAction lSymAction = new SymAction();
	ApplyButton.addActionListener(lSymAction);
	CancelButton.addActionListener(lSymAction);
	//}}
    }

	
    public StandardDialog()
    {
	this((Frame)null);
    }

    public StandardDialog(String sTitle)
    {
	this();	
	setTitle(sTitle);	
    }
	
    public StandardDialog(ActivePanel pPanel)
    {
	this();
	addActivePanel(pPanel);	
    }
	         
    //CONSTRUCTORS
    //----------------------------------------------------------------------
    public void setVisible(boolean b)
    {
	if (b)
	    setLocation(50, 50);
	super.setVisible(b);	
    }

    //--------------------------------------------------------------------
    //ADD ACTIVE PANEL
    //this function adds an active panel to the dialog
    public void addActivePanel(ActivePanel pPanel)
    {
	if( (pPanel == null) ||
		(pPanel.getCancelInfoDialog() == true))	
	{
	    this.dispose();
	}
	activePanel = pPanel;
	setModal(activePanel.getModal());
	setTitle(activePanel.getTabTitle());
	activePanel.initializeDisplayValues();
	Dimension d = activePanel.getSize();
	getContentPane().add(activePanel);
	activePanel.setBounds(0,0,d.width, d.height);
    }
    //ADD ACTIVE PANEL
    //--------------------------------------------------------------------

    //--------------------------------------------------------------------
    //APPLY CHANGES
    //this function is called when the "OK" button is pushed
    public void applyChanges()
    {
	activePanel.applyChanges();
    }
    //APPLY CHANGES
    //--------------------------------------------------------------------

    public void addNotify()
    {
	// Record the size of the window prior to calling parents addNotify.
	Dimension size = getSize();
	super.addNotify();
	if (frameSizeAdjusted)
	    return;
	frameSizeAdjusted = true;
	// Adjust size of frame according to the insets
	Insets insets = getInsets();
	setSize(insets.left + insets.right + size.width, insets.top + insets.bottom + size.height);
    }

    // Used by addNotify
    boolean frameSizeAdjusted = false;
    //{{DECLARE_CONTROLS
    javax.swing.JButton ApplyButton = new javax.swing.JButton();
    javax.swing.JButton CancelButton = new javax.swing.JButton();
    //}}

    class SymAction implements java.awt.event.ActionListener
    {
	public void actionPerformed(java.awt.event.ActionEvent event)
	{
	    Object object = event.getSource();
	    if (object == ApplyButton)
		ApplyButton_actionPerformed(event);
	    else if (object == CancelButton)
		CancelButton_actionPerformed(event);
	}
    }

    void ApplyButton_actionPerformed(java.awt.event.ActionEvent event)
    {         //this is the code that is called when the "OK" button is clicked
	// to do: code goes here.
	ApplyButton_actionPerformed_Interaction1(event);
    }

    void ApplyButton_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
    {         //this is the code that is called when the "OK" button is clicked
	try {
	    this.applyChanges();
//	
//	this.dispose();		don't close this so we can keep working
	} catch (java.lang.Exception e) {
	}
    }
	
    void CancelButton_actionPerformed(java.awt.event.ActionEvent event)
    {         //this is the code that is called when the "OK" button is clicked
	// to do: code goes here.
	CancelButton_actionPerformed_Interaction1(event);
    }

    void CancelButton_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
    {
	try {
	    this.dispose();
	} catch (java.lang.Exception e) {
	}
    }
}
