/**
 * PlanetLabAnalyzer.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
package Sonar.PacketAnalyzers; 

import Sonar.util.NodeInfo;

import ostore.util.NodeId;
import ostore.util.Debug;

/**
 * @author	Jason Lee <jlee81@cs.berkeley.edu>
 */

public class PlanetLabAnalyzer extends PacketAnalyzer
{ 
    private final String class_tag = "PlanetLabAnalyzer";
    private final boolean DEBUG = false;
    protected int default_berkeley_nodes = 0;
    protected int default_nodes = 0;
    
    public PlanetLabAnalyzer() 
    { 
      int default_berkeley_nodes = 0;
      int default_nodes = 0;
	//String test = "128.1.10.13";
	//setNodePosition(test);
    } 

    public void setNodePosition(NodeInfo node) {

	final String tag = class_tag + ".setNodePosition";
	String name = node.getNodeId().address().getHostAddress();

	String temp;
	int i = name.indexOf('.');
	int [] ip_addr = new int[4];
	ip_addr[0] = (new Integer(name.substring(0,i))).intValue();

	temp = name.substring(i+1, name.length());
	i = temp.indexOf('.');
	ip_addr[1] = (new Integer(temp.substring(0, i))).intValue();

	temp = temp.substring(i+1, temp.length());
	i = temp.indexOf('.');
	ip_addr[2] = (new Integer(temp.substring(0, i))).intValue();

	temp = temp.substring(i+1, temp.length());
	//i = temp.indexOf('.');
	//ip_addr[3] = Integer(temp.substring(0, i-1)).intValue();
	ip_addr[3] = (new Integer(temp)).intValue();

	boolean node_fixed = true; //TODO: actually useless - gonna strip out later...

	/*
	if(node.getNodeId().address().getHostName().toLowerCase().endsWith("berkeley.edu")) {
	    //if(DEBUG) Debug.printtagln(tag, "unknown berkeley node " + node.getNodeId().toString());
	    node.setX(111 + default_berkeley_nodes*2);
	    default_berkeley_nodes++;
	    node.setY(137);
	    
	} 
	*/
	switch(ip_addr[0]) {
	    case 12:
		switch(ip_addr[1]) {
		    case 155: //intel lab berkeley
			switch(ip_addr[3]) {
			    case 147:
				node.setX(111);
				node.setY(143);
				break;
			    case 148:
				node.setX(113);
				node.setY(143);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 161: //pittsburgh 
			switch(ip_addr[3]) {
			    case 195:
				node.setX(199);
				node.setY(140);
				break;
			    case 196:
				node.setX(197);
				node.setY(140);
				break;
			    case 197:
				node.setX(195);
				node.setY(140);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 17:  //seattle
			switch(ip_addr[3]) {
			    case 136:
				node.setX(111);
				node.setY(120);
				break;
			    case 137:
				node.setX(113);
				node.setY(120);
				break;
			    case 138:
				node.setX(115);
				node.setY(120);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 18: //mit.edu - 3
		switch(ip_addr[3]) {
		    case 190:
			node.setX(210);
			node.setY(132);
			break;
		    case 191:
			node.setX(208);
			node.setY(132);
			break;
		    case 192:
			node.setX(206);
			node.setY(132);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 128:
		switch (ip_addr[1]) {
		    case 232: //cambridge uk - 3
			switch(ip_addr[3]) {
			    case 201:
				node.setX(347);
				node.setY(106);
				break;
			    case 202:
				node.setX(349);
				node.setY(106);
				break;
			    case 203:
				node.setX(351);
				node.setY(106);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 2:   //cmu - 3
			switch(ip_addr[3]) {
			    case 199:
				node.setX(199);
				node.setY(138);
				break;
			    case 188:
				node.setX(197);
				node.setY(138);
				break;
			    case 196:
				node.setX(195);
				node.setY(138);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 59:  //columbia - 3
			switch(ip_addr[3]) {
			    case 200:
				node.setX(203);
				node.setY(143);
				break;
			    case 201:
				node.setX(201);
				node.setY(143);
				break;
			    case 202:
				node.setX(199);
				node.setY(143);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 84:  //cornell - 2
			switch(ip_addr[3]) {
			    case 49:
				node.setX(203);
				node.setY(135);
				break;
			    case 71: 
				node.setX(205);
				node.setY(135);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 112: //planetlab princeton - 2
			switch(ip_addr[3]) {
			    case 122:
				node.setX(203);
				node.setY(145);
				break;
			    case 123:
				node.setX(201);
				node.setY(145);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 42:  //rice - 3
			switch(ip_addr[3]) {
			    case 143:
				node.setX(162);
				node.setY(163);
				break;
			    case 144:
				node.setX(164);
				node.setY(163);
				break;
			    case 145:
				node.setX(166);
				node.setY(163);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 111: //ucsb - 2
			switch(ip_addr[3]) {
			    case 61:
				node.setX(114);
				node.setY(155);
				break;
			    case 62:
				node.setX(116);
				node.setY(155);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 119: //umass - 2
			switch(ip_addr[3]) {
			    case 210:
				node.setX(208);
				node.setY(130);
				break;
			    case 211:
				node.setX(210);
				node.setY(130);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 95:  //washington
			switch(ip_addr[3]) {
			    case 192:
				node.setX(111);
				node.setY(122);
				break;
			    case 193:
				node.setX(113);
				node.setY(122);
				break;
			    case 194:
				node.setX(115);
				node.setY(122);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 83:  //utexas
			switch(ip_addr[3]) {
			    case 152:
				node.setX(148);
				node.setY(155);
				break;
			    case 153:
				node.setX(150);
				node.setY(155);
				break;
			    case 154:
				node.setX(152);
				node.setY(155);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 252: //st.louis
			switch(ip_addr[3]) {
			    case 20:
				node.setX(166);
				node.setY(146);
				break;
			    case 21:
				node.setX(168);
				node.setY(146);
				break;
			    case 22:
				node.setX(164);
				node.setY(146);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 129: //kupl1.ittc.ku.edu -2
		switch(ip_addr[3]) { //kansas
		    case 250:
			node.setX(158);
			node.setY(146);
			break;
		    case 251:
			node.setX(156);
			node.setY(146);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 130: //unibo.it - 2 //bologna
		switch(ip_addr[3]) {
		    case 21:
			node.setX(374);
			node.setY(132);
			break;
		    case 22:
			node.setX(376);
			node.setY(132);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 131:
		switch (ip_addr[1]) {
		    case 215: //caltech - 3
			switch(ip_addr[3]) {
			    case 71:
				node.setX(122);
				node.setY(156);
				break;
			    case 72:
				node.setX(124);
				node.setY(156);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 243: //lbl - 2
			switch(ip_addr[3]) {
			    case 35:
				node.setX(113);
				node.setY(139);
				break;
			    case 36:
				node.setX(111);
				node.setY(139);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 179: //ucla - 2
			switch(ip_addr[3]) {
			    case 70:
				node.setX(120);
				node.setY(154);
				break;
			    case 71:
				node.setX(122);
				node.setY(154);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 132:
		switch (ip_addr[1]) {
		    case 239: //ucsd
			switch(ip_addr[3]) {
			    case 224:
				node.setX(120);
				node.setY(159);
				break;
			    case 225:
				node.setX(122);
				node.setY(159);
				break;
			    case 226:
				node.setX(124);
				node.setY(159);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 181: //New Zealand
			switch(ip_addr[3]) {
			    case 1:
				node.setX(666);
				node.setY(252);
				break;
			    case 2:
				node.setX(668);
				node.setY(252);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    default:
			node_fixed = false;
		} 
		break;
	    case 140: //harvard.edu
		switch(ip_addr[3]) {
		    case 123:
			node.setX(208);
			node.setY(134);
			break;
		    case 126:
			node.setX(210);
			node.setY(134);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 141: //umich.edu
		switch(ip_addr[3]) {
		    case 201:
			node.setX(184);
			node.setY(131);
			break;
		    case 202:
			node.setX(186);
			node.setY(131);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 142: //ubc.ca
		switch(ip_addr[3]) {
		    case 1:
			node.setX(110);
			node.setY(114);
			break;
		    case 2:
			node.setX(112);
			node.setY(114);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 150: //arizona
		switch(ip_addr[3]) {
		    case 2:
			node.setX(130);
			node.setY(152);
			break;
		    case 3:
			node.setX(132);
			node.setY(152);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 152: //duke - 3
		switch(ip_addr[3]) {
		    case 1:
			node.setX(200);
			node.setY(152);
			break;
		    case 2:
			node.setX(202);
			node.setY(152);
			break;
		    case 3:
			node.setX(198);
			node.setY(152);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 155: //utah - 3
		switch(ip_addr[3]) {
		    case 2:
			node.setX(125);
			node.setY(139);
			break;
		    case 3:
			node.setX(123);
			node.setY(139);
			break;
		    case 4:
			node.setX(127);
			node.setY(139);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 158: //upenn - 2
		switch(ip_addr[3]) {
		    case 254:
			node.setX(199);
			node.setY(142);
			break;
		    case 253:
			node.setX(197);
			node.setY(142);
			break;
		    case 22:
			node.setX(295);
			node.setY(142);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 160: //utk.edu - 2 , tennessee
		switch(ip_addr[3]) {
		    case 172:
			node.setX(181);
			node.setY(151);
			break;
		    case 173:
			node.setX(183);
			node.setY(151);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 169: //millennium.berkeley.edu - 3
		switch(ip_addr[3]) {
		    case 250:
			node.setX(111);
			node.setY(141);
			break;
		    case 251:
			node.setX(113);
			node.setY(141);
			break;
		    case 252:
			node.setX(115);
			node.setY(141);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 171: //stanford - 2
		switch(ip_addr[3]) {
		    case 216:
			node.setX(113);
			node.setY(145);
			break;
		    case 217:
			node.setX(111);
			node.setY(145);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 192: //diku.dk -2
		switch(ip_addr[3]) {
		    case 143:
			node.setX(368);
			node.setY(93);
			break;
		    case 144:
			node.setX(370);
			node.setY(93);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 193: //uu.se - 2
		switch(ip_addr[3]) {
		    case 128:
			node.setX(380);
			node.setY(78);
			break;
		    case 129:
			node.setX(382);
			node.setY(78);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 194: //lancs.ac.uk
		switch(ip_addr[3]) {
		    case 242:
			node.setX(346);
			node.setY(99);
			break;
		    case 243:
			node.setX(348);
			node.setY(99);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 198: //wisc
		switch(ip_addr[3]) {
		    case 145:
			node.setX(170);
			node.setY(130);
			break;
		    case 146:
			node.setX(172);
			node.setY(130);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 199: //atl,ga
		switch(ip_addr[3]) {
		    case 193:
			node.setX(190);
			node.setY(156);
			break;
		    case 194:
			node.setX(192);
			node.setY(156);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 206:
		switch ( ip_addr[1] ) {
		    case 117: //postel.org - LA?
			switch(ip_addr[3]) {
			    case 4:
				node.setX(170);
				node.setY(163);
				break;
			    case 5:
				node.setX(172);
				node.setY(163);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    case 240: //uky.edu
			switch(ip_addr[3]) {
			    case 20:
				node.setX(186);
				node.setY(153);
				break;
			    case 21:
				node.setX(188);
				node.setY(153);
				break;
			    default:
				node_fixed = false;
			}
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 208: //nbgisp.com
		switch(ip_addr[3]) { //oregon
		    case 98:
			node.setX(110);
			node.setY(130);
			break;
		    case 99:
			node.setX(112);
			node.setY(130);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    case 216: //nyu
		switch(ip_addr[3]) {
		    case 81:
			node.setX(203);
			node.setY(141);
			break;
		    case 82:
			node.setX(201);
			node.setY(141);
			break;
		    default:
			node_fixed = false;
		}
		break;
	    default:
		if(DEBUG) Debug.printtagln(tag, "unknown ip address:"+ name );
		//do nothing! keep the positions generated by GridLocationAnalyzer
		node_fixed = false;
	}
	
	//if(node.getX() < 2 || node_fixed == false) {
	if(node_fixed == false && node.getX() < 1) {
	    if(node.getNodeId().address().getHostName().toLowerCase().endsWith("berkeley.edu")) {
		//if(DEBUG) Debug.printtagln(tag, "unknown berkeley node " + node.getNodeId().toString());
		if(node.getAlreadySet()) {
		    node.setX(111 + node.getPosition());
		} else {
		    node.setX(111 + default_berkeley_nodes*2);
		    node.setPositioned(default_berkeley_nodes * 2);
		    default_berkeley_nodes++;
		}
		node.setY(137);
	    } else {
		//if(DEBUG) Debug.printtagln(tag, "unknown node (def to tahoe) " + node.getNodeId().toString());
		if(node.getAlreadySet()) {
		    node.setX(111 + node.getPosition());
		} else {
		    node.setX(117 + default_nodes*2);
		    node.setPositioned(default_nodes * 2);
		    default_nodes++;
		}
		node.setY(140);
	    }
	
	    if(DEBUG) Debug.printtagln("PLA: " , default_berkeley_nodes +"/"+ default_nodes + ">"+ node.getX() +", "+ node.getY());

	    node_fixed = true;
	}
    }

}


