/**
 * NodeEvent.java
 *
 * Copyright (c) 2002 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package Sonar.event; 
 
import Sonar.*; 
import Sonar.util.NodeInfo;
import java.util.*;
import ostore.util.NodeId;

import seda.sandStorm.api.QueueElementIF;

/**
 * This event is triggered every time a node is created or deleted.
 *
 * @author Kamin Whitehouse <kamin@cs.berkeley.edu>
 */
public class NodeEvent extends java.util.EventObject implements QueueElementIF
{ 
  protected NodeId _node_id;
  protected NodeInfo _node_info;
  //these events carry a reference to the node that was clicked 
  protected Date time;//and the time the event was generated 

  protected int event = 0;
  public final static int CREATE = 0;
  public final static int DELETE = 1;
  public final static int ID = 2;

  protected int id_type = 0;

  public int getEventType() { return event; }
  public int getIdType() { return id_type; }
	 
  //*****---CONSTRUCTOR---******// 
  /*
  public NodeEvent(Object source, NodeId pNode, Date pTime) 
  { 
    super(source); 
    _node_id = pNode; 
    _node_info = null;
    time = pTime; 
  }
  */

  public NodeEvent(Object source, NodeInfo pNodeInfo, Date pTime) {
      super(source);
      _node_id = pNodeInfo.getNodeId();
      _node_info = pNodeInfo;
      time = pTime;

  }

  public NodeEvent(Object source, NodeInfo pNodeInfo, Date pTime, int event_type) {
     this(source, pNodeInfo, pTime);
     event = event_type;
  }

  public NodeEvent(Object source, NodeId pNodeId, Date pTime, int event_type, int id_type) {
      super(source);
      _node_id = pNodeId;
      _node_info = null;
      time = pTime;
      event = event_type;
      this.id_type = id_type;
  }
  //*****---CONSTRUCTOR---******// 
  
  
  
  //*****---Get Functions---******// 
  public NodeId getNodeId()
  { 
    return _node_id;
  } 

  public NodeInfo getNodeInfo() { return _node_info; }
  
  public Date getTime() 
  { 
    return time; 
  } 
  //*****---Get Functions---******// 
}	
