/**
 * NodeInfo.java
 *
 * Copyright (c) 2002 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
package Sonar.util; 

import java.util.Calendar;
import java.util.Date;
import java.util.Vector;
import java.awt.Color;
import java.awt.Image;
import javax.swing.ImageIcon;

import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.tapestry.impl.RoutingTable;

public class NodeInfo { 
    protected static int arrowNum = 0;
    protected static final int maxArrowColors = 10;

    protected NodeId _node_id; 
    //display manager
    protected ImageIcon imageHelper; 
    protected Image image; 
    protected double imageWidth, imageHeight; 
    protected boolean displayThisNode; 
    protected boolean displayNodeId; 
    protected boolean fitOnScreen;

    protected int nodeType = Sonar.DisplayManager.NEVER_SET;
    protected int arrowValue = 0;
    protected Color arrowColor;

    //object maintainer
    protected Date timeCreated; 
    protected Date timeLastSeen; 
    protected boolean created;

    //routing table analyzer
    protected SecureHash _self_guid;
    protected Vector _neighbors;
    protected long _updateTime;
    protected boolean _deleted = false;
    protected boolean _notResponding = false;

    //grid location analyzer
    protected double x;
    protected double y;
    //determines if the node can move around automatically or not
    protected boolean fixed; 
    //determines if the XY coords should be drawn
    protected boolean displayCoords;
    protected double degree = 0.0; //of the given layer for circular view
    protected double num_on_partition = 0; //# of other nodes on the same layer
    protected int stratum = 0; //layer number

    protected NodeInfo group_leader = null;

    public boolean getAlreadySet() {
	return alreadyPositioned;
    }
    public void setPositioned(int p) {
	alreadyPositioned = true;
	pos = p;
    }
    public int getPosition() {
	return pos;
    }
    protected int pos;
    protected boolean alreadyPositioned;
    

    public NodeInfo(NodeId pNodeId) 
    { 
	alreadyPositioned = false;
	pos = 0;

	_node_id = pNodeId; 
	imageHelper = new ImageIcon("images/newimac.gif","images/newimac.gif"); 
	image = imageHelper.getImage(); 
	imageWidth = .2;//note that this width and height is in node coordinates (hence it scales automatically with the size of the network, but must be initialized properly) 
	imageHeight = .2; 
	displayThisNode = true; 
	displayNodeId = false; //MainClass.displayManager.showNodeIds; //false;
	fitOnScreen = true;

	arrowNum = ( arrowNum + 1) % maxArrowColors;
	arrowValue = arrowNum;
	//MainClass.out("DM> arrow num of new node:" +arrowValue);	
	
	x = Math.random();
	y = Math.random();
	fixed = false;
	displayCoords = false;
    } 

    public NodeInfo(NodeId node_id, Date time) {
	this(node_id);
	timeCreated = time;
	timeLastSeen = time;
    }

    public NodeInfo(NodeId node_id, SecureHash guid) {
	this(node_id);
	_self_guid = guid;
	_updateTime = Calendar.getInstance().getTime().getTime();
	_neighbors = new Vector();
    }

    public NodeInfo(RoutingTable tbl) {
	this(tbl.get_self_node_id());
	_self_guid = tbl.node_id_to_guid(_node_id);
	_updateTime = Calendar.getInstance().getTime().getTime();
	_neighbors = new Vector();
    }
    public  NodeId getNodeId(){return _node_id;} 

    //display manager
    public  ImageIcon getImageHelper(){ return imageHelper;} 
    public  Image getImage(){ return image;} 
    public  double getImageWidth(){ return imageWidth;} 
    public  double getImageHeight(){ return imageHeight;} 
    public  boolean getDisplayThisNode(){ return displayThisNode;} 
    public  boolean getDisplayNodeId(){ return displayNodeId;} 
    public  boolean getFitOnScreen(){ return fitOnScreen;} 

    public  void setImageHelper(ImageIcon pImageHelper){  imageHelper =pImageHelper;} 
    public  void setImage(Image pImage){  image =pImage;} 
    public  void setImageWidth(double w){  imageWidth = w;} 
    public  void setImageHeight(double h){  imageHeight = h;} 
    public  void setDisplayThisNode(boolean pDisplay){  displayThisNode=pDisplay;} 
    public  void setDisplayNodeId(boolean pDisplayNumber){  displayNodeId=pDisplayNumber;} 
    public  void setFitOnScreen(boolean pFit){  fitOnScreen=pFit;} 

    public void setNodeType(int value) { nodeType = value; }
    public int getNodeType() { return nodeType; }

    public int getArrowValue() { return arrowValue; }


    //object maintainer
    public Date getTimeCreated() { return timeCreated; } 
    public void setTimeCreated(Date time) { timeCreated = time; } 
    public Date getTimeLastSeen() { return timeLastSeen; } 
    public void setTimeLastSeen(Date time) { timeLastSeen = time; } 
    public boolean getCreated(){return created;} 
    public void setCreated(boolean pCreated){created = pCreated;} 

    //routing table
    public SecureHash get_self_guid() { return _self_guid; }
    public void set_self_guid(SecureHash new_value) { _self_guid = new_value; }

    public Vector get_neighbors() { return _neighbors; }

    
    public void updateTime (long time) { _updateTime = time; }
    public long getUpdateTime() { return _updateTime; }

    public void markDeleted (boolean state) {
	_deleted = state; 
	if(!state) _notResponding = false;
    }
    public void markNotResponding (boolean state) {
	_notResponding = state;
    }
    public boolean isDeleted() { return _deleted; }
    public boolean isNotResponding() { return _notResponding; }

    //grid location analyzer
    public double getX(){return x;} 
    public double getY(){return y;} 
    public boolean getFixed(){return fixed;} 
    public boolean getDisplayCoords(){return displayCoords;} 
 
    public void setX(double pX){x=pX;} 
    public void setY(double pY){y=pY;} 
    public void setFixed(boolean pFixed){fixed = pFixed;} 
    public void setDisplayCoords(boolean pDisplayCoords){displayCoords= pDisplayCoords;} 

    public double getPartitionNum() { return num_on_partition; }
    public void setPartitionNum (double value) { num_on_partition = value; }
    public void setStratum (int value) { stratum = value; }
    public void setDegree (double value) { degree = value; }
    public double getDegree() { return degree; }

    public void setGroupLeader(NodeInfo leader) { group_leader = leader; }
    public NodeInfo getGroupLeader() { return group_leader; }
} 

